package demo.combinecharts;

import java.awt.BorderLayout ;
import java.awt.Color ;
import java.util.Calendar ;

import javax.swing.JPanel ;

import com.ijchart.xychart.axis.value.IntervalXNumberYBaseValueAxis ;
import com.ijchart.xychart.axis.value.XYNumberValueAxis ;
import com.ijchart.xychart.client.ChartPanel ;
import com.ijchart.xychart.client.IJChart ;
import com.ijchart.xychart.constant.AxisLabelType ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.constant.IAngle ;
import com.ijchart.xychart.constant.XYValueAxisType ;
import com.ijchart.xychart.data.IntervalXNumberYBaseChartData ;
import com.ijchart.xychart.data.XYNumberChartData ;
import com.ijchart.xychart.dataset.DefaultMatrixDataset ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.plot.XYPlot ;
import com.ijchart.xychart.plot.combine.CombineValueAxisXYPlot ;
import com.ijchart.xychart.render.xy.IntervalXNumberYBaseBar2DRender ;
import com.ijchart.xychart.render.xy.XYLineAndShapeRender ;
import com.ijchart.xychart.title.LegendTitle ;
import com.ijchart.xychart.ui.RectangleEdge ;
import com.ijchart.xychart.util.DateUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class CombinedXYPlotDemo2 extends JPanel {
  private Calendar cal = Calendar.getInstance() ;

  public CombinedXYPlotDemo2() {
    setLayout(new BorderLayout()) ;
    addPanel1() ;
  }

  private void addPanel1() {
    ChartOrientation orientation = ChartOrientation.VERTICAL ;

    IMatrixDataset dataset1 = createDataset1() ;
    IMatrixDataset dataset2 = createDataset2() ;

    IntervalXNumberYBaseValueAxis xAxis1 =
      new IntervalXNumberYBaseValueAxis("Date", dataset1, XYValueAxisType.XY_VALUE_AXIS_X, true) ;
    xAxis1.setAxisLabelType(AxisLabelType.COLUMN) ;

    IntervalXNumberYBaseValueAxis yAxis1 =
      new IntervalXNumberYBaseValueAxis("Value", dataset1, XYValueAxisType.XY_VALUE_AXIS_Y, false) ;

    IntervalXNumberYBaseBar2DRender render1 = new IntervalXNumberYBaseBar2DRender(dataset1) ;
    if (orientation.equals(ChartOrientation.HORIZONTAL)) {
      xAxis1.setAxisLabelAngle(IAngle.ANGLE_UP_90) ;
    } else if (orientation.equals(ChartOrientation.VERTICAL)) {
      yAxis1.setAxisLabelAngle(IAngle.ANGLE_UP_90) ;
    }

    XYPlot plot1 = new XYPlot(xAxis1, yAxis1, render1) ;
    plot1.setOrientation(orientation) ;

    XYNumberValueAxis xAxis2 = new XYNumberValueAxis("Date", dataset2,
                                                     XYValueAxisType.XY_VALUE_AXIS_X, true) ;
    XYNumberValueAxis yAxis2 = new XYNumberValueAxis("Value", dataset2,
                                                     XYValueAxisType.XY_VALUE_AXIS_Y, false) ;

    XYLineAndShapeRender render2 = new XYLineAndShapeRender(dataset2) ;
    render2.setBaseItemShapeVisible(false) ;
    if (orientation.equals(ChartOrientation.HORIZONTAL)) {
      xAxis2.setAxisLabelAngle(IAngle.ANGLE_UP_90) ;
    } else if (orientation.equals(ChartOrientation.VERTICAL)) {
      yAxis2.setAxisLabelAngle(IAngle.ANGLE_UP_90) ;
    }

    XYPlot plot2 = new XYPlot(xAxis2, yAxis2, render2) ;

    CombineValueAxisXYPlot combinePlot = new CombineValueAxisXYPlot() ;
    combinePlot.addSubPlot(plot1, 1) ;
    combinePlot.addSubPlot(plot2, 1) ;
    combinePlot.setPlotGap(15);

    IJChart chart = new IJChart("Combined (Range) XY Plot", IJChart.DEFAULT_TITLE_FONT,
                                combinePlot, true) ;
    LegendTitle title = chart.getLegendTitle() ;
    title.setPosition(RectangleEdge.RIGHT) ;
    title.setBackgroundPaint(Color.WHITE) ;
    title.setFillBackground(true);

    ChartPanel chartpanel = new ChartPanel(chart) ;
    add(chartpanel) ;
  }

  private DefaultMatrixDataset createDataset1() {
    DefaultMatrixDataset dataset = new DefaultMatrixDataset() ;
    dataset.addRowKey("Series 1") ;
    dataset.addRowData() ;

    dataset.addData(0, createIntervalDayData(1, 3, 2002, 12353.299999999999D)) ;
    dataset.addData(0, createIntervalDayData(2, 3, 2002, 13734.4D)) ;
    dataset.addData(0, createIntervalDayData(3, 3, 2002, 14525.299999999999D)) ;
    dataset.addData(0, createIntervalDayData(4, 3, 2002, 13984.299999999999D)) ;
    dataset.addData(0, createIntervalDayData(5, 3, 2002, 12999.4D)) ;
    dataset.addData(0, createIntervalDayData(6, 3, 2002, 14274.299999999999D)) ;
    dataset.addData(0, createIntervalDayData(7, 3, 2002, 15943.5D)) ;
    dataset.addData(0, createIntervalDayData(8, 3, 2002, 14845.299999999999D)) ;
    dataset.addData(0, createIntervalDayData(9, 3, 2002, 14645.4D)) ;
    dataset.addData(0, createIntervalDayData(10, 3, 2002, 16234.6D)) ;
    dataset.addData(0, createIntervalDayData(11, 3, 2002, 17232.299999999999D)) ;
    dataset.addData(0, createIntervalDayData(12, 3, 2002, 14232.200000000001D)) ;
    dataset.addData(0, createIntervalDayData(13, 3, 2002, 13102.200000000001D)) ;
    dataset.addData(0, createIntervalDayData(14, 3, 2002, 14230.200000000001D)) ;
    dataset.addData(0, createIntervalDayData(15, 3, 2002, 11235.200000000001D)) ;

    return dataset ;
  }

  private static DefaultMatrixDataset createDataset2() {
    DefaultMatrixDataset dataset = new DefaultMatrixDataset() ;
    dataset.addRowKey("Series 2") ;
    dataset.addRowData() ;

    dataset.addData(0, createXYDayData(3, 3, 2002, 6853.1999999999998D)) ;
    dataset.addData(0, createXYDayData(4, 3, 2002, 9642.2999999999993D)) ;
    dataset.addData(0, createXYDayData(5, 3, 2002, 8253.5D)) ;
    dataset.addData(0, createXYDayData(6, 3, 2002, 5352.3000000000002D)) ;
    dataset.addData(0, createXYDayData(7, 3, 2002, 3532D)) ;
    dataset.addData(0, createXYDayData(8, 3, 2002, 2635.3000000000002D)) ;
    dataset.addData(0, createXYDayData(9, 3, 2002, 3998.1999999999998D)) ;
    dataset.addData(0, createXYDayData(10, 3, 2002, 1943.2D)) ;
    dataset.addData(0, createXYDayData(11, 3, 2002, 6943.8999999999996D)) ;
    dataset.addData(0, createXYDayData(12, 3, 2002, 7843.1999999999998D)) ;
    dataset.addData(0, createXYDayData(13, 3, 2002, 6495.3000000000002D)) ;
    dataset.addData(0, createXYDayData(14, 3, 2002, 7943.6000000000004D)) ;
    dataset.addData(0, createXYDayData(15, 3, 2002, 8500.7000000000007D)) ;
    dataset.addData(0, createXYDayData(16, 3, 2002, 9595.8999999999996D)) ;

    return dataset ;
  }

  private IntervalXNumberYBaseChartData createIntervalDayData(int day, int month, int year,
                                                              double value) {
    cal.set(Calendar.YEAR, year) ;
    cal.set(Calendar.MONTH, month - 1) ;
    cal.set(Calendar.DAY_OF_MONTH, day) ;

    long startTime = DateUtil.dateToTimeInMillis(cal.get(Calendar.YEAR),
                                                 cal.get(Calendar.MONTH) + 1, day) ;

    long endTime = startTime + 20 * 60 * 60 * 1000L ;

    return new IntervalXNumberYBaseChartData(startTime, endTime, value) ;
  }

  private static XYNumberChartData createXYDayData(int day, int month, int year, double value) {
    long mills = DateUtil.dateToTimeInMillis(year, month, day) ;
    return new XYNumberChartData(mills, value) ;
  }
}
