package demo.granttcharts ;

import java.awt.Color ;
import java.awt.GridLayout ;
import java.text.SimpleDateFormat ;
import java.util.Calendar ;

import javax.swing.JPanel ;

import com.ijchart.xychart.axis.value.IntervalXYValueAxis ;
import com.ijchart.xychart.axis.value.SymbolValueAxis ;
import com.ijchart.xychart.axis.value.XYNumberValueAxis ;
import com.ijchart.xychart.client.ChartPanel ;
import com.ijchart.xychart.client.IJChart ;
import com.ijchart.xychart.constant.AxisLabelType ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.constant.DateTickUnitType ;
import com.ijchart.xychart.constant.IAngle ;
import com.ijchart.xychart.constant.XYValueAxisType ;
import com.ijchart.xychart.data.IntervalXYNumberChartData ;
import com.ijchart.xychart.data.XYNumberChartData ;
import com.ijchart.xychart.dataset.DefaultMatrixDataset ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.plot.XYPlot ;
import com.ijchart.xychart.plot.combine.CombineValueAxisXYPlot ;
import com.ijchart.xychart.render.xy.XYIntervalBar2DRender ;
import com.ijchart.xychart.render.xy.XYLineAndShapeRender ;
import com.ijchart.xychart.tickunit.DateTickUnit ;
import com.ijchart.xychart.tickunit.LongTickUnit ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class XYTaskDatasetDemo2 extends JPanel {

  private ChartOrientation orientation = ChartOrientation.HORIZONTAL ;

  private Calendar calendar = Calendar.getInstance() ;

  public XYTaskDatasetDemo2() {
    calendar.clear() ;
    setLayout(new GridLayout(1, 1)) ;
    createChart() ;
  }

  private void createChart() {
    CombineValueAxisXYPlot plot = new CombineValueAxisXYPlot() ;
    plot.addSubPlot(createPlot1()) ;
    plot.addSubPlot(createPlot2()) ;

    IJChart chart = new IJChart("XYTaskDatasetDemo2", plot) ;
    ChartPanel chartpanel = new ChartPanel(chart) ;
    add(chartpanel) ;
  }

  private XYPlot createPlot1() {
    IMatrixDataset dataset = createDataset1() ;
    IntervalXYValueAxis xAxis = new IntervalXYValueAxis("Resources", dataset,
                                                        XYValueAxisType.XY_VALUE_AXIS_X, false) ;
    xAxis.setAxisLabelType(AxisLabelType.COLUMN) ;
    xAxis.setTickUnit(new LongTickUnit()) ;
    xAxis.setTickMarkVisible(false) ;
    String[] symbols = {"", "Team A", "Team B", "Team C", "Team D", "Team E", ""} ;
    SymbolValueAxis symbolAxis = new SymbolValueAxis("Resource", xAxis, symbols) ;
    symbolAxis.setAxisLabelAngle(IAngle.ANGLE_UP_90) ;

    IntervalXYValueAxis yAxis = new IntervalXYValueAxis("Date/Time", dataset,
                                                        XYValueAxisType.XY_VALUE_AXIS_Y, false) ;
    yAxis.setValueLineVisible(true) ;

    XYIntervalBar2DRender render = new XYIntervalBar2DRender(dataset) ;
    render.setGeneratorTooltip(false) ;
    if (orientation.equals(ChartOrientation.HORIZONTAL)) {
      xAxis.setAxisLabelAngle(IAngle.ANGLE_UP_90) ;
    } else if (orientation.equals(ChartOrientation.VERTICAL)) {
      yAxis.setAxisLabelAngle(IAngle.ANGLE_UP_90) ;
    }

    XYPlot plot = new XYPlot(xAxis, yAxis, render) ;
    plot.setOrientation(orientation) ;
    plot.setDomainAxis(symbolAxis) ;

    IntervalXYValueAxis valueAxis = (IntervalXYValueAxis) plot.getValueAxis() ;
    valueAxis.setTickUnit(new DateTickUnit(DateTickUnitType.DATE_UNIT_HOUR,
                                           new SimpleDateFormat("HH"))) ;

    return plot ;
  }

  private XYPlot createPlot2() {
    IMatrixDataset dataset = createDataset2() ;
    XYNumberValueAxis xAxis = new XYNumberValueAxis("Y", dataset,
                                                    XYValueAxisType.XY_VALUE_AXIS_X, false) ;
    xAxis.setAxisLabelType(AxisLabelType.COLUMN) ;
    XYNumberValueAxis yAxis = new XYNumberValueAxis(null, dataset,
                                                    XYValueAxisType.XY_VALUE_AXIS_Y, true) ;
    yAxis.setValueLineVisible(true) ;
    yAxis.setTickUnit(new DateTickUnit(DateTickUnitType.DATE_UNIT_HOUR,
                                       new SimpleDateFormat("HH"))) ;

    XYLineAndShapeRender render = new XYLineAndShapeRender(dataset) ;
    render.setGeneratorTooltip(false) ;
    if (orientation.equals(ChartOrientation.HORIZONTAL)) {
      xAxis.setAxisLabelAngle(IAngle.ANGLE_UP_90) ;
    } else if (orientation.equals(ChartOrientation.VERTICAL)) {
      yAxis.setAxisLabelAngle(IAngle.ANGLE_UP_90) ;
    }
    render.setGeneratorTooltip(false) ;
    render.setAutoSelectFillPaint(false);
    render.setBaseItemFillPaint(Color.white) ;

    XYPlot plot = new XYPlot(xAxis, yAxis, render) ;
    plot.setOrientation(orientation) ;

    return plot ;
  }

  private IMatrixDataset createDataset1() {
    DefaultMatrixDataset dataset = new DefaultMatrixDataset() ;
    String[] rowKeys = {"Team A", "Team B", "Team C", "Team D", "Team E"} ;
    for (int i = 0 ; i < rowKeys.length ; i++) {
      dataset.addRowKey(rowKeys[i]) ;
    }

    dataset.addRowData(rowKeys.length) ;
    dataset.addData(0, createData(1, 11, 12)) ;
    dataset.addData(0, createData(1, 14, 15)) ;
    dataset.addData(0, createData(1, 16, 17)) ;

    dataset.addData(1, createData(2, 13, 14)) ;
    dataset.addData(1, createData(2, 19, 20)) ;
    dataset.addData(1, createData(2, 21, 22)) ;

    dataset.addData(2, createData(3, 13, 14)) ;
    dataset.addData(2, createData(3, 19, 20)) ;
    dataset.addData(2, createData(3, 21, 22)) ;

    dataset.addData(3, createData(4, 0, 24)) ;
    dataset.addData(4, createData(5, 0, 24)) ;

    return dataset ;
  }

  private IMatrixDataset createDataset2() {
    DefaultMatrixDataset dataset = new DefaultMatrixDataset() ;
    dataset.addRowKey("Time Series 1") ;
    dataset.addRowData() ;

    dataset.addData(0, createXYData(0, 20214.5D)) ;
    dataset.addData(0, createXYData(4, 73346.5D)) ;
    dataset.addData(0, createXYData(8, 54643.599999999999D)) ;
    dataset.addData(0, createXYData(12, 92683.800000000003D)) ;
    dataset.addData(0, createXYData(16, 110235.39999999999D)) ;
    dataset.addData(0, createXYData(20, 120742.5D)) ;
    dataset.addData(0, createXYData(24, 90654.5D)) ;

    return dataset ;
  }

  private XYNumberChartData createXYData(int hour, double value) {
    return new XYNumberChartData(value, getMills(hour)) ;
  }

  private IntervalXYNumberChartData createData(double centerX, int startHour, int endHour) {
    return new IntervalXYNumberChartData(centerX - 0.25, centerX + 0.25, getMills(startHour),
                                         getMills(endHour)) ;
  }

  private long getMills(int hour) {
    calendar.set(Calendar.HOUR_OF_DAY, hour) ;
    return calendar.getTimeInMillis() ;
  }

}
