package demo.images ;

import java.awt.Color ;
import java.awt.GradientPaint ;
import java.util.List ;

import com.ijchart.xychart.axis.category.CategoryAxis ;
import com.ijchart.xychart.axis.value.NumberValueAxis ;
import com.ijchart.xychart.chartinfo.ChartShapeCollection ;
import com.ijchart.xychart.chartinfo.LegendShapeCollection ;
import com.ijchart.xychart.chartinfo.draw.ChangeChartShapeAdapter ;
import com.ijchart.xychart.client.ChartFactory ;
import com.ijchart.xychart.client.IJChart ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.constant.GradientPaintTransformType ;
import com.ijchart.xychart.data.NumberChartData ;
import com.ijchart.xychart.dataset.DefaultMatrixDataset ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.plot.CategoryPlot ;
import com.ijchart.xychart.render.category.Bar2DRender ;
import com.ijchart.xychart.shape.IChartShape ;
import com.ijchart.xychart.tickunit.LongTickUnit ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class BarChartHotspotTest {

  public IJChart createChart() {
    final IMatrixDataset dataset = createDataset1() ;
    final IJChart chart = ChartFactory.createCategoryBar2DChart(
      "Bar Chart Demo 1", "Category", "Value", dataset, ChartOrientation.VERTICAL, true, true) ;

    CategoryPlot plot = (CategoryPlot) chart.getPlot() ;
    plot.setDomainGridlinePaint(Color.white) ;
    plot.setValueGridlinePaint(Color.white) ;
    plot.getRender().setBackgroundPaint(Color.lightGray) ;

    plot.setValueCrosshairVisible(true) ;
    plot.setValueCrosshairPaint(Color.blue) ;
    NumberValueAxis numberaxis = (NumberValueAxis) plot.getValueAxis() ;
    numberaxis.setTickUnit(new LongTickUnit()) ;

    Bar2DRender render = (Bar2DRender) plot.getRender() ;
    render.setBaseItemDrawBorder(false) ;
    GradientPaint gradientpaint = new GradientPaint(0.0F, 0.0F, Color.blue, 0.0F, 0.0F,
                                                    new Color(0, 0, 64)) ;
    GradientPaint gradientpaint1 = new GradientPaint(0.0F, 0.0F, Color.green, 0.0F, 0.0F,
                                                     new Color(0, 64, 0)) ;
    GradientPaint gradientpaint2 = new GradientPaint(0.0F, 0.0F, Color.red, 0.0F, 0.0F,
                                                     new Color(64, 0, 0)) ;
    render.setAutoSelectFillPaint(false) ;
    render.setBaseItemFillPaintTransformer(GradientPaintTransformType.HORIZONTAL) ;
    render.setColumnItemFillPaint(0, gradientpaint) ;
    render.setColumnItemFillPaint(1, gradientpaint1) ;
    render.setColumnItemFillPaint(2, gradientpaint2) ;

    CategoryAxis categoryaxis = plot.getCategoryAxis() ;
    categoryaxis.setTickLabelAngle( -0.52359877559829882D) ;

    chart.setChangeChartShape(new ChangeChartShapeAdapter() {
      public void beforeDraw(ChartShapeCollection chartShapes) {
        LegendShapeCollection legendShapes = chart.getChartShapes().getLegendShapes() ;
        List shapes = legendShapes.getLegendShapes() ;

        IChartShape shape = (IChartShape) shapes.get(0) ;
        shape.setTooltipText("Tooltip: " + dataset.getColKey(0)) ;
        shape = (IChartShape) shapes.get(1) ;
        shape.setTooltipText("Tooltip: " + dataset.getColKey(1)) ;
        shape = (IChartShape) shapes.get(2) ;
        shape.setTooltipText("Tooltip: " + dataset.getColKey(2)) ;
      }

    }) ;

    return chart ;

  }

  private static IMatrixDataset createDataset1() {
    DefaultMatrixDataset dataset = new DefaultMatrixDataset() ;
    dataset.addRowKey("Category 1") ;
    dataset.addRowKey("Category 2") ;
    dataset.addRowKey("Category 3") ;
    dataset.addRowKey("Category 4") ;
    dataset.addRowKey("Category 5") ;

    dataset.addColKey("First") ;
    dataset.addColKey("Second") ;
    dataset.addColKey("Third") ;

    double[][] data = { {1.0D, 5D, 4D}, {4D, 7D, 3D}, {3D, 6D, 2D}, {5D, 8D, 3D}, {5D, 4D, 6D}
    } ;
    dataset.addRowData(data.length) ;

    for (int i = 0 ; i < data.length ; i++) {
      for (int j = 0 ; j < 3 ; j++) {
        dataset.addData(i, new NumberChartData(data[i][j])) ;
      }
    }

    return dataset ;
  }
}
