package demo.timeseriescharts;

import java.awt.Color ;
import java.awt.GridLayout ;

import javax.swing.JPanel ;

import com.ijchart.xychart.axis.value.XYNumberValueAxis ;
import com.ijchart.xychart.client.ChartFactory ;
import com.ijchart.xychart.client.ChartPanel ;
import com.ijchart.xychart.client.IJChart ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.constant.DateTickUnitType ;
import com.ijchart.xychart.data.XYNumberChartData ;
import com.ijchart.xychart.dataset.DefaultMatrixDataset ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.format.MillisecondsToDateFormatter ;
import com.ijchart.xychart.plot.XYPlot ;
import com.ijchart.xychart.render.xy.XYLineAndShapeRender ;
import com.ijchart.xychart.tickunit.DateTickUnit ;
import com.ijchart.xychart.tooltip.DefaultTooltipGenerator ;
import com.ijchart.xychart.util.DateUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class TimeSeriesDemo3 extends JPanel {

  public TimeSeriesDemo3() {
    setLayout(new GridLayout(1, 1)) ;
    addPanel1() ;
  }

  private void addPanel1() {
    IMatrixDataset dataset = createDataset() ;
    IJChart chart = ChartFactory.createXYLineAndShapeChart("Time Series Demo 3", "Time",
                                                           "Value", dataset,
                                                           ChartOrientation.VERTICAL,
                                                           true, true, true, false) ;

    XYPlot xyplot = (XYPlot) chart.getPlot() ;

    XYLineAndShapeRender render = (XYLineAndShapeRender) xyplot.getRender() ;
    render.setBaseItemShapeVisible(true) ;
    render.setRowItemFillPaint(0, Color.red) ;
    render.setRowItemFillPaint(1, Color.white) ;
    render.setAutoSelectFillPaint(false) ;
    render.setToolTipGenerator(new DefaultTooltipGenerator()) ;

    XYNumberValueAxis yAxis = (XYNumberValueAxis) xyplot.getValueAxis() ;
    yAxis.setAxisRangeIncludeZero(false) ;

    XYNumberValueAxis xAxis = (XYNumberValueAxis) xyplot.getDomainAxis() ;
    xAxis.setTickUnit(new DateTickUnit(DateTickUnitType.DATE_UNIT_MONTH)) ;
    xAxis.setTickLabelAngle( -Math.PI / 2) ;
    xAxis.setTickValueFormatter(new MillisecondsToDateFormatter("MMM-yyyy")) ;

    ChartPanel chartpanel = new ChartPanel(chart) ;
    add(chartpanel) ;
  }

  private static IMatrixDataset createDataset() {
    DefaultMatrixDataset dataset = new DefaultMatrixDataset() ;
    dataset.addRowKey("Series 1") ;
    dataset.addRowKey("Series 2") ;
    dataset.addRowData(2) ;

    dataset.addData(0, createData(1, 2002, 500.19999999999999D)) ;
    dataset.addData(0, createData(2, 2002, 694.10000000000002D)) ;
    dataset.addData(0, createData(3, 2002, 734.39999999999998D)) ;
    dataset.addData(0, createData(4, 2002, 453.19999999999999D)) ;
    dataset.addData(0, createData(5, 2002, 500.19999999999999D)) ;
    dataset.addData(0, createData(6, 2002, 345.60000000000002D)) ;
    dataset.addData(0, createData(7, 2002, 500.19999999999999D)) ;
    dataset.addData(0, createData(8, 2002, 694.10000000000002D)) ;
    dataset.addData(0, createData(9, 2002, 734.39999999999998D)) ;
    dataset.addData(0, createData(10, 2002, 453.19999999999999D)) ;
    dataset.addData(0, createData(11, 2002, 500.19999999999999D)) ;
    dataset.addData(0, createData(12, 2002, 345.60000000000002D)) ;

    dataset.addData(1, createData(1, 2002, 234.09999999999999D)) ;
    dataset.addData(1, createData(2, 2002, 623.70000000000005D)) ;
    dataset.addData(1, createData(3, 2002, 642.5D)) ;
    dataset.addData(1, createData(4, 2002, 651.39999999999998D)) ;
    dataset.addData(1, createData(5, 2002, 643.5D)) ;
    dataset.addData(1, createData(6, 2002, 785.60000000000002D)) ;
    dataset.addData(1, createData(7, 2002, 234.09999999999999D)) ;
    dataset.addData(1, createData(8, 2002, 623.70000000000005D)) ;
    dataset.addData(1, createData(9, 2002, 642.5D)) ;
    dataset.addData(1, createData(10, 2002, 651.39999999999998D)) ;
    dataset.addData(1, createData(11, 2002, 643.5D)) ;
    dataset.addData(1, createData(12, 2002, 785.60000000000002D)) ;

    return dataset ;
  }

  private static XYNumberChartData createData(int month, int year, double value) {
    return new XYNumberChartData(DateUtil.dateToTimeInMillis(year, month), value) ;
  }

}
