package com.ijchart.xychart.chartinfo ;

import java.awt.Graphics2D ;
import java.awt.geom.Point2D ;
import java.awt.geom.Rectangle2D ;
import java.util.List ;

import com.ijchart.xychart.constant.ChartShapeDrawType ;
import com.ijchart.xychart.shape.IChartShape ;
import com.ijchart.xychart.web.IAreaFragment ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ͼеÿԪضһͼζ{@link com.ijchart.xychart.shape.IChartShape} ,
 * ¼:⡢ͼXᡢY(жY)ͼ.ÿҪ嵥洢ͼԪ.</zh>
 * <en>All elements in chart is a {@link com.ijchart.xychart.shape.IChartShape} ,
 * type is:title,legend,X-axis,Y-axis(possible multiple),main.
 * Each type need to define a separate class to store {@link com.ijchart.xychart.shape.IChartShape}
 * elements. </en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public interface IShapeCollection {
  /**
   * <zh>һϵеͼ,ǹͬĳ,ͨ¼,ڴ¼ʱṩٶ,Ҫ
   * еͼжǷڱѡ,ѡͨСѰҵķΧ.</zh>
   * <en>For a series of graphics and their common belonging to a particular region, by recording
   * the region, in dealing with the incident when the mouse to provide faster, no need to
   * determine whether all the graphics are selected by the mouse, optional through this area
   * To narrow the scope of the search.</en>
   *
   * @param drawArea Rectangle2D <zh>ͼμڵ (Ϊnull).</zh>
   * <en>The draw area for all shapes (null permitted).</en>
   */
  void setDrawArea(Rectangle2D drawArea) ;

  /**
   * <zh>ͼμͬ.</zh>
   * <en>Returns the draw area for all shapes.</en>
   *
   * @return Rectangle2D <zh>ͼμͬ.</zh>
   * <en>The draw area for all shapes.</en>
   */
  Rectangle2D getDrawArea() ;

  /**
   * <zh>ͼμϹͬǷpoint.</zh>
   * <en>Whether or not the draw area contains a point.</en>
   *
   * @param point Point2D <zh>ҪǷĵ (Ϊnull).</zh>
   * <en>The point (null not permitted).</en>
   *
   * @return boolean <zh>򷵻true,򷵻false.</zh>
   * <en>Returns true if the draw area contains point,otherwise return false.</en>
   */
  boolean isDrawAreaContains(Point2D point) ;

  /**
   * <zh>ذpointͼ.</zh>
   * <en>Returns the {@link IChartShape} whose area contains the specified point.</en>
   *
   * @param point Point2D <zh>ҪԵĵ (Ϊnull).</zh>
   * <en>The point (null not permitted).</en>
   * @param drawType ChartShapeDrawType <zh>ͼεĻ (Ϊnull).</zh>
   * <en>The shape draw shape (null permitted).</en>
   *
   * @return IChartShape <zh>ͼ (Ϊnull).</zh>
   * <en>The chart shape (maybe null).</en>
   */
  IChartShape findChartShape(Point2D point, ChartShapeDrawType drawType) ;

  /**
   * <zh>λƼÿһͼ.</zh>
   * <en>Drawing all ChartShape in this collection.</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(ӡĻ) (Ϊnull).</zh>
   * <en>The graphics device (null not permitted).</en>
   */
  void draw(Graphics2D g2) ;

  /**
   * <zh>ƶͼεԭ㵽(x,y).</zh>
   * <en>Translates the origin of the all shape in this chart to the point (x,&nbsp;y)
   * in the current coordinate system. </en>
   *
   * @param  x double <zh>x.</zh>
   * <en>The x coordinate.</en>
   * @param  y double <zh>y.</zh>
   * <en>The y coordinate.</en>
   */
  void translate(double x, double y) ;

  /**
   * <zh>еͼεļ.</zh>
   * <en>Remove all the shapes of the listener.</en>
   */
  void clearShapesListener() ;

  /**
   * <zh>ͼкΪ0.</zh>
   * <en>Reset the sequence of shape to 0.</en>
   */
  void resetShapeIdSequence() ;

  /**
   * <zh>еͼΪѡ״̬.</zh>
   * <en>Sets all shapes to unselected.</en>
   */
  void clearSelectedShapes() ;

  /**
   * <zh>ͼεȵArea.</zh>
   * <en>Returns the hotspot areas.</en>
   *
   * @param drawTypes List <zh>ʾȵͼ (Ϊnull).</zh>
   * <en>The types of hotspot shape (null not permitted).</en>
   * @param fragment IAreaFragment <zh>ʵֽӿ{@link IAreaFragment}Ķ,(Ϊnull)</zh>
   * <en>The IAreaFragment object (maybe null).</en>
   *
   * @return List <zh>ͼεȵArea (Ϊnull).</zh>
   * <en>The hotspot areas (never null).</en>
   */
  List getHotspotAreas(List drawTypes, IAreaFragment fragment) ;
}
