package com.ijchart.xychart.constant ;

import java.io.Serializable ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ֺ{@link com.ijchart.xychart.shape.IChartShape}λùϵ.</zh>
 * <en>Define the text position with {@link com.ijchart.xychart.shape.IChartShape}.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public final class TextPosition implements Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = 7102364970022089634L ;

  /**
   * <zh>ֲʾ.</zh>
   * <en>Text is not display.</en>
   */
  public static final TextPosition NONE = new TextPosition("TextPosition.NONE") ;

  /**
   * <zh>{@link com.ijchart.xychart.shape.IChartShape}ڲ.</zh>
   * <en>The text in {@link com.ijchart.xychart.shape.IChartShape} internal.</en>
   */
  public static final TextPosition INSIDE = new TextPosition("TextPosition.INSIDE") ;

  /**
   * <zh>{@link com.ijchart.xychart.shape.IChartShape}ⲿ.</zh>
   * <en>The text in the top of the external {@link com.ijchart.xychart.shape.IChartShape}.</en>
   */
  public static final TextPosition OUTSIDE_TOP = new TextPosition("TextPosition.OUTSIDE_TOP") ;

  /**
   * <zh>{@link com.ijchart.xychart.shape.IChartShape}ⲿײ.</zh>
   * <en>The text in the bottom side of the external {@link com.ijchart.xychart.shape.IChartShape}.</en>
   */
  public static final TextPosition OUTSIDE_BOTTOM = new TextPosition(
    "TextPosition.OUTSIDE_BOTTOM") ;

  /**
   * <zh>{@link com.ijchart.xychart.shape.IChartShape}ⲿ.</zh>
   * <en>The text in the left side of the external {@link com.ijchart.xychart.shape.IChartShape}.</en>
   */
  public static final TextPosition OUTSIDE_LEFT = new TextPosition("TextPosition.OUTSIDE_LEFT") ;

  /**
   * <zh>{@link com.ijchart.xychart.shape.IChartShape}ⲿҲ.</zh>
   * <en>The text in the right side of the external {@link com.ijchart.xychart.shape.IChartShape}.</en>
   */
  public static final TextPosition OUTSIDE_RIGHT = new TextPosition("TextPosition.OUTSIDE_RIGHT") ;

  /**
   * <zh>.</zh>
   * <en>The name.</en>
   */
  private String name ;

  /**
   * <zh>˽йֹʵ.</zh>
   * <en>Private constructor to prevent instance.</en>
   *
   * @param name <zh>.</zh>
   * <en>The name.</en>
   */
  private TextPosition(String name) {
    this.name = name ;
  }

  /**
   * <zh>õ෴λ.</zh>
   * <en>Returns the opposite position.</en>
   *
   * @param orientation ChartOrientation <zh>ͼķ (Ϊnull).</zh>
   * <en>The chart orientation (null not permitted).</en>
   *
   * @return TextPosition <zh>µĵλ (Ϊnull).</zh>
   * <en>The new position (not null).</en>
   */
  public TextPosition getOppositePosition(ChartOrientation orientation) {
    if (orientation == null) {
      throw new IllegalArgumentException("Null 'orientation' argument.") ;
    }

    if (orientation.equals(ChartOrientation.VERTICAL)) {
      if (this.equals(OUTSIDE_TOP)) {
        return OUTSIDE_BOTTOM ;
      } else if (this.equals(OUTSIDE_BOTTOM)) {
        return OUTSIDE_TOP ;
      }
    } else if (orientation.equals(ChartOrientation.HORIZONTAL)) {
      if (this.equals(OUTSIDE_LEFT)) {
        return OUTSIDE_RIGHT ;
      } else if (this.equals(OUTSIDE_RIGHT)) {
        return OUTSIDE_LEFT ;
      }
    }

    return this ;
  }

  /**
   * <zh>equals.</zh>
   * <en>Override equals method.</en>
   * @param obj Object
   * @return boolean
   */
  public boolean equals(Object obj) {
    if (this == obj) {
      return true ;
    }

    if (!(obj instanceof TextPosition)) {
      return false ;
    }

    TextPosition tmp = (TextPosition) obj ;

    return this.name.equals(tmp.name) ;
  }

  /**
   * <zh>toString().</zh>
   * <en>Override toString method.</en>
   * @return String
   */
  public String toString() {
    return "[name=" + name + "]" ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object. </en>
   * @return  a hash code value for this object.
   */
  public int hashCode() {
    return (this.name == null) ? 0 : this.name.hashCode() ;
  }

}
