package com.ijchart.xychart.data ;

import java.io.Serializable ;

import com.ijchart.xychart.util.EqualsUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>Interval()ͼݶ,ÿһֵһֵ,ⲿǲ޸ĵ.</zh>
 * <en>Interval ChartData object,each object has a height value and a low value,
 * this class is immutable.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class IntervalChartData extends AbstractChartData implements Cloneable, Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = -8379483985317556165L ;

  /**
   * <zh>ʼֵ.</zh>
   * <en>The start value.</en>
   */
  private Number startValue ;

  /**
   * <zh>ֵ.</zh>
   * <en>The end value.</en>
   */
  private Number endValue ;

  /**
   * <zh>.</zh>
   * <en>Constructor.</en>
   *
   * @param startValue double <zh>ʼֵ.</zh>
   * <en>The start value.</en>
   * @param endValue double <zh>ֵ.</zh>
   * <en>The end value.</en>
   */
  public IntervalChartData(double startValue, double endValue) {
    this(new Double(startValue), new Double(endValue)) ;
  }

  /**
   * <zh>.</zh>
   * <en>Constructor.</en>
   *
   * @param startValue Number <zh>ʼֵ (Ϊnull).</zh>
   * <en>The start value (null not permitted).</en>
   * @param endValue Number <zh>ֵ (Ϊnull).</zh>
   * <en>The end value (null not permitted).</en>
   */
  public IntervalChartData(Number startValue, Number endValue) {
    if (startValue == null) {
      throw new IllegalArgumentException("Null 'startValue' argument.") ;
    }
    if (endValue == null) {
      throw new IllegalArgumentException("Null 'endValue' argument.") ;
    }

    this.startValue = startValue ;
    this.endValue = endValue ;
  }

  /**
   * <zh>ؿʼֵ(Ϊnull).</zh>
   * <en>Get Number type of start value (maybe null).</en>
   *
   * @return Number <zh>Ϊnull.</zh>
   * <en>Maybe null.</en>
   */
  public Number getStartData() {
    return this.startValue ;
  }

  /**
   * <zh>õdouble͵Ŀʼֵ,ֵΪnull򷵻Double.NaN.</zh>
   * <en>Returns double type of start value,if value is null then return Double.NaN.</en>
   *
   * @return double <zh>double͵Ŀʼֵ,ֵΪnull򷵻Double.NaN.</zh>
   * <en>The double type start value,if value is null then return Double.NaN.</en>
   */
  public double getStartValue() {
    if (this.startValue == null) {
      return Double.NaN ;
    }

    return this.startValue.doubleValue() ;
  }

  /**
   * <zh>ؽֵ(Ϊnull).</zh>
   * <en>Get Number type of end value (maybe null).</en>
   *
   * @return Number <zh>Ϊnull.</zh>
   * <en>Maybe null.</en>
   */
  public Number getEndData() {
    return this.endValue ;
  }

  /**
   * <zh>õdouble͵Ľֵ,ֵΪnull򷵻Double.NaN.</zh>
   * <en>Returns double type of end value,if value is null then return Double.NaN.</en>
   *
   * @return double <zh>double͵Ľֵ,ֵΪnull򷵻Double.NaN.</zh>
   * <en>The double type end value,if value is null then return Double.NaN.</en>
   */
  public double getEndValue() {
    if (this.endValue == null) {
      return Double.NaN ;
    }

    return this.endValue.doubleValue() ;
  }

  /**
   * <zh>رǩַ.</zh>
   * <en>Returns the label text.</en>
   *
   * @return String <zh>ǩַ (Ϊnull).</zh>
   * <en>The label text (maybe null).</en>
   */
  public String getLabelText() {
    return "(" + formatNumber(this.startValue) + "," + formatNumber(this.endValue) + ")" ;
  }

  /**
   * <zh>ͼȵareaĺ.</zh>
   * <en>Returns the function arguments of hotspot area.</en>
   *
   * @return String <zh> (Ϊnull).</zh>
   * <en>The function arguments (maybe null).</en>
   */
  public String getAreaFunctionArguments() {
    return formatNumberForFunction(this.startValue) + "," + formatNumberForFunction(this.endValue) ;
  }

  /**
   * <zh>ͼȵareaurl.</zh>
   * <en>Returns the url for hotspot area.</en>
   *
   * @return String <zh>url (Ϊnull).</zh>
   * <en>The url fragment (maybe null).</en>
   */
  public String getAreaURLFragment() {
    return "startValue=" + formatNumber(this.startValue) + "&endValue="
      + formatNumber(this.endValue) ;
  }

  /**
   * <zh>ʵComparableӿڵcompareTo().</zh>
   * <en>Implements the compareTo() method of interface Comparable.</en>
   *
   * @param o Object <zh>ҪȽϵĶ.</zh>
   * <en>The Object to be compared.</en>
   * @return int <zh>ֵΪʾС,0ʾ,ʾ.</zh>
   * <en>A negative integer, zero, or a positive integer as this object	is less than, equal to, or
   * greater than the specified object.</en>
   *
   * @throws ClassCastException if the specified object's type prevents it
   *         from being compared to this Object.
   */
  public int compareTo(Object o) {
    if (o == null) {
      return -1 ;
    } else {
      if (!(o instanceof IntervalChartData)) {
        throw new ClassCastException("Must be 'IntervalChartData'.") ;
      }

      IntervalChartData other = (IntervalChartData) o ;
      double diff = this.startValue.doubleValue() - other.getStartValue() ;

      return (diff < 0) ? -1 : ((diff == 0) ? 0 : 1) ;
    }
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Tests this instance for equality with an arbitrary object.</en>
   *
   * @param obj  the object (null permitted).
   *
   * @return A boolean.
   */
  public boolean equals(Object obj) {
    if (obj == this) {
      return true ;
    }
    if (!(obj instanceof IntervalChartData)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    IntervalChartData that = (IntervalChartData) obj ;
    if (EqualsUtil.equals(this.startValue, that.startValue)) {
      return false ;
    }
    if (EqualsUtil.equals(this.endValue, that.endValue)) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return int <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    int result = super.hashCode() ;
    result = 37 * result + (int) (100 * ((startValue == null) ? 0 : startValue.doubleValue())) ;
    result = 37 * result + (int) (100 * ((endValue == null) ? 0 : endValue.doubleValue())) ;

    return result ;
  }

  /**
   * <zh>ؿ¡Ķ.</zh>
   * <en>Creates and returns a copy of this object.</en>
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the Cloneable interface. Subclasses
   *               that override the clone method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object.</en>
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "[startValue=" + startValue + ",endValue=" + endValue + "]" ;
  }
}
