package com.ijchart.xychart.data ;

import java.io.Serializable ;

import com.ijchart.xychart.util.EqualsUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>Statistical()ͼݶ,ÿһmean(ƽ)һdeviation(ƫ),
 * ⲿǲ޸ĵ</zh>
 * <en>Statistical ChartData object,each object has a mean value and a deviation value,
 * this class is immutable.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class StatisticalChartData extends AbstractChartData implements Cloneable, Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = -8379483985317556165L ;

  /**
   * <zh>ƽ.</zh>
   * <en>The mean value.</en>
   */
  private Number mean ;

  /**
   * <zh>ƫ.</zh>
   * <en>The stand deviation value.</en>
   */
  private Number standardDeviation ;

  /**
   * <zh>.</zh>
   * <en>Constructor.</en>
   *
   * @param mean double <zh>ƽ.</zh>
   * <en>The mean value.</en>
   * @param standardDeviation double <zh>ƫ.</zh>
   * <en>The stand deviation value.</en>
   */
  public StatisticalChartData(double mean, double standardDeviation) {
    this(new Double(mean), new Double(standardDeviation)) ;
  }

  /**
   * <zh>.</zh>
   * <en>Constructor.</en>
   *
   * @param mean Number <zh>ƽ (Ϊnull).</zh>
   * <en>The mean value (null permitted).</en>
   * @param standardDeviation Number <zh>ƫ (Ϊnull).</zh>
   * <en>The stand deviation value (null not permitted).</en>
   */
  public StatisticalChartData(Number mean, Number standardDeviation) {
    if (mean == null) {
      throw new IllegalArgumentException("Null 'mean' argument.") ;
    }
    if (standardDeviation == null) {
      throw new IllegalArgumentException("Null 'standardDeviation' argument.") ;
    }

    this.mean = mean ;
    this.standardDeviation = standardDeviation ;
  }

  /**
   * <zh>ƽ(Ϊnull).</zh>
   * <en>Get Number type of mean value (maybe null).</en>
   *
   * @return Number <zh>Ϊnull.</zh>
   * <en>Maybe null.</en>
   */
  public Number getMeanData() {
    return this.mean ;
  }

  /**
   * <zh>õdouble͵ƽ,ֵΪnull򷵻Double.NaN.</zh>
   * <en>Returns double type of mean value,if value is null then return Double.NaN.</en>
   *
   * @return double <zh>double͵ƽ,ֵΪnull򷵻Double.NaN.</zh>
   * <en>The double type mean value,if value is null then return Double.NaN.</en>
   */
  public double getMeanValue() {
    if (this.mean == null) {
      return Double.NaN ;
    }

    return this.mean.doubleValue() ;
  }

  /**
   * <zh>ؾƫ(Ϊnull).</zh>
   * <en>Get Number type of deviation value (maybe null).</en>
   *
   * @return Number <zh>Ϊnull.</zh>
   * <en>Maybe null.</en>
   */
  public Number getStandDeviationData() {
    return this.standardDeviation ;
  }

  /**
   * <zh>õdouble͵ľƫ,ֵΪnull򷵻Double.NaN.</zh>
   * <en>Returns double type of deviation value,if value is null then return Double.NaN.</en>
   *
   * @return double <zh>double͵ľƫ,ֵΪnull򷵻Double.NaN.</zh>
   * <en>The double type deviation value,if value is null then return Double.NaN.</en>
   */
  public double getStandDeviationValue() {
    if (this.standardDeviation == null) {
      return Double.NaN ;
    }

    return this.standardDeviation.doubleValue() ;
  }

  /**
   * <zh>رǩַ.</zh>
   * <en>Returns the label text.</en>
   *
   * @return String <zh>ǩַ (Ϊnull).</zh>
   * <en>The label text (maybe null).</en>
   */
  public String getLabelText() {
    return "(" + formatNumber(this.mean) + "," + formatNumber(this.standardDeviation) + ")" ;
  }

  /**
   * <zh>ͼȵareaĺ.</zh>
   * <en>Returns the function arguments of hotspot area.</en>
   *
   * @return String <zh> (Ϊnull).</zh>
   * <en>The function arguments (maybe null).</en>
   */
  public String getAreaFunctionArguments() {
    return formatNumberForFunction(this.mean) + ","
      + formatNumberForFunction(this.standardDeviation) ;
  }

  /**
   * <zh>ͼȵareaurl.</zh>
   * <en>Returns the url for hotspot area.</en>
   *
   * @return String <zh>url (Ϊnull).</zh>
   * <en>The url fragment (maybe null).</en>
   */
  public String getAreaURLFragment() {
    return "mean=" + formatNumber(this.mean) + "&standardDeviation="
      + formatNumber(this.standardDeviation) ;
  }

  /**
   * <zh>ʵComparableӿڵcompareTo().</zh>
   * <en>Implements the compareTo() method of interface Comparable.</en>
   *
   * @param o Object <zh>ҪȽϵĶ.</zh>
   * <en>The Object to be compared.</en>
   * @return int <zh>ֵΪʾС,0ʾ,ʾ.</zh>
   * <en>A negative integer, zero, or a positive integer as this object	is less than, equal to, or
   * greater than the specified object.</en>
   *
   * @throws ClassCastException if the specified object's type prevents it
   *         from being compared to this Object.
   */
  public int compareTo(Object o) {
    if (o == null) {
      return -1 ;
    } else {
      if (!(o instanceof StatisticalChartData)) {
        throw new ClassCastException("Must be 'StatisticalChartData'.") ;
      }

      StatisticalChartData other = (StatisticalChartData) o ;
      double diff = this.mean.doubleValue() - other.getMeanValue() ;

      return (diff < 0) ? -1 : ((diff == 0) ? 0 : 1) ;
    }
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Tests this instance for equality with an arbitrary object.</en>
   *
   * @param obj  the object (null permitted).
   *
   * @return A boolean.
   */
  public boolean equals(Object obj) {
    if (obj == this) {
      return true ;
    }
    if (!(obj instanceof StatisticalChartData)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    StatisticalChartData that = (StatisticalChartData) obj ;
    if (EqualsUtil.equals(this.mean, that.mean)) {
      return false ;
    }
    if (EqualsUtil.equals(this.standardDeviation, that.standardDeviation)) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return int <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    int result = super.hashCode() ;
    result = 37 * result + (int) (100 * ((mean == null) ? 0 : mean.doubleValue())) ;
    result = 37 * result + (int) (100 * ((standardDeviation == null) ?
                                         0 : standardDeviation.doubleValue())) ;

    return result ;
  }

  /**
   * <zh>ؿ¡Ķ.</zh>
   * <en>Creates and returns a copy of this object.</en>
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the Cloneable interface. Subclasses
   *               that override the clone method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object.</en>
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "[mean=" + mean + ",standardDeviation=" + standardDeviation + "]" ;
  }
}
