package com.ijchart.xychart.format ;

import java.io.Serializable ;
import java.text.DecimalFormat ;
import java.text.NumberFormat ;

import com.ijchart.xychart.data.StatisticalChartData ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.HashcodeUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ʽ{@link com.ijchart.xychart.data.StatisticalChartData}͵.</zh>
 * <en>A formatter class for {@link com.ijchart.xychart.data.StatisticalChartData} chart data.</en>
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class StatisticalChartDataFormatter extends AbstractChartDataFormatterAdapter implements
  IChartDataFormatter, Cloneable, Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = 5417561258233846241L ;

  /**
   * <zh>ֻʾmeanֵ.</zh>
   * <en>Show mean value only.</en>
   */
  public static final int STATISTICAL_FORMATTER_TYPE_MEAN = 1 ;

  /**
   * <zh>ֻʾdeviationֵ.</zh>
   * <en>Show deviation value only.</en>
   */
  public static final int STATISTICAL_FORMATTER_TYPE_DEVIATION = 2 ;

  /**
   * <zh>ʾ(mean,deviation)ֵ.</zh>
   * <en>Show (mean,deviation) value.</en>
   */
  public static final int STATISTICAL_FORMATTER_TYPE_MEAN_DEVIATION = 3 ;

  /**
   * <zh>ʽ.</zh>
   * <en>The format type;</en>
   */
  private int formatType ;

  /**
   * <zh>ʽmeanֵ.</zh>
   * <en>The mean-value format object.</en>
   */
  private DecimalFormat meanFormat ;

  /**
   * <zh>ʽdeviationֵ.</zh>
   * <en>The deviation-value format object.</en>
   */
  private DecimalFormat deviationFormat ;

  /**
   * <zh>.</zh>
   * <en>Constructor.</en>
   *
   * @param meanMask String <zh>meanֵ (Ϊnull).</zh>
   * <en>The mean-value format mask (null not permitted).</en>
   * @param deviationMask String <zh>deviationֵ (Ϊnull).</zh>
   * <en>The deviation-value format mask (null not permitted).</en>
   * @param formatType int <zh>ʽ,ֵܵΪ{@link #STATISTICAL_FORMATTER_TYPE_MEAN},
   * {@link #STATISTICAL_FORMATTER_TYPE_DEVIATION},
   * {@link #STATISTICAL_FORMATTER_TYPE_MEAN_DEVIATION}.</zh>
   * <en>The format type,see {@link #STATISTICAL_FORMATTER_TYPE_MEAN},
   * {@link #STATISTICAL_FORMATTER_TYPE_DEVIATION},
   * {@link #STATISTICAL_FORMATTER_TYPE_MEAN_DEVIATION}.</en>
   */
  public StatisticalChartDataFormatter(String meanMask, String deviationMask, int formatType) {
    if (meanMask == null) {
      throw new IllegalArgumentException("Null 'meanMask' argument.") ;
    }
    if (deviationMask == null) {
      throw new IllegalArgumentException("Null 'deviationMask' argument.") ;
    }

    this.meanFormat = (DecimalFormat) NumberFormat.getInstance() ;
    this.meanFormat.applyPattern(meanMask) ;

    this.deviationFormat = (DecimalFormat) NumberFormat.getInstance() ;
    this.deviationFormat.applyPattern(deviationMask) ;

    this.formatType = formatType ;
  }

  /**
   * <zh>ظʽ.</zh>
   * <en>Returns the format type;</en>
   *
   * @return int <zh>ʽ.</zh>
   * <en>The format type;</en>
   */
  public int getFormatType() {
    return this.formatType ;
  }

  /**
   * <zh>ظʽmeanֵ.</zh>
   * <en>Returns the mean-value format object.</en>
   *
   * @return DecimalFormat <zh>ʽmeanֵ.</zh>
   * <en>The mean-value format object.</en>
   */
  public DecimalFormat getMeanFormat() {
    return this.meanFormat ;
  }

  /**
   * <zh>ظʽdeviationֵ.</zh>
   * <en>Returns the deviation-value format object.</en>
   *
   * @return DecimalFormat <zh>ʽdeviationֵ.</zh>
   * <en>The deviation-value format object.</en>
   */
  public DecimalFormat getDeviationFormat() {
    return this.deviationFormat ;
  }

  /**
   * <zh>ʽݼĳ{@link com.ijchart.xychart.data.IChartData}Ϊַ.</zh>
   * <en>Formatted an {@link com.ijchart.xychart.data.IChartData} object to a string.</en>
   *
   * @param dataset IMatrixDataset <zh>ݼ (Ϊnull).</zh>
   * <en>The dataset (null not permitted).</en>
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row number,must be greater than or equal to 0.</en>
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column number,must be greater than or equal to 0.</en>
   *
   * @return String <zh>ַ (Ϊnull).</zh>
   * <en>A string (maybe null).</en>
   */
  public String format(IMatrixDataset dataset, int row, int column) {
    if (dataset == null) {
      throw new IllegalArgumentException("Null 'dataset' argument.") ;
    }
    if (row < 0) {
      throw new IllegalArgumentException("Must be 'row'>=0.") ;
    }
    if (column < 0) {
      throw new IllegalArgumentException("Must be 'column'>=0.") ;
    }

    Object obj = dataset.getData(row, column) ;
    if (obj != null && obj instanceof StatisticalChartData) {
      StatisticalChartData data = (StatisticalChartData) obj ;
      if (this.formatType == STATISTICAL_FORMATTER_TYPE_MEAN) {
        return this.meanFormat.format(data.getMeanValue()) ;
      } else if (this.formatType == STATISTICAL_FORMATTER_TYPE_DEVIATION) {
        return this.deviationFormat.format(data.getStandDeviationValue()) ;
      } else if (this.formatType == STATISTICAL_FORMATTER_TYPE_MEAN_DEVIATION) {
        return "(" + this.meanFormat.format(data.getMeanValue()) + ","
          + this.deviationFormat.format(data.getStandDeviationValue()) + ")" ;
      }
    }

    return null ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Tests this object for equality with an arbitrary object.</en>
   *
   * @param obj <zh>ҪȽϵһ (Ϊnull).</zh>
   * <en>The object to test against (null permitted).</en>
   *
   * @return A boolean.
   */
  public boolean equals(Object obj) {
    if (this == obj) {
      return true ;
    }
    if (!(obj instanceof XYNumberChartDataFormatter)) {
      return false ;
    }

    StatisticalChartDataFormatter that = (StatisticalChartDataFormatter) obj ;
    if (!EqualsUtil.equals(this.meanFormat, that.meanFormat)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.deviationFormat, that.deviationFormat)) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>¡.</zh>
   * <en>Creates and returns a copy of this object.</en>
   *
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the <code>Cloneable</code> interface. Subclasses
   *               that override the <code>clone</code> method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  protected Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>hashcode.</zh>
   * <en>Returns a hash code.</en>
   *
   * @return A hash code.
   */
  public int hashCode() {
    int hashCode = HashcodeUtil.hashCode(this.meanFormat) ;
    hashCode = 37 * hashCode + HashcodeUtil.hashCode(this.deviationFormat) ;

    return hashCode ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of this Range.</en>
   *
   * @return A String.
   */
  public String toString() {
    return "[meanFormat=" + this.meanFormat + ",deviationFormat=" + this.deviationFormat + "]" ;
  }

}
