package com.ijchart.xychart.label ;

import java.io.Serializable ;

import com.ijchart.xychart.data.IChartData ;
import com.ijchart.xychart.data.group.AbstractGroupChartData ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.format.IChartDataFormatter ;
import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.HashcodeUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ȱʡıǩ.ֶformatterûset,Բüı仯.</zh>
 * <en>The default label generator,this class is immutable.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class DefaultLabelGenerator extends LabelGeneratorAdaptor implements ILabelGenerator,
  Cloneable, Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = 7901078192031332107L ;

  /**
   * <zh>ȱʡǷʽɱǩ.</zh>
   * <en>The default flag controls whether or not the data type is group.</en>
   */
  public static final boolean DEFAULT_LABEL_GENERATOR_GROUP = false ;

  /**
   * <zh>ʽݵ.</zh>
   * <en>The data formatter.</en>
   */
  private IChartDataFormatter formatter ;

  /**
   * <zh>Ƿʽɱǩ.</zh>
   * <en>The flag controls whether or not the data type is group.</en>
   */
  private boolean group ;

  /**
   * <zh>ûвĹ.</zh>
   * <en>No argument constructor.</en>
   */
  public DefaultLabelGenerator() {
    this(null, DEFAULT_LABEL_GENERATOR_GROUP) ;
  }

  /**
   * <zh>.</zh>
   * <en>Constructor.</en>
   *
   * @param group boolean <zh>Ƿʽɱǩ.</zh>
   * <en>The flag controls whether or not the data type is group.</en>
   */
  public DefaultLabelGenerator(boolean group) {
    this(null, group) ;
  }

  /**
   * <zh>.</zh>
   * <en>Constructor.</en>
   *
   * @param formatter IChartDataFormatter <zh>ʽݵ (Ϊnull).</zh>
   * <en>The data formatter (null permitted).</en>
   */
  public DefaultLabelGenerator(IChartDataFormatter formatter) {
    this(formatter, DEFAULT_LABEL_GENERATOR_GROUP) ;
  }

  /**
   * <zh>.</zh>
   * <en>Constructor.</en>
   *
   * @param formatter IChartDataFormatter <zh>ʽݵ (Ϊnull).</zh>
   * <en>The data formatter (null permitted).</en>
   * @param group boolean <zh>Ƿʽɱǩ.</zh>
   * <en>The flag controls whether or not the data type is group.</en>
   */
  public DefaultLabelGenerator(IChartDataFormatter formatter, boolean group) {
    this.formatter = formatter ;
    this.group = group ;
  }

  /**
   * <zh>ظʽݵ.</zh>
   * <en>Returns this data formatter.</en>
   *
   * @return IChartDataFormatter <zh>ʽݵ (Ϊnull).</zh>
   * <en>The data formatter (maybe null).</en>
   */
  public IChartDataFormatter getFormatter() {
    return this.formatter ;
  }

  /**
   * <zh>Ƿʽɱǩ.</zh>
   * <en>The flag controls whether or not the data type is group.</en>
   *
   * @return boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  public boolean isGroup() {
    return this.group ;
  }

  /**
   * <zh>ĳͼεıǩ,ͨ±ǩ֪ͨʽrowKeycolKeyֵɵ.
   * ҲԸʽΪַ.</zh>
   * <en>Generates a label for the specified item. The label is typically a
   * formatted version of the data value, but any text can be used.</en>
   *
   * @param dataset IMatrixDataset <zh>ݼ (Ϊnull).</zh>
   * <en>The dataset (null not permitted).</en>
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row number,must be greater than or equal to 0.</en>
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column number,must be greater than or equal to 0.</en>
   *
   * @return String <zh>ǩַ.</zh>
   * <en>The label (possibly null).</en>
   */
  public String generateLabel(IMatrixDataset dataset, int row, int column) {
    if (dataset == null) {
      throw new IllegalArgumentException("Null 'dataset' argument.") ;
    }
    if (row < 0) {
      throw new IllegalArgumentException("Must be row>=0.") ;
    }
    if (column < 0) {
      throw new IllegalArgumentException("Must be column>=0.") ;
    }

    if (this.formatter != null) {
      return this.formatter.format(dataset, row, column) ;
    }

    IChartData data = dataset.getData(row, column) ;
    if (data == null) {
      return "" ;
    }

    return data.getLabelText() ;
  }

  /**
   * <zh>ĳͼεıǩ,ͨ±ǩ֪ͨʽrowKeycolKeyֵɵ.
   * ҲԸʽΪַ.</zh>
   * <en>Generates a label for the specified item. The label is typically a
   * formatted version of the data value, but any text can be used.</en>
   *
   * @param dataset IMatrixDataset <zh>ݼ (Ϊnull).</zh>
   * <en>The dataset (null not permitted).</en>
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row number,must be greater than or equal to 0.</en>
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column number,must be greater than or equal to 0.</en>
   * @param index int <zh>еһݵɶ,ֵʾ.</zh>
   * <en>The index of a dataset data.</en>
   *
   * @return String <zh>ǩַ.</zh>
   * <en>The label (possibly null).</en>
   */
  public String generateGroupLabel(IMatrixDataset dataset, int row, int column, int index) {
    if (dataset == null) {
      throw new IllegalArgumentException("Null 'dataset' argument.") ;
    }
    if (row < 0) {
      throw new IllegalArgumentException("Must be row>=0.") ;
    }
    if (column < 0) {
      throw new IllegalArgumentException("Must be column>=0.") ;
    }
    if (index < 0) {
      throw new IllegalArgumentException("Must be index>=0.") ;
    }

    if (this.formatter != null) {
      return this.formatter.format(dataset, row, column, index) ;
    }

    IChartData data = dataset.getData(row, column) ;
    if (data == null) {
      return "" ;
    } else {
      if (group && (data instanceof AbstractGroupChartData)) {
        AbstractGroupChartData groupData = (AbstractGroupChartData) data ;
        groupData.toLabelText(index) ;
      }
    }

    return data.getLabelText() ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Tests this object for equality with an arbitrary object.</en>
   *
   * @param obj <zh>ҪȽϵһ (Ϊnull).</zh>
   * <en>The object to test against (null permitted).</en>
   *
   * @return A boolean.
   */
  public boolean equals(Object obj) {
    if (this == obj) {
      return true ;
    }
    if (!(obj instanceof DefaultLabelGenerator)) {
      return false ;
    }

    DefaultLabelGenerator that = (DefaultLabelGenerator) obj ;
    if (!EqualsUtil.equals(this.formatter, that.formatter)) {
      return false ;
    }
    if (this.group != that.group) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>¡.</zh>
   * <en>Creates and returns a copy of this object.</en>
   *
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the <code>Cloneable</code> interface. Subclasses
   *               that override the <code>clone</code> method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  protected Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>hashcode.</zh>
   * <en>Returns a hash code.</en>
   *
   * @return A hash code.
   */
  public int hashCode() {
    int result = HashcodeUtil.hashCode(this.formatter) ;
    return 37 * result + HashcodeUtil.hashCode(this.group) ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of this Range.</en>
   *
   * @return A String.
   */
  public String toString() {
    return "[formatter=" + this.formatter + ",group=" + this.group + "]" ;
  }

}
