package com.ijchart.xychart.marker ;

import java.awt.BasicStroke ;
import java.awt.Color ;
import java.awt.Paint ;
import java.awt.Stroke ;
import java.io.Serializable ;

import com.ijchart.xychart.ui.DefaultGradientPaintTransformer ;
import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.HashcodeUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ڱʾһݷΧı.</zh>
 * <en>Represents an interval to be highlighted in some way.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class IntervalValueMarker extends AbstractMarker implements Cloneable, Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = -1903053798912850872L ;

  /**
   * <zh>ʼֵ.</zh>
   * <en>The start value.</en>
   */
  private double startValue ;

  /**
   * <zh>ֵ.</zh>
   * <en>The end value.</en>
   */
  private double endValue ;

  /**
   * <zh>GradientPaintTransformTypeGradient任.</zh>
   * <en>The gradient paint transformer (optional).</en>
   */
  private DefaultGradientPaintTransformer gradientPaintTransformer ;

  /**
   * <zh>ʼֵͽֵ.</zh>
   * <en>Constructs an interval marker.</en>
   *
   * @param start double <zh>ʼֵ.</zh>
   * <en>The start of the interval.</en>
   * @param end double <zh>ֵ.</zh>
   * <en>The end of the interval.</en>
   */
  public IntervalValueMarker(double start, double end) {
    this(start, end, Color.LIGHT_GRAY, Color.gray, new BasicStroke(0.5f), 0.8f) ;
  }

  /**
   * <zh>.</zh>
   * <en>Constructs an interval marker.</en>
   *
   * @param start double <zh>ʼֵ.</zh>
   * <en>The start of the interval.</en>
   * @param end double <zh>ֵ.</zh>
   * <en>The end of the interval.</en>
   * @param fillPaint Paint <zh>ʹõpaint (Ϊnull).</zh>
   * <en>The fill paint (null not permitted).</en>
   * @param borderPaint Paint <zh>ʹõPaint (Ϊnull).</zh>
   * <en>The border paint (null permitted).</en>
   * @param borderStroke Stroke <zh>ʹõStroke (Ϊnull).</zh>
   * <en>The border stroke (null permitted).</en>
   * @param fillAlpha float <zh>䱳alphaֵ,ڱ[0.0,1.0].</zh>
   * <en>The alpha transparency (must be in the range 0.0f to 1.0f).</en>
   */
  public IntervalValueMarker(double start, double end, Paint fillPaint, Paint borderPaint,
                             Stroke borderStroke, float fillAlpha) {
    super(fillPaint, borderPaint, borderStroke, fillAlpha) ;

    this.startValue = start ;
    this.endValue = end ;
    this.gradientPaintTransformer = null ;
  }

  /**
   * <zh>ʼֵ.</zh>
   * <en>Returns the start value for the interval.</en>
   *
   * @return double <zh>ʼֵ.</zh>
   * <en>The start value.</en>
   */
  public double getStartValue() {
    return this.startValue ;
  }

  /**
   * <zh>ʼֵ,м{@link com.ijchart.xychart.event.MarkerChangeEvent}¼.</zh>
   * <en>Sets the start value for the marker and sends a
   * {@link com.ijchart.xychart.event.MarkerChangeEvent} to all registered listeners.</en>
   *
   * @param value double <zh>ʼֵ.</zh>
   * <en>The value.</en>
   */
  public void setStartValue(double value) {
    if (this.startValue != value) {
      this.startValue = value ;
      fireMarkerChanged() ;
    }
  }

  /**
   * <zh>ؽֵ.</zh>
   * <en>Returns the end value for the interval.</en>
   *
   * @return double <zh>ֵ.</zh>
   * <en>The end value.</en>
   */
  public double getEndValue() {
    return this.endValue ;
  }

  /**
   * <zh>ýֵ,м{@link com.ijchart.xychart.event.MarkerChangeEvent}¼.</zh>
   * <en>Sets the end value for the marker and sends a
   * {@link com.ijchart.xychart.event.MarkerChangeEvent} to all registered listeners.</en>
   *
   * @param value double <zh>ֵ.</zh>
   * <en>The value.</en>
   */
  public void setEndValue(double value) {
    if (this.endValue != value) {
      this.endValue = value ;
      fireMarkerChanged() ;
    }
  }

  /**
   * <zh>DefaultGradientPaintTransformer.</zh>
   * <en>Returns the gradient paint transformer.</en>
   *
   * @return DefaultGradientPaintTransformer <zh>DefaultGradientPaintTransformer (Ϊnull).</zh>
   * <en>The gradient paint transformer (possibly null).</en>
   */
  public DefaultGradientPaintTransformer getGradientPaintTransformer() {
    return this.gradientPaintTransformer ;
  }

  /**
   * <zh>DefaultGradientPaintTransformer,м
   * {@link com.ijchart.xychart.event.MarkerChangeEvent}¼.</zh>
   * <en>Sets the gradient paint transformer and sends a
   * {@link com.ijchart.xychart.event.MarkerChangeEvent} to all registered listeners.</en>
   *
   * @param transformer DefaultGradientPaintTransformer
   * <zh>DefaultGradientPaintTransformer (Ϊnull).</zh>
   * <en>The transformer (null permitted).</en>
   */
  public void setGradientPaintTransformer(DefaultGradientPaintTransformer transformer) {
    this.gradientPaintTransformer = transformer ;
    fireMarkerChanged() ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Tests the marker for equality with an arbitrary object.</en>
   *
   * @param obj  the object (null permitted).
   *
   * @return A boolean.
   */
  public boolean equals(Object obj) {
    if (obj == this) {
      return true ;
    }
    if (!(obj instanceof IntervalValueMarker)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    IntervalValueMarker that = (IntervalValueMarker) obj ;
    if (this.startValue != that.startValue) {
      return false ;
    }
    if (this.endValue != that.endValue) {
      return false ;
    }
    if (!EqualsUtil.equals(this.gradientPaintTransformer, that.gradientPaintTransformer)) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return int The hashcode value.
   */
  public int hashCode() {
    int result = HashcodeUtil.hashCode(this.gradientPaintTransformer) ;
    result = 37 * result + (int) (100 * this.startValue) ;
    result = 37 * result + (int) (100 * this.endValue) ;

    return result ;
  }

  /**
   * <zh>¡.</zh>
   * <en>Returns a clone of the marker.</en>
   *
   * @return A clone.
   *
   * @throws CloneNotSupportedException Not thrown by this class, but the
   *         exception is declared for the use of subclasses.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

}
