package com.ijchart.xychart.plot.combine ;

import java.awt.Paint ;
import java.awt.Stroke ;
import java.awt.geom.Point2D ;
import java.awt.geom.Rectangle2D ;
import java.io.Serializable ;
import java.util.ArrayList ;
import java.util.Collection ;
import java.util.Collections ;
import java.util.Iterator ;
import java.util.List ;

import com.ijchart.xychart.constant.AxisLocation ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.plot.AbstractAxisPlot ;
import com.ijchart.xychart.plot.AbstractPlot ;
import com.ijchart.xychart.plot.CategoryPlot ;
import com.ijchart.xychart.render.AbstractRender ;
import com.ijchart.xychart.ui.RectangleEdge ;
import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.HashcodeUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>CategoryPlot֮乲,Ϊ.</zh>
 * <en>A abstract class for combined category plot where the axis is shared.</en>
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public abstract class AbstractCombineCategoryPlot extends CategoryPlot implements Cloneable,
  Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = -3495602516175104655L ;

  /**
   * <zh>ȱʡplot֮ļ.</zh>
   * <en>The default gap between plots.</en>
   */
  public static final double DEFAULT_PLOT_GAP = 5.0 ;

  /**
   * <zh>plot.</zh>
   * <en>Storage for the subplot.</en>
   */
  protected List subPlots ;

  /**
   * <zh>plotȨ.</zh>
   * <en>The wight of sub plots.</en>
   */
  protected List subPlotWight ;

  /**
   * <zh>plot֮ļ.</zh>
   * <en>The gap between plots.</en>
   */
  protected double plotGap ;

  /**
   * <zh>޲.</zh>
   * <en>A constructor.</en>
   */
  public AbstractCombineCategoryPlot() {
    this.subPlots = new ArrayList() ;
    this.subPlotWight = new ArrayList() ;
    this.plotGap = DEFAULT_PLOT_GAP ;
  }

  /**
   * <zh>һplot,м{@link com.ijchart.xychart.event.PlotChangeEvent}¼.</zh>
   * <en>Add a sub plot and sends a
   * {@link com.ijchart.xychart.event.PlotChangeEvent} to all registered listeners.</en>
   *
   * @param plot CategoryPlot <zh>CategoryPlot (Ϊnull).</zh>
   * <en>The CategoryPlot object (null not permitted).</en>
   */
  public void addSubPlot(CategoryPlot plot) {
    addSubPlot(plot, 1) ;
  }

  /**
   * <zh>һplot,м{@link com.ijchart.xychart.event.PlotChangeEvent}¼.</zh>
   * <en>Add a sub plot and sends a
   * {@link com.ijchart.xychart.event.PlotChangeEvent} to all registered listeners.</en>
   *
   * @param plot CategoryPlot <zh>CategoryPlot (Ϊnull).</zh>
   * <en>The CategoryPlot object (null not permitted).</en>
   * @param wight double <zh>Ȩ,0.</zh>
   * <en>The widght,must be >0.</en>
   */
  public void addSubPlot(CategoryPlot plot, double wight) {
    if (plot == null) {
      throw new IllegalArgumentException("Null 'plot' argument") ;
    }
    if (wight <= 0) {
      throw new IllegalArgumentException("Argument 'wight' must greater than 0.") ;
    }

    Collection renders = plot.getRenders() ;
    Iterator iter = renders.iterator() ;
    while (iter.hasNext()) {
      AbstractRender render = (AbstractRender) iter.next() ;
      if (render != null) {
        render.addChangeListener(this) ;
      }
    }

    this.subPlots.add(plot) ;
    this.subPlotWight.add(new Double(wight)) ;
    firePlotChanged() ;
  }

  /**
   * <zh>ɾһplot,м{@link com.ijchart.xychart.event.PlotChangeEvent}¼.</zh>
   * <en>Remove a sub plot and sends a
   * {@link com.ijchart.xychart.event.PlotChangeEvent} to all registered listeners.</en>
   *
   * @param plot CategoryPlot <zh>CategoryPlot (Ϊnull).</zh>
   * <en>The CategoryPlot object (null not permitted).</en>
   */
  public void removeSubPlot(CategoryPlot plot) {
    if (plot == null) {
      throw new IllegalArgumentException("Null 'plot' argument") ;
    }

    boolean find = false ;
    for (int i = 0 ; i < this.subPlots.size() ; i++) {
      if (this.subPlots.get(i) == plot) {
        this.subPlots.remove(i) ;
        this.subPlotWight.remove(i) ;
        find = true ;
      }
    }

    if (find) {
      firePlotChanged() ;
    }
  }

  /**
   * <zh>еplot.</zh>
   * <en>Returns all sub plots.</en>
   *
   * @return Collection <zh>plot󼯺,˶󲻿޸.</zh>
   * <en>The collection of sub plots (unmodifiable).</en>
   */
  public Collection getSubPlots() {
    return Collections.unmodifiableCollection(this.subPlots) ;
  }

  /**
   * <zh>ɾеplot.</zh>
   * <en>Removes all sub plots.</en>
   */
  public void clearSubPlots() {
    this.subPlots.clear() ;
    this.subPlotWight.clear() ;

    firePlotChanged() ;
  }

  /**
   * <zh>ָplotȨ(wight).</zh>
   * <en>Returns the wight for the specified plot.</en>
   *
   * @param plot CategoryPlot <zh>CategoryPlot (Ϊnull).</zh>
   * <en>The CategoryPlot object (null not permitted).</en>
   *
   * @return double <zh>Ȩ,ûҵplot򷵻-1.</zh>
   * <en>The wight,-1 if not found.</en>
   */
  public double getPlotWight(CategoryPlot plot) {
    if (plot == null) {
      throw new IllegalArgumentException("Null 'plot' argument") ;
    }

    for (int i = 0 ; i < this.subPlots.size() ; i++) {
      if (this.subPlots.get(i) == plot) {
        return ((Double)this.subPlotWight.get(i)).doubleValue() ;
      }
    }

    return -1 ;
  }

  /**
   * <zh>ָplotȨ(wight),м
   * {@link com.ijchart.xychart.event.PlotChangeEvent}¼.</zh>
   * <en>Sets the wight for the specified plot and sends a
   * {@link com.ijchart.xychart.event.PlotChangeEvent} to all registered listeners.</en>
   *
   * @param plot CategoryPlot <zh>CategoryPlot (Ϊnull).</zh>
   * <en>The CategoryPlot object (null not permitted).</en>
   * @param wight double <zh>Ȩ,0.</zh>
   * <en>The widght,must be >0.</en>
   */
  public void setPlotWight(CategoryPlot plot, double wight) {
    if (plot == null) {
      throw new IllegalArgumentException("Null 'plot' argument") ;
    }
    if (wight <= 0) {
      throw new IllegalArgumentException("Argument 'wight' must greater than 0.") ;
    }

    boolean find = false ;
    for (int i = 0 ; i < this.subPlots.size() ; i++) {
      if (this.subPlots.get(i) == plot) {
        this.subPlotWight.set(i, new Double(wight)) ;
        find = true ;
      }
    }

    if (find) {
      firePlotChanged() ;
    }
  }

  /**
   * <zh>plot֮ļ.</zh>
   * <en>Returns the gap between plots.</en>
   *
   * @return double <zh>.</zh>
   * <en>The gap.</en>
   */
  public double getPlotGap() {
    return this.plotGap ;
  }

  /**
   * <zh>plot֮ļ,м{@link com.ijchart.xychart.event.PlotChangeEvent}¼.</zh>
   * <en>Sets the gap between plots and sends a
   * {@link com.ijchart.xychart.event.PlotChangeEvent} to all registered listeners.</en>
   *
   * @param gap double
   */
  public void setPlotGap(double gap) {
    if (this.plotGap != gap) {
      this.plotGap = gap ;
      firePlotChanged() ;
    }
  }

  /**
   * <zh>render,˼ϲ޸.</zh>
   * <en>Returns the collection of renders,and this collection is unmodifiable.</en>
   *
   * @return Collection <zh>render.</zh>
   * <en>The collection of renders.</en>
   */
  public Collection getRenders() {
    List allRenders = new ArrayList() ;
    for (int i = 0 ; i < this.subPlots.size() ; i++) {
      CategoryPlot plot = (CategoryPlot)this.subPlots.get(i) ;
      allRenders.addAll(plot.getRenders()) ;
    }

    return Collections.unmodifiableCollection(allRenders) ;
  }

  /**
   * <zh>domain᷽ʮǷɼ,м
   * {@link com.ijchart.xychart.event.PlotChangeEvent}¼.</zh>
   * <en>Sets the flag indicating whether or not the domain crosshair is visible
   * and, if the flag changes, sends a {@link com.ijchart.xychart.event.PlotChangeEvent}
   *  to all registered listeners.</en>
   *
   * @param flag boolean <zh>ֵ.</zh>
   * <en>The new value of the flag.</en>
   */
  public void setDomainCrosshairVisible(boolean flag) {
    if (isDomainCrosshairVisible() != flag) {
      for (int i = 0 ; i < this.subPlots.size() ; i++) {
        AbstractPlot subPlot = (AbstractPlot)this.subPlots.get(i) ;
        if (subPlot instanceof AbstractAxisPlot) {
          ((AbstractAxisPlot) subPlot).setDomainCrosshairVisible(flag) ;
        }
      }

      super.setDomainCrosshairVisible(flag) ;
      firePlotChanged() ;
    }
  }

  /**
   * <zh>ʮǷԶݵ,м
   * {@link com.ijchart.xychart.event.PlotChangeEvent}¼.</zh>
   * <en>Sets the flag indicating whether or not the domain crosshair should
   * "lock-on" to actual data values.  If the flag value changes, this
   * method sends a {@link com.ijchart.xychart.event.PlotChangeEvent}
   *  to all registered listeners.</en>
   *
   * @param flag boolean <zh>ֵ.</zh>
   * <en>The flag.</en>
   */
  public void setDomainCrosshairLockedOnData(boolean flag) {
    if (isDomainCrosshairLockedOnData() != flag) {
      for (int i = 0 ; i < this.subPlots.size() ; i++) {
        AbstractPlot subPlot = (AbstractPlot)this.subPlots.get(i) ;
        if (subPlot instanceof AbstractAxisPlot) {
          ((AbstractAxisPlot) subPlot).setDomainCrosshairLockedOnData(flag) ;
        }
      }

      super.setDomainCrosshairLockedOnData(flag) ;
      firePlotChanged() ;
    }
  }

  /**
   * <zh>domainʮߵStroke,м
   * {@link com.ijchart.xychart.event.PlotChangeEvent}¼.</zh>
   * <en>Sets the Stroke used to draw the crosshairs (if visible) and notifies
   * registered listeners that the axis has been modified.</en>
   *
   * @param stroke Stroke <zh>Stroke (Ϊnull).</zh>
   * <en>The new crosshair stroke (null not permitted).</en>
   */
  public void setDomainCrosshairStroke(Stroke stroke) {
    if (stroke == null) {
      throw new IllegalArgumentException("Null 'stroke' argument.") ;
    }

    for (int i = 0 ; i < this.subPlots.size() ; i++) {
      AbstractPlot subPlot = (AbstractPlot)this.subPlots.get(i) ;
      if (subPlot instanceof AbstractAxisPlot) {
        ((AbstractAxisPlot) subPlot).setDomainCrosshairStroke(stroke) ;
      }
    }

    super.setDomainCrosshairStroke(stroke) ;
    firePlotChanged() ;
  }

  /**
   * <zh>domainʮߵPaint,м
   * {@link com.ijchart.xychart.event.PlotChangeEvent}¼.</zh>
   * <en>Sets the paint used to draw the crosshairs (if visible) and sends a
   * {@link com.ijchart.xychart.event.PlotChangeEvent} to all registered listeners.</en>
   *
   * @param paint Paint <zh>Paint (Ϊnull).</zh>
   * <en>The new crosshair paint (null not permitted).</en>
   */
  public void setDomainCrosshairPaint(Paint paint) {
    if (paint == null) {
      throw new IllegalArgumentException("Null 'paint' argument.") ;
    }

    for (int i = 0 ; i < this.subPlots.size() ; i++) {
      AbstractPlot subPlot = (AbstractPlot)this.subPlots.get(i) ;
      if (subPlot instanceof AbstractAxisPlot) {
        ((AbstractAxisPlot) subPlot).setDomainCrosshairPaint(paint) ;
      }
    }

    setDomainCrosshairPaint(paint) ;
    firePlotChanged() ;
  }

  /**
   * <zh>ÿvalue᷽ʮǷɼ,м
   * {@link com.ijchart.xychart.event.PlotChangeEvent}¼.</zh>
   * <en>Sets the flag indicating whether or not the value crosshair is visible
   * and, if the flag changes, sends a {@link com.ijchart.xychart.event.PlotChangeEvent}
   *  to all registered listeners.</en>
   *
   * @param flag boolean <zh>ֵ.</zh>
   * <en>The new value of the flag.</en>
   */
  public void setValueCrosshairVisible(boolean flag) {
    if (isValueCrosshairVisible() != flag) {
      for (int i = 0 ; i < this.subPlots.size() ; i++) {
        AbstractPlot subPlot = (AbstractPlot)this.subPlots.get(i) ;
        if (subPlot instanceof AbstractAxisPlot) {
          ((AbstractAxisPlot) subPlot).setValueCrosshairVisible(flag) ;
        }
      }

      super.setValueCrosshairVisible(flag) ;
      firePlotChanged() ;
    }
  }

  /**
   * <zh>ÿʮǷԶݵ,м
   * {@link com.ijchart.xychart.event.PlotChangeEvent}¼.</zh>
   * <en>Sets the flag indicating whether or not the value crosshair should
   * "lock-on" to actual data values.  If the flag value changes, this method sends a
   *  {@link com.ijchart.xychart.event.PlotChangeEvent} to all registered listeners.</en>
   *
   * @param flag boolean <zh>ֵ.</zh>
   * <en>The flag.</en>
   */
  public void setValueCrosshairLockedOnData(boolean flag) {
    if (isValueCrosshairLockedOnData() != flag) {
      for (int i = 0 ; i < this.subPlots.size() ; i++) {
        AbstractPlot subPlot = (AbstractPlot)this.subPlots.get(i) ;
        if (subPlot instanceof AbstractAxisPlot) {
          ((AbstractAxisPlot) subPlot).setValueCrosshairLockedOnData(flag) ;
        }
      }

      super.setValueCrosshairLockedOnData(flag) ;
      firePlotChanged() ;
    }
  }

  /**
   * <zh>valueʮߵStroke,м
   * {@link com.ijchart.xychart.event.PlotChangeEvent}¼.</zh>
   * <en>Sets the Stroke used to draw the crosshairs (if visible) and notifies
   * registered listeners that the axis has been modified.</en>
   *
   * @param stroke Stroke <zh>Stroke (Ϊnull).</zh>
   * <en>The new crosshair stroke (null not permitted).</en>
   */
  public void setValueCrosshairStroke(Stroke stroke) {
    if (stroke == null) {
      throw new IllegalArgumentException("Null 'stroke' argument.") ;
    }

    for (int i = 0 ; i < this.subPlots.size() ; i++) {
      AbstractPlot subPlot = (AbstractPlot)this.subPlots.get(i) ;
      if (subPlot instanceof AbstractAxisPlot) {
        ((AbstractAxisPlot) subPlot).setValueCrosshairStroke(stroke) ;
      }
    }

    super.setValueCrosshairStroke(stroke) ;
    firePlotChanged() ;
  }

  /**
   * <zh>valueʮߵPaint,м
   * {@link com.ijchart.xychart.event.PlotChangeEvent}¼.</zh>
   * <en>Sets the paint used to draw the crosshairs (if visible) and sends a
   * {@link com.ijchart.xychart.event.PlotChangeEvent} to all registered listeners.</en>
   *
   * @param paint Paint <zh>Paint (Ϊnull).</zh>
   * <en>The new crosshair paint (null not permitted).</en>
   */
  public void setValueCrosshairPaint(Paint paint) {
    if (paint == null) {
      throw new IllegalArgumentException("Null 'paint' argument.") ;
    }

    for (int i = 0 ; i < this.subPlots.size() ; i++) {
      AbstractPlot subPlot = (AbstractPlot)this.subPlots.get(i) ;
      if (subPlot instanceof AbstractAxisPlot) {
        ((AbstractAxisPlot) subPlot).setValueCrosshairPaint(paint) ;
      }
    }

    super.setValueCrosshairPaint(paint) ;
    firePlotChanged() ;
  }

  /**
   * <zh>ʮ.</zh>
   * <en>Draws the crosshair.</en>
   *
   *
   * @param mouseClickPoint Point2D <zh>ʱ (Ϊnull).</zh>
   * <en>The mouse click point (null permitted).</en>
   *
   * @return boolean <zh>Ƿʮ߷˱仯.</zh>
   * <en>Returns whether or not crosshair is changed.</en>
   */
  public boolean drawCrosshair(Point2D mouseClickPoint) {
    boolean isDrawOne = false ;
    for (int i = 0 ; i < this.subPlots.size() ; i++) {
      AbstractPlot subPlot = (AbstractPlot)this.subPlots.get(i) ;
      if (subPlot instanceof AbstractAxisPlot) {
        boolean isDraw = ((AbstractAxisPlot) subPlot).drawCrosshair(mouseClickPoint) ;
        if (isDraw) {
          isDrawOne = true ;
        }
      }
    }

    return isDrawOne ;
  }

  /**
   * <zh>value.</zh>
   * <en>Zoom value axes.</en>
   *
   * @param selectArea Rectangle2D <zh>ѡ,Ϊnullʾָǰ״̬.</zh>
   * <en>The selected area,null to restore chart.</en>
   * @param dataArea Rectangle2D <zh>,Ϊnullʾָǰ״̬.</zh>
   * <en>The data area,null to restore chart.</en>
   * @param source Point2D <zh>Դ (Ϊnull).</zh>
   * <en>The source mouse point (null not permitted).</en>
   */
  public void zoomValueAxes(Rectangle2D selectArea, Rectangle2D dataArea, Point2D source) {
    if (source == null) {
      throw new IllegalArgumentException("Null 'source' argument.") ;
    }

    for (int i = 0 ; i < this.subPlots.size() ; i++) {
      AbstractPlot subPlot = (AbstractPlot)this.subPlots.get(i) ;
      if (subPlot instanceof AbstractAxisPlot && subPlot.getDataArea().contains(source)) {
        AbstractAxisPlot axisPlot = (AbstractAxisPlot) subPlot ;
        axisPlot.zoomValueAxes(selectArea, dataArea, source) ;
        firePlotChanged();
      }
    }
  }

  /**
   * <zh>ָλõĸ.</zh>
   * <en>Calculate the axis count for the specified position.</en>
   *
   * @param plot CategoryPlot <zh>CategoryPlot.</zh>
   * <en>The CategoryPlot object.</en>
   * @param orientation ChartOrientation <zh>chartķ.</zh>
   * <en>The direction of chart.</en>
   * @param edge RectangleEdge <zh>λ.</zh>
   * <en>The position.</en>
   *
   * @return int <zh>.</zh>
   * <en>The axis count.</en>
   */
  protected int getAxisCountByPosition(CategoryPlot plot, ChartOrientation orientation,
                                       RectangleEdge edge) {
    if (edge == null || plot == null) {
      return 0 ;
    }

    int result = 0 ;
    int domainCount = plot.getCategoryAxisCount() ;
    for (int i = 0 ; i < domainCount ; i++) {
      AxisLocation location = plot.getCategoryAxisLocation(i) ;
      RectangleEdge axisEdge = resolveDomainAxisLocation(location, orientation) ;
      if (axisEdge != null && axisEdge.equals(edge)) {
        result++ ;
      }
    }

    int valueCount = plot.getValueAxisCount() ;
    for (int i = 0 ; i < valueCount ; i++) {
      AxisLocation location = plot.getValueAxisLocation(i) ;
      RectangleEdge axisEdge = resolveValueAxisLocation(location, orientation) ;
      if (axisEdge != null && axisEdge.equals(edge)) {
        result++ ;
      }
    }

    return result ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Tests the plot for equality with an arbitrary object.</en>
   *
   * @param obj <zh>ҪȽϵĶ (Ϊnull).</zh>
   * <en>The object to test against (null permitted).</en>
   *
   * @return A boolean.
   */
  public boolean equals(Object obj) {
    if (obj == this) {
      return true ;
    }
    if (!(obj instanceof AbstractCombineCategoryPlot)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    AbstractCombineCategoryPlot that = (AbstractCombineCategoryPlot) obj ;
    if (!EqualsUtil.equals(this.subPlots, that.subPlots)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.subPlotWight, that.subPlotWight)) {
      return false ;
    }
    if (this.plotGap != that.plotGap) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcode.</zh>
   * <en>Returns a hash code for this object.</en>
   *
   * @return <zh>hashcode.</zh>
   * <en>A hash code.</en>
   */
  public int hashCode() {
    double result = super.hashCode() ;
    result = 29 * result + HashcodeUtil.hashCode(this.subPlots) ;
    result = 29 * result + HashcodeUtil.hashCode(this.subPlotWight) ;
    result = 29 * result + this.plotGap ;

    return (int) result ;
  }

  /**
   * <zh>¡.</zh>
   * <en>Returns a clone of the plot.</en>
   *
   * @return A clone.
   *
   * @throws CloneNotSupportedException  if the cloning is not supported.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

}
