package com.ijchart.xychart.render ;

import java.awt.Paint ;
import java.awt.Stroke ;
import java.awt.geom.Rectangle2D ;
import java.io.Serializable ;
import java.util.Collection ;
import java.util.Iterator ;

import com.ijchart.xychart.axis.value.AbstractValueAxis ;
import com.ijchart.xychart.chartinfo.RenderShapeCollection ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.constant.HorizontalAlignment ;
import com.ijchart.xychart.constant.TextPosition ;
import com.ijchart.xychart.constant.ValueAxisOffsetType ;
import com.ijchart.xychart.constant.VerticalAlignment ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.label.ILabelGenerator ;
import com.ijchart.xychart.marker.IntervalValueMarker ;
import com.ijchart.xychart.marker.ValueMarker ;
import com.ijchart.xychart.shape.AbstractRectangleTextChartShape ;
import com.ijchart.xychart.shape.IChartShape ;
import com.ijchart.xychart.shape.RectangleTextChartShape ;
import com.ijchart.xychart.tooltip.IToolTipGenerator ;
import com.ijchart.xychart.ui.Layer ;
import com.ijchart.xychart.ui.RectangleEdge ;
import com.ijchart.xychart.util.StringUtil ;

/**
 * <p>Title: IJChart</p>
 * <zh>ṩchartrenderĳʵ.</zh>
 * <en>Base class providing common services for renders with axis.</en>
 * <p>Description: a chart library for the Java(tm) platform.</p>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class AbstractWithAxisRender extends AbstractRender implements IRenderWithAxis, Cloneable,
  Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = 8433482242289479988L ;

  /**
   * <zh>.</zh>
   * <en>constructor.</en>
   *
   * @param dataset IMatrixDataset <zh>ݼ (Ϊnull).</zh>
   * <en>The dataset (null not permitted).</en>
   */
  protected AbstractWithAxisRender(IMatrixDataset dataset) {
    super(dataset) ;
  }

  /**
   * <zh>ƫ.</zh>
   * <en>The value axis offset type.</en>
   *
   * @return ValueAxisOffsetType <zh>ƫ (Ϊnull).</zh>
   * <en>The axis offset type (possibly null).</en>
   */
  public ValueAxisOffsetType getValueAxisOffsetType() {
    return ValueAxisOffsetType.OFFSET_TYPE_NONE ;
  }

  /**
   * <zh>һRectangleTextChartShape.</zh>
   * <en>Create a RectangleTextChartShape object.</en>
   *
   * @param orientation ChartOrientation <zh>ͼķ (Ϊnull).</zh>
   * <en>The chart orientation (null not permitted).</en>
   * @param valueAxis AbstractValueAxis <zh>value (Ϊnull).</zh>
   * <en>The value axis (null not permitted).</en>
   * @param row int <zh>ݼ.</zh>
   * <en>The dataset row num.</en>
   * @param column int <zh>ݼ.</zh>
   * <en>The dataset column num.</en>
   * @param rect Rectangle2D <zh>ͼľ (Ϊnull).</zh>
   * <en>The bar rectangle (null permitted).</en>
   * @param autoFillPaint <zh>ԶѡPaint (Ϊnull).</zh>
   * <en>The auto selected fill paint (null permitted).</en>
   * @param autoBorderPaint <zh>ԶѡĻƱ߿Paint (Ϊnull).</zh>
   * <en>The auto selected border paint (null permitted).</en>
   * @param autoBorderStroke <zh>ԶѡĻƱ߿Stroke (Ϊnull).</zh>
   * <en>The auto selected border stroke (null permitted).</en>
   * @param alpha float <zh>alphaֵ,ڱ[0.0,1.0].</zh>
   * <en>The alpha transparency (in the range 0.0f to 1.0f, where 0.0f is fully transparent,
   * and 1.0f is fully opaque).</en>
   * @param appendParam Object <zh>ӵĲ,ڴݶĶ (Ϊnull).</zh>
   * <en>The append parameter,used for special propose (null permitted).</en>
   *
   * @return IChartShape
   */
  protected IChartShape createItemShape(ChartOrientation orientation, AbstractValueAxis valueAxis,
                                        int row, int column, Rectangle2D rect, Paint autoFillPaint,
                                        Paint autoBorderPaint, Stroke autoBorderStroke,
                                        float alpha, Object appendParam) {
    if (orientation == null) {
      throw new IllegalArgumentException("Null 'orientation' argument.") ;
    }

    if (!isItemVisible(row, column)) {
      return null ;
    }

    String label = null ;
    ILabelGenerator labelGen = getLabelGenerator() ;
    if (labelGen != null && isItemLabelVisible(row, column)) {
      if (labelGen.isGroup()) {
        int index = 0 ;
        if (appendParam != null && (appendParam instanceof Number)) {
          index = ((Number) appendParam).intValue() ;
        }
        label = labelGen.generateGroupLabel(getDataset(), row, column, index) ;
      } else {
        label = labelGen.generateLabel(getDataset(), row, column) ;
      }
    }

    IChartShape shape = createItemShapeOnly(rect, label, appendParam) ;
    shape.setFillAlpha(alpha) ;
    Paint fillPaint = getItemFillPaintHelper(autoFillPaint, row, column) ;
    if (fillPaint != null) {
      shape.setFillPaint(fillPaint) ;
    }

    shape.setFillShape(isItemFill(row, column)) ;
    shape.setGradientPaintTransformType(getItemFillPaintTransformer(row, column)) ;
    shape.setDrawBorder(isItemDrawBorder(row, column)) ;

    if (isAutoSelectBorderPaint() && autoBorderPaint != null) {
      shape.setBorderPaint(autoBorderPaint) ;
    } else {
      shape.setBorderPaint(getItemBorderPaint(row, column)) ;
    }
    if (isAutoSelectBorderStroke() && autoBorderStroke != null) {
      shape.setBorderStroke(autoBorderStroke) ;
    } else {
      shape.setBorderStroke(getItemBorderStroke(row, column)) ;
    }

    if (shape instanceof AbstractRectangleTextChartShape) {
      AbstractRectangleTextChartShape textShape = (AbstractRectangleTextChartShape) shape ;
      if (StringUtil.isEmpty(textShape.getText())) {
        textShape.setText(label) ;
      }

      TextPosition position = getItemLabelPosition(row, column) ;
      if (valueAxis.isInverted()) {
        position = position.getOppositePosition(orientation) ;
      }
      textShape.setTextPosition(position) ;
      textShape.setTextDirection(getItemLabelDirection(row, column)) ;
      textShape.setDrawText(isItemLabelVisible(row, column)) ;
      textShape.setTextFont(getItemLabelFont(row, column)) ;
      textShape.setTextPaint(getItemLabelPaint(row, column)) ;

      if (orientation.equals(ChartOrientation.HORIZONTAL)) { //ˮƽ뷽ʽ
        if (valueAxis != null && valueAxis.isInverted()) {
          HorizontalAlignment align = getItemLabelHorizontalAlignment(row, column) ;
          align = HorizontalAlignment.getOpposite(align) ;
          textShape.setTextHorizontalAlign(align) ;
        } else {
          textShape.setTextHorizontalAlign(getItemLabelHorizontalAlignment(row, column)) ;
        }
      } else {
        textShape.setTextHorizontalAlign(getItemLabelHorizontalAlignment(row, column)) ;
      }

      if (orientation.equals(ChartOrientation.VERTICAL)) { //ôֱ뷽ʽ
        if (valueAxis != null && valueAxis.isInverted()) {
          VerticalAlignment align = getItemLabelVerticalAlignment(row, column) ;
          align = VerticalAlignment.getOpposite(align) ;
          textShape.setTextVerticalAlign(align) ;
        } else {
          textShape.setTextVerticalAlign(getItemLabelVerticalAlignment(row, column)) ;
        }
      } else {
        textShape.setTextVerticalAlign(getItemLabelVerticalAlignment(row, column)) ;
      }

      textShape.setForceInsideTextNoWrap(true) ; //ǿƲ
    }

    IToolTipGenerator tooltipGen = getToolTipGenerator() ;
    if (tooltipGen != null && isGeneratorTooltip()) {
      String tooltip = null ;
      if (tooltipGen.isGroup()) {
        int index = 0 ;
        if (appendParam != null && (appendParam instanceof Number)) {
          index = ((Number) appendParam).intValue() ;
        }
        tooltip = tooltipGen.generateGroupToolTip(getDataset(), row, column, index) ;
      } else {
        tooltip = tooltipGen.generateToolTip(getDataset(), row, column) ;
      }
      if (!StringUtil.isEmptyWithTrim(tooltip)) {
        shape.setTooltipText(tooltip) ;
      }
    }

    if (shape != null) {
      shape.setData(getDataset().getData(row, column)) ;
    }

    return shape ;
  }

  /**
   * <zh>value marker.</zh>
   * <en>Draws a marker to indicate a particular value on the value axis.</en>
   *
   * @param valueAxis AbstractValueAxis <zh>value (Ϊnull).</zh>
   * <en>The value axis (null not permitted).</en>
   * @param yAxis boolean <zh>Ƿy.</zh>
   * <en>Whether or not is y axis.</en>
   * @param axisEdge RectangleEdge <zh>λ (Ϊnull).</zh>
   * <en>The axis position (null not permitted).</en>
   * @param dataArea Rectangle2D <zh> (Ϊnull).</zh>
   * <en>The chart data area (null not permitted).</en>
   * @param orientation ChartOrientation <zh>ͼķ (Ϊnull).</zh>
   * <en>The chart orientation (null not permitted).</en>
   * @param renderShapes RenderShapeCollection <zh>renderͼμ (Ϊnull).</zh>
   * <en>Collects information about the render (null not permitted).</en>
   */
  public void drawValueMarker(AbstractValueAxis valueAxis, boolean yAxis, RectangleEdge axisEdge,
                              Rectangle2D dataArea, ChartOrientation orientation,
                              RenderShapeCollection renderShapes) {
    if (valueAxis == null) {
      throw new IllegalArgumentException("Null 'valueAxis' argument.") ;
    }
    if (axisEdge == null) {
      throw new IllegalArgumentException("Null 'axisEdge' argument.") ;
    }
    if (dataArea == null) {
      throw new IllegalArgumentException("Null 'dataArea' argument.") ;
    }
    if (orientation == null) {
      throw new IllegalArgumentException("Null 'orientation' argument.") ;
    }
    if (renderShapes == null) {
      throw new IllegalArgumentException("Null 'renderShapes' argument.") ;
    }

    Collection foregroundMarkers = valueAxis.getValueMarkers(Layer.FOREGROUND) ;
    Collection backgroundMarkers = valueAxis.getValueMarkers(Layer.BACKGROUND) ;

    if (orientation.equals(ChartOrientation.VERTICAL)) { //ֱ
      Iterator iter = foregroundMarkers.iterator() ;
      while (iter.hasNext()) {
        Object next = iter.next() ;
        if (next instanceof ValueMarker) {
          ValueMarker marker = (ValueMarker) next ;
          double position = valueAxis.valueToJava2D(dataArea, axisEdge, marker.getValue(), false) ;
          if (position == AbstractValueAxis.INVALID_COORDINATE_VALUE) {
            continue ;
          }

          VerticalAlignment vAlign = marker.getTextVerticalAlign() ;
          if (vAlign.equals(VerticalAlignment.TOP) || vAlign.equals(VerticalAlignment.CENTER)) { //ֱ
            IChartShape shape = null ;
            if (yAxis) {
              Rectangle2D rect = new Rectangle2D.Double(dataArea.getX(), dataArea.getY(),
                                                        dataArea.getWidth(),
                                                        Math.abs(position - dataArea.getY())) ;
              shape = createValueMarkerShape(marker, rect, marker.getTextHorizontalAlign(),
                                             VerticalAlignment.BOTTOM, false, false, false, true) ;
            } else {
              Rectangle2D rect = new Rectangle2D.Double(dataArea.getX(), dataArea.getY(),
                                                        Math.abs(position - dataArea.getX()),
                                                        dataArea.getHeight()) ;
              shape = createValueMarkerShape(marker, rect, HorizontalAlignment.RIGHT,
                                             marker.getTextVerticalAlign(),
                                             false, true, false, false) ;
            }
            renderShapes.addForegroundMarkerShape(shape) ;
          } else { //ֱ
            IChartShape shape = null ;
            if (yAxis) {
              Rectangle2D rect = new Rectangle2D.Double(dataArea.getX(), position,
                                                        dataArea.getWidth(),
                                                        Math.abs(dataArea.getMaxY() - position)) ;
              shape = createValueMarkerShape(marker, rect, marker.getTextHorizontalAlign(),
                                             VerticalAlignment.TOP, false, false, true, false) ;
            } else {
              Rectangle2D rect = new Rectangle2D.Double(position, dataArea.getY(),
                                                        Math.abs(dataArea.getMaxX() - position),
                                                        dataArea.getHeight()) ;
              shape = createValueMarkerShape(marker, rect, HorizontalAlignment.LEFT,
                                             marker.getTextVerticalAlign(),
                                             true, false, false, false) ;
            }
            renderShapes.addForegroundMarkerShape(shape) ;
          }
        }
      }

      iter = backgroundMarkers.iterator() ;
      while (iter.hasNext()) {
        Object next = iter.next() ;
        if (next instanceof ValueMarker) {
          ValueMarker marker = (ValueMarker) next ;
          double position = valueAxis.valueToJava2D(dataArea, axisEdge, marker.getValue(), false) ;
          if (position == AbstractValueAxis.INVALID_COORDINATE_VALUE) {
            continue ;
          }

          VerticalAlignment vAlign = marker.getTextVerticalAlign() ;
          if (vAlign.equals(VerticalAlignment.TOP) || vAlign.equals(VerticalAlignment.CENTER)) { //ֱ
            IChartShape shape = null ;
            if (yAxis) {
              Rectangle2D rect = new Rectangle2D.Double(dataArea.getX(), dataArea.getY(),
                                                        dataArea.getWidth(),
                                                        Math.abs(position - dataArea.getY())) ;
              shape = createValueMarkerShape(marker, rect, marker.getTextHorizontalAlign(),
                                             VerticalAlignment.BOTTOM, false, false, false, true) ;
            } else {
              Rectangle2D rect = new Rectangle2D.Double(dataArea.getX(), dataArea.getY(),
                                                        Math.abs(position - dataArea.getX()),
                                                        dataArea.getHeight()) ;
              shape = createValueMarkerShape(marker, rect, HorizontalAlignment.RIGHT,
                                             marker.getTextVerticalAlign(),
                                             false, true, false, false) ;
            }
            renderShapes.addBackgroundMarkerShape(shape) ;
          } else { //ֱ
            IChartShape shape = null ;
            if (yAxis) {
              Rectangle2D rect = new Rectangle2D.Double(dataArea.getX(), position,
                                                        dataArea.getWidth(),
                                                        Math.abs(dataArea.getMaxY() - position)) ;
              shape = createValueMarkerShape(marker, rect, marker.getTextHorizontalAlign(),
                                             VerticalAlignment.TOP, false, false, true, false) ;
            } else {
              Rectangle2D rect = new Rectangle2D.Double(position, dataArea.getY(),
                                                        Math.abs(dataArea.getMaxX() - position),
                                                        dataArea.getHeight()) ;
              shape = createValueMarkerShape(marker, rect, HorizontalAlignment.LEFT,
                                             marker.getTextVerticalAlign(),
                                             true, false, false, false) ;
            }
            renderShapes.addBackgroundMarkerShape(shape) ;
          }
        }
      }

    } else if (orientation.equals(ChartOrientation.HORIZONTAL)) { //ˮƽ
      Iterator iter = foregroundMarkers.iterator() ;
      while (iter.hasNext()) {
        Object next = iter.next() ;
        if (next instanceof ValueMarker) {
          ValueMarker marker = (ValueMarker) next ;
          double position = valueAxis.valueToJava2D(dataArea, axisEdge, marker.getValue(), false) ;
          if (position == AbstractValueAxis.INVALID_COORDINATE_VALUE) {
            continue ;
          }

          HorizontalAlignment hAlign = marker.getTextHorizontalAlign() ;
          if (hAlign.equals(HorizontalAlignment.LEFT) || hAlign.equals(HorizontalAlignment.CENTER)) { //ֱ
            IChartShape shape = null ;
            if (yAxis) {
              Rectangle2D rect = new Rectangle2D.Double(dataArea.getX(), dataArea.getY(),
                                                        Math.abs(position - dataArea.getX()),
                                                        dataArea.getHeight()) ;
              shape = createValueMarkerShape(marker, rect, HorizontalAlignment.RIGHT,
                                             marker.getTextVerticalAlign(),
                                             false, true, false, false) ;
            } else {
              Rectangle2D rect = new Rectangle2D.Double(dataArea.getX(), dataArea.getY(),
                                                        dataArea.getWidth(),
                                                        Math.abs(position - dataArea.getY())) ;
              shape = createValueMarkerShape(marker, rect, marker.getTextHorizontalAlign(),
                                             VerticalAlignment.BOTTOM, false, false, false, true) ;
            }
            renderShapes.addForegroundMarkerShape(shape) ;
          } else { //ֱұ
            IChartShape shape = null ;
            if (yAxis) {
              Rectangle2D rect = new Rectangle2D.Double(position, dataArea.getY(),
                                                        Math.abs(dataArea.getMaxX() - position),
                                                        dataArea.getHeight()) ;
              shape = createValueMarkerShape(marker, rect, HorizontalAlignment.LEFT,
                                             marker.getTextVerticalAlign(),
                                             true, false, false, false) ;
            } else {
              Rectangle2D rect = new Rectangle2D.Double(dataArea.getX(), position,
                                                        dataArea.getWidth(),
                                                        Math.abs(dataArea.getMaxY() - position)) ;
              shape = createValueMarkerShape(marker, rect, HorizontalAlignment.LEFT,
                                             VerticalAlignment.TOP, false, false, true, false) ;
            }
            renderShapes.addForegroundMarkerShape(shape) ;
          }
        }
      }

      iter = backgroundMarkers.iterator() ;
      while (iter.hasNext()) {
        Object next = iter.next() ;
        if (next instanceof ValueMarker) {
          ValueMarker marker = (ValueMarker) next ;

          double position = valueAxis.valueToJava2D(dataArea, axisEdge, marker.getValue(), false) ;
          if (position == AbstractValueAxis.INVALID_COORDINATE_VALUE) {
            continue ;
          }

          HorizontalAlignment hAlign = marker.getTextHorizontalAlign() ;
          if (hAlign.equals(HorizontalAlignment.LEFT) || hAlign.equals(HorizontalAlignment.CENTER)) { //ֱ
            IChartShape shape = null ;
            if (yAxis) {
              Rectangle2D rect = new Rectangle2D.Double(dataArea.getX(), dataArea.getY(),
                                                        Math.abs(position - dataArea.getX()),
                                                        dataArea.getHeight()) ;
              shape = createValueMarkerShape(marker, rect, HorizontalAlignment.RIGHT,
                                             marker.getTextVerticalAlign(),
                                             false, true, false, false) ;
            } else {
              Rectangle2D rect = new Rectangle2D.Double(dataArea.getX(), dataArea.getY(),
                                                        dataArea.getWidth(),
                                                        Math.abs(position - dataArea.getY())) ;
              shape = createValueMarkerShape(marker, rect, marker.getTextHorizontalAlign(),
                                             VerticalAlignment.BOTTOM, false, false, false, true) ;
            }
            renderShapes.addBackgroundMarkerShape(shape) ;
          } else { //ֱұ
            IChartShape shape = null ;
            if (yAxis) {
              Rectangle2D rect = new Rectangle2D.Double(position, dataArea.getY(),
                                                        Math.abs(dataArea.getMaxX() - position),
                                                        dataArea.getHeight()) ;
              shape = createValueMarkerShape(marker, rect, HorizontalAlignment.LEFT,
                                             marker.getTextVerticalAlign(),
                                             true, false, false, false) ;
            } else {
              Rectangle2D rect = new Rectangle2D.Double(dataArea.getX(), position,
                                                        dataArea.getWidth(),
                                                        Math.abs(dataArea.getMaxY() - position)) ;
              shape = createValueMarkerShape(marker, rect, HorizontalAlignment.LEFT,
                                             VerticalAlignment.TOP, false, false, true, false) ;
            }
            renderShapes.addBackgroundMarkerShape(shape) ;
          }

        }
      }
    }

  }

  /**
   * <zh>value marker.</zh>
   * <en>Draws a marker to indicate a particular value on the value axis.</en>
   *
   * @param valueAxis AbstractValueAxis <zh>value (Ϊnull).</zh>
   * <en>The value axis (null not permitted).</en>
   * @param yAxis boolean <zh>Ƿy.</zh>
   * <en>Whether or not is y axis.</en>
   * @param axisEdge RectangleEdge <zh>λ (Ϊnull).</zh>
   * <en>The axis position (null not permitted).</en>
   * @param dataArea Rectangle2D <zh> (Ϊnull).</zh>
   * <en>The chart data area (null not permitted).</en>
   * @param orientation ChartOrientation <zh>ͼķ (Ϊnull).</zh>
   * <en>The chart orientation (null not permitted).</en>
   * @param renderShapes RenderShapeCollection <zh>renderͼμ (Ϊnull).</zh>
   * <en>Collects information about the render (null not permitted).</en>
   */
  public void drawIntervalValueMarker(AbstractValueAxis valueAxis, boolean yAxis,
                                      RectangleEdge axisEdge, Rectangle2D dataArea,
                                      ChartOrientation orientation,
                                      RenderShapeCollection renderShapes) {
    if (valueAxis == null) {
      throw new IllegalArgumentException("Null 'valueAxis' argument.") ;
    }
    if (axisEdge == null) {
      throw new IllegalArgumentException("Null 'axisEdge' argument.") ;
    }
    if (dataArea == null) {
      throw new IllegalArgumentException("Null 'dataArea' argument.") ;
    }
    if (orientation == null) {
      throw new IllegalArgumentException("Null 'orientation' argument.") ;
    }
    if (renderShapes == null) {
      throw new IllegalArgumentException("Null 'renderShapes' argument.") ;
    }

    Collection foregroundMarkers = valueAxis.getIntervalValueMarkers(Layer.FOREGROUND) ;
    Collection backgroundMarkers = valueAxis.getIntervalValueMarkers(Layer.BACKGROUND) ;

    if (orientation.equals(ChartOrientation.VERTICAL)) { //ֱ
      Iterator iter = foregroundMarkers.iterator() ;
      while (iter.hasNext()) {
        Object next = iter.next() ;
        if (next instanceof IntervalValueMarker) {
          IntervalValueMarker marker = (IntervalValueMarker) next ;
          double startPosition = valueAxis.valueToJava2D(dataArea, axisEdge,
                                                         marker.getStartValue(), false) ;
          double endPosition = valueAxis.valueToJava2D(dataArea, axisEdge, marker.getEndValue(), false) ;
          if (startPosition == AbstractValueAxis.INVALID_COORDINATE_VALUE
              || endPosition == AbstractValueAxis.INVALID_COORDINATE_VALUE) {
            continue ;
          }

          IChartShape shape = null ;
          if (yAxis) {
            Rectangle2D rect = new Rectangle2D.Double(dataArea.getX(),
                                                      Math.min(startPosition, endPosition),
                                                      dataArea.getWidth(),
                                                      Math.abs(startPosition - endPosition)) ;
            shape = createIntervalValueMarkerShape(marker, rect, false, false, true, true) ;
          } else {
            Rectangle2D rect = new Rectangle2D.Double(Math.min(startPosition, endPosition),
                                                      dataArea.getY(),
                                                      Math.abs(startPosition - endPosition),
                                                      dataArea.getHeight()) ;
            shape = createIntervalValueMarkerShape(marker, rect, true, true, false, false) ;
          }
          renderShapes.addForegroundIntervalMarkerShape(shape) ;
        }
      }

      iter = backgroundMarkers.iterator() ;
      while (iter.hasNext()) {
        Object next = iter.next() ;
        if (next instanceof IntervalValueMarker) {
          IntervalValueMarker marker = (IntervalValueMarker) next ;
          double startPosition = valueAxis.valueToJava2D(dataArea, axisEdge,
                                                         marker.getStartValue(), false) ;
          double endPosition = valueAxis.valueToJava2D(dataArea, axisEdge, marker.getEndValue(), false) ;
          if (startPosition == AbstractValueAxis.INVALID_COORDINATE_VALUE
              || endPosition == AbstractValueAxis.INVALID_COORDINATE_VALUE) {
            continue ;
          }

          IChartShape shape = null ;
          if (yAxis) {
            Rectangle2D rect = new Rectangle2D.Double(dataArea.getX(),
                                                      Math.min(startPosition, endPosition),
                                                      dataArea.getWidth(),
                                                      Math.abs(startPosition - endPosition)) ;
            shape = createIntervalValueMarkerShape(marker, rect, false, false, true, true) ;
          } else {
            Rectangle2D rect = new Rectangle2D.Double(Math.min(startPosition, endPosition),
                                                      dataArea.getY(),
                                                      Math.abs(startPosition - endPosition),
                                                      dataArea.getHeight()) ;
            shape = createIntervalValueMarkerShape(marker, rect, true, true, false, false) ;

          }
          renderShapes.addBackgroundIntervalMarkerShape(shape) ;
        }
      }
    } else if (orientation.equals(ChartOrientation.HORIZONTAL)) { //ˮƽ
      Iterator iter = foregroundMarkers.iterator() ;
      while (iter.hasNext()) {
        Object next = iter.next() ;
        if (next instanceof IntervalValueMarker) {
          IntervalValueMarker marker = (IntervalValueMarker) next ;

          double startPosition = valueAxis.valueToJava2D(dataArea, axisEdge,
                                                         marker.getStartValue(), false) ;
          double endPosition = valueAxis.valueToJava2D(dataArea, axisEdge, marker.getEndValue(), false) ;
          if (startPosition == AbstractValueAxis.INVALID_COORDINATE_VALUE
              || endPosition == AbstractValueAxis.INVALID_COORDINATE_VALUE) {
            continue ;
          }

          IChartShape shape = null ;
          if (yAxis) {
            Rectangle2D rect = new Rectangle2D.Double(Math.min(startPosition, endPosition),
                                                      dataArea.getY(),
                                                      Math.abs(startPosition - endPosition),
                                                      dataArea.getHeight()) ;
            shape = createIntervalValueMarkerShape(marker, rect, true, true, false, false) ;
          } else {
            Rectangle2D rect = new Rectangle2D.Double(dataArea.getX(),
                                                      Math.min(startPosition, endPosition),
                                                      dataArea.getWidth(),
                                                      Math.abs(startPosition - endPosition)) ;
            shape = createIntervalValueMarkerShape(marker, rect, false, false, true, true) ;
          }
          renderShapes.addForegroundIntervalMarkerShape(shape) ;
        }
      }

      iter = backgroundMarkers.iterator() ;
      while (iter.hasNext()) {
        Object next = iter.next() ;
        if (next instanceof IntervalValueMarker) {
          IntervalValueMarker marker = (IntervalValueMarker) next ;
          double startPosition = valueAxis.valueToJava2D(dataArea, axisEdge,
                                                         marker.getStartValue(), false) ;
          double endPosition = valueAxis.valueToJava2D(dataArea, axisEdge, marker.getEndValue(), false) ;
          if (startPosition == AbstractValueAxis.INVALID_COORDINATE_VALUE
              || endPosition == AbstractValueAxis.INVALID_COORDINATE_VALUE) {
            continue ;
          }

          IChartShape shape = null ;
          if (yAxis) {
            Rectangle2D rect = new Rectangle2D.Double(Math.min(startPosition, endPosition),
                                                      dataArea.getY(),
                                                      Math.abs(startPosition - endPosition),
                                                      dataArea.getHeight()) ;
            shape = createIntervalValueMarkerShape(marker, rect, true, true, false, false) ;
          } else {
            Rectangle2D rect = new Rectangle2D.Double(dataArea.getX(),
                                                      Math.min(startPosition, endPosition),
                                                      dataArea.getWidth(),
                                                      Math.abs(startPosition - endPosition)) ;
            shape = createIntervalValueMarkerShape(marker, rect, false, false, true, true) ;
          }
          renderShapes.addBackgroundIntervalMarkerShape(shape) ;
        }
      }

    }

  }

  /**
   * <zh>value markerͼ.</zh>
   * <en>Create a value marker shape.</en>
   *
   * @param marker ValueMarker
   * @param rect Rectangle2D
   * @param hAlign HorizontalAlignment
   * @param vAlign VerticalAlignment
   * @param leftLineVisible boolean
   * @param rightLineVisible boolean
   * @param topLineVisible boolean
   * @param bottomLineVisible boolean
   *
   * @return IChartShape
   */
  protected static IChartShape createValueMarkerShape(ValueMarker marker, Rectangle2D rect,
                                                      HorizontalAlignment hAlign,
                                                      VerticalAlignment vAlign,
                                                      boolean leftLineVisible,
                                                      boolean rightLineVisible,
                                                      boolean topLineVisible,
                                                      boolean bottomLineVisible) {
    RectangleTextChartShape shape = new RectangleTextChartShape(rect, marker.getText()) ;

    shape.setFillShape(false) ;
    shape.setDrawBorder(true) ;
    shape.setTextFont(marker.getTextFont()) ;
    shape.setTextPaint(marker.getTextPaint()) ;
    shape.setDrawText(true) ;
    shape.setMargin(marker.getMargin()) ;
    shape.setTextHorizontalAlign(hAlign) ;
    shape.setTextVerticalAlign(vAlign) ;
    shape.setTextPosition(marker.getTextPosition()) ;

    shape.setBorderVisible(RectangleEdge.LEFT, leftLineVisible) ;
    shape.setBorderVisible(RectangleEdge.RIGHT, rightLineVisible) ;
    shape.setBorderVisible(RectangleEdge.TOP, topLineVisible) ;
    shape.setBorderVisible(RectangleEdge.BOTTOM, bottomLineVisible) ;

    shape.setBorderPaint(marker.getBorderPaint()) ;
    shape.setBorderStroke(marker.getBorderStroke()) ;
    shape.setFillPaint(marker.getFillPaint()) ;
    shape.setFillAlpha(marker.getFillAlpha()) ;

    return shape ;
  }

  /**
   * <zh>value markerͼ.</zh>
   * <en>Create a value marker shape.</en>
   *
   * @param marker CategoryMarker
   * @param rect Rectangle2D
   * @param leftLineVisible boolean
   * @param rightLineVisible boolean
   * @param topLineVisible boolean
   * @param bottomLineVisible boolean
   *
   * @return IChartShape
   */
  private static IChartShape createIntervalValueMarkerShape(IntervalValueMarker marker,
                                                            Rectangle2D rect,
                                                            boolean leftLineVisible,
                                                            boolean rightLineVisible,
                                                            boolean topLineVisible,
                                                            boolean bottomLineVisible) {
    RectangleTextChartShape shape = new RectangleTextChartShape(rect, marker.getText()) ;

    shape.setFillShape(marker.isFillShape()) ;
    shape.setFillPaint(marker.getFillPaint()) ;
    shape.setFillAlpha(marker.getFillAlpha()) ;

    shape.setDrawBorder(true) ;
    shape.setTextFont(marker.getTextFont()) ;
    shape.setTextPaint(marker.getTextPaint()) ;
    shape.setDrawText(marker.isDrawText()) ;
    shape.setMargin(marker.getMargin()) ;

    shape.setTextHorizontalAlign(marker.getTextHorizontalAlign()) ;
    shape.setTextVerticalAlign(marker.getTextVerticalAlign()) ;
    shape.setTextPosition(marker.getTextPosition()) ;

    shape.setBorderVisible(RectangleEdge.LEFT, leftLineVisible) ;
    shape.setBorderVisible(RectangleEdge.RIGHT, rightLineVisible) ;
    shape.setBorderVisible(RectangleEdge.TOP, topLineVisible) ;
    shape.setBorderVisible(RectangleEdge.BOTTOM, bottomLineVisible) ;

    shape.setBorderPaint(marker.getBorderPaint()) ;
    shape.setBorderStroke(marker.getBorderStroke()) ;

    return shape ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Indicates if some other object is "equal to" this one.</en>
   *
   * @param obj <zh>ҪȽϵĶ</zh>
   * <en>the reference object with which to compare.</en>
   * @return <zh>ȷtrue,򷵻false.</zh>
   * <en>true if this object is the same as the obj argument; false otherwise.</en>
   */
  public boolean equals(Object obj) {
    if (this == obj) {
      return true ;
    }

    if (!(obj instanceof AbstractWithAxisRender)) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return int <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    return super.hashCode() ;
  }

  /**
   * <zh>ؿ¡Ķ.</zh>
   * <en>Creates and returns a copy of this object.</en>
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the Cloneable interface. Subclasses
   *               that override the clone method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object.</en>
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "AbstractRenderWithAxis" ;
  }
}
