package com.ijchart.xychart.render.noaxis ;

import java.awt.BasicStroke ;
import java.awt.Color ;
import java.awt.GradientPaint ;
import java.awt.Paint ;
import java.awt.Polygon ;
import java.awt.Shape ;
import java.awt.Stroke ;
import java.awt.geom.AffineTransform ;
import java.awt.geom.Arc2D ;
import java.awt.geom.Area ;
import java.awt.geom.GeneralPath ;
import java.awt.geom.Point2D ;
import java.awt.geom.Rectangle2D ;
import java.io.IOException ;
import java.io.ObjectInputStream ;
import java.io.ObjectOutputStream ;
import java.io.Serializable ;
import java.util.Enumeration ;
import java.util.HashMap ;
import java.util.Hashtable ;
import java.util.Iterator ;
import java.util.Map ;

import com.ijchart.xychart.chartinfo.RenderShapeCollection ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.constant.Rotation ;
import com.ijchart.xychart.data.IChartData ;
import com.ijchart.xychart.data.NumberChartData ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.dataset.SingleRowDataset ;
import com.ijchart.xychart.label.DefaultPieChartLabelGenerator ;
import com.ijchart.xychart.label.ILabelGenerator ;
import com.ijchart.xychart.render.INoAxisRender ;
import com.ijchart.xychart.render.noaxis.pie.PieLabelDistributor ;
import com.ijchart.xychart.render.noaxis.pie.PieLabelRecord ;
import com.ijchart.xychart.shape.EllipseTextChartShape ;
import com.ijchart.xychart.shape.GeneralPathChartShape ;
import com.ijchart.xychart.shape.IChartShape ;
import com.ijchart.xychart.shape.PieOutsideLabelChartShape ;
import com.ijchart.xychart.shape.RingChartShape ;
import com.ijchart.xychart.shape.SectorChartShape ;
import com.ijchart.xychart.ui.RectangleInsets ;
import com.ijchart.xychart.util.DatasetUtil ;
import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.GeometryUtil ;
import com.ijchart.xychart.util.HashcodeUtil ;
import com.ijchart.xychart.util.SerialUtil ;
import com.ijchart.xychart.util.StringUtil ;
import com.ijchart.xychart.util.TextUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>Ʊͼ.</zh>
 * <en>Draw pie chart shapes.</en>
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class PieRender extends AbstractNoAxisRender implements INoAxisRender, Cloneable,
  Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = -8256706621708900226L ;

  /**
   * <zh>ȱʡڲĿհ(ٷֱ).</zh>
   * <en>The default interior gap.</en>
   */
  public static final double DEFAULT_INTERIOR_GAP = 0.1 ;

  /**
   * <zh>ȱʡǷǿƱͼΪԲ.</zh>
   * <en>The default flag determining whether to draw an ellipse or a perfect circle.</en>
   */
  public static final boolean DEFAULT_IS_CIRCLE = false ;

  /**
   * <zh>ڲհ״С(ٷֱ).</zh>
   * <en>The maximum interior gap.</en>
   */
  public static final double MAX_INTERIOR_GAP = 0.40 ;

  /**
   * <zh>ȱʡĿʼεĽǶ.</zh>
   * <en>The default starting angle for the pie chart.</en>
   */
  public static final double DEFAULT_START_ANGLE = 90.0 ;

  /**
   * <zh>ȱʡĻƱͼķ.</zh>
   * <en>The default direction for the pie segments.</en>
   */
  public static final Rotation DEFAULT_PIE_ROTATION = Rotation.CLOCKWISE ;

  /**
   * <zh>ȱʡǷεı߿.</zh>
   * <en>A default flag that controls whether or not an outline is drawn for each
   * section in the render area.</en>
   */
  public static final boolean DEFAULT_SECTION_OUTLINES_VISIBLE = true ;

  /**
   * <zh>ȱʡxӰ.</zh>
   * <en>The default x-offset for the shadow effect.</en>
   */
  public static final double DEFAULT_SHADOW_X_OFFSET = 2F ;

  /**
   * <zh>ȱʡyӰ.</zh>
   * <en>The default y-offset for the shadow effect.</en>
   */
  public static final double DEFAULT_SHADOW_Y_OFFSET = 2F ;

  /**
   * <zh>ȱʡıǩıpaint.</zh>
   * <en>The default section label background paint.</en>
   */
  public static final Paint DEFAULT_LABEL_BACKGROUND_PAINT = new Color(255, 255, 192) ;

  /**
   * <zh>Ʊǩı߿paint.</zh>
   * <en>The default section label outline paint.</en>
   */
  public static final Paint DEFAULT_LABEL_OUTLINE_PAINT = Color.black ;

  /**
   * <zh>ȱʡĻƱ߿stroke.</zh>
   * <en>The default section label outline stroke.</en>
   */
  public static final Stroke DEFAULT_LABEL_OUTLINE_STROKE = new BasicStroke(0.5f) ;

  /**
   * <zh>ȱʡĻƱǩӰpaint.</zh>
   * <en>The default section label shadow paint.</en>
   */
  public static final Paint DEFAULT_LABEL_SHADOW_PAINT = new Color(151, 151, 151, 128) ;

  /**
   * <zh>Сλƻ.</zh>
   * <en>The default minimum arc angle to draw.</en>
   */
  public static final double DEFAULT_MINIMUM_ARC_ANGLE_TO_DRAW = 0.00001 ;

  /**
   * <zh>ȱʡĻƱͼӰpaint.</zh>
   * <en>The default pie shadow paint.</en>
   */
  public static final Paint DEFAULT_PIE_SHADOW_PAINT = Color.gray ;

  /**
   * <zh>ȱʡǩǷڱͼڲ.</zh>
   * <en>A default flag that controls whether or not label in pie.</en>
   */
  public static final boolean DEFAULT_LABEL_IN_PIE = true ;

  /**
   * <zh>ȱʡǩĲ.</zh>
   * <en>The default padding between the labels and the label outlines.</en>
   */
  public static final RectangleInsets DEFAULT_LABEL_PADDING = new RectangleInsets(2, 2, 2, 2) ;

  /**
   * <zh>ȱʡıǩ(İٷֱ).</zh>
   * <en>The default maximum label width as a percentage of the draw area width.</en>
   */
  public static final double DEFAULT_MAXIMUM_LABEL_WIDTH = 0.20 ;

  /**
   * <zh>ȱʡıǩrender߿ľ(ٷֱ).</zh>
   * <en>The default gap between the labels and the render as a
   *  percentage of the render area width.</en>
   */
  public static final double DEFAULT_LABEL_GAP = 0.02 ;

  /**
   * <zh>ȱʡǷʾǩͱͼ֮.</zh>
   * <en>A default flag that controls whether or not the label links are drawn.</en>
   */
  public static final boolean DEFAULT_LABEL_LINKS_VISIBLE = true ;

  /**
   * <zh>ȱʡߵĿհ.</zh>
   * <en>The default link margin.</en>
   */
  public static final double DEFAULT_LABEL_LINK_MARGIN = 0.05 ;

  /**
   * <zh>ȱʡߵpaint.</zh>
   * <en>The default paint used for the label linking lines.</en>
   */
  public static final Paint DEFAULT_LABEL_LINK_PAINT = Color.black ;

  /**
   * <zh>ȱʡߵstroke.</zh>
   * <en>The default stroke used for the label linking lines.</en>
   */
  public static final Stroke DEFAULT_LABEL_LINK_STROKE = new BasicStroke(0.5F) ;

  /**
   * <zh>ȱʡĻlabel,ڱͼʾʾ.</zh>
   * <en>A default flag that controls whether simple or extended labels are used.</en>
   */
  public static final boolean DEFAULT_SIMPLE_LABELS = false ;

  /**
   * <zh>ȱʡǷ3D͵ıͼ.</zh>
   * <en>A default flag that controls whether or not the pie is 3D.</en>
   */
  public static final boolean DEFAULT_PIE_IS_3D = false ;

  /**
   * <zh>ȱʡĻ3DЧ.</zh>
   * <en>The default depth of 3D pie.</en>
   */
  public static final double DEFAULT_DEPTH_3D = 8.0 ;

  /**
   * <zh>ǷʾΪԲͼȱʡֵ.</zh>
   * <en>A default flag that controls whether or not the shape is ring.</en>
   */
  public static final boolean DEFAULT_DRAW_AS_RING = false ;

  /**
   * <zh>ڲĿհ(ٷֱ).</zh>
   * <en>The interior gap.</en>
   */
  private double interiorGap ;

  /**
   * <zh>ǷǿƱͼΪԲ.</zh>
   * <en>Flag determining whether to draw an ellipse or a perfect circle.</en>
   */
  private boolean circular ;

  /**
   * <zh>ʼεĽǶ.</zh>
   * <en>The starting angle.</en>
   */
  private double startAngle ;

  /**
   * <zh>Ʊͼķ.</zh>
   * <en>The direction for the pie segments.</en>
   */
  private Rotation pieRotation ;

  /**
   * <zh>Ƿεı߿.</zh>
   * <en>A flag that controls whether or not an outline is drawn for each
   * section in the render area.</en>
   */
  private boolean sectionOutlinesVisible ;

  /**
   * <zh>ƱͼӰpaint.</zh>
   * <en>The pie shadow paint.</en>
   */
  private transient Paint pieShadowPaint ;

  /**
   * <zh>xӰ.</zh>
   * <en>The x-offset for the shadow effect.</en>
   */
  private double shadowXOffset ;

  /**
   * <zh>yӰ.</zh>
   * <en>The y-offset for the shadow effect.</en>
   */
  private double shadowYOffset ;

  /**
   * <zh>洢ЩӦƫԲļƫС(Բ뾶С).</zh>
   * <en>The percentage amount to explode each pie section.</en>
   */
  private Map explodePercentages ;

  /**
   * <zh>ǩıɫ.</zh>
   * <en>The color used to draw the background of the section labels.</en>
   */
  private transient Paint labelBackgroundPaint ;

  /**
   * <zh>Ʊǩı߿paint.</zh>
   * <en>The paint used to draw the outline of the section labels.</en>
   */
  private transient Paint labelOutlinePaint ;

  /**
   * <zh>Ʊ߿stroke.</zh>
   * <en>The stroke used to draw the outline of the section labels.</en>
   */
  private transient Stroke labelOutlineStroke ;

  /**
   * <zh>ƱǩӰpaint.</zh>
   * <en> The paint used to draw the shadow for the section labels.</en>
   */
  private transient Paint labelShadowPaint ;

  /**
   * <zh>ǩǷڱͼڲ.</zh>
   * <en>A flag that controls whether or not label in pie.</en>
   */
  private boolean labelInPie ;

  /**
   * <zh>ǩĲ.</zh>
   * <en>The padding between the labels and the label outlines.</en>
   */
  private RectangleInsets labelPadding ;

  /**
   * <zh>ǩ(İٷֱ).</zh>
   * <en>The maximum label width as a percentage of the draw area width.</en>
   */
  private double maximumLabelWidth ;

  /**
   * <zh>ǩrender߿ľ(ٷֱ).</zh>
   * <en>The gap between the labels and the render area
   *  as a percentage of the render area width.</en>
   */
  private double labelGap ;

  /**
   * <zh>Ƿʾǩͱͼ֮.</zh>
   * <en>A flag that controls whether or not the label links are drawn.</en>
   */
  private boolean labelLinksVisible ;

  /**
   * <zh>ߵĿհ.</zh>
   * <en>The link margin.</en>
   */
  private double labelLinkMargin ;

  /**
   * <zh>ߵpaint.</zh>
   * <en>The paint used for the label linking lines.</en>
   */
  private transient Paint labelLinkPaint ;

  /**
   * <zh>ߵstroke.</zh>
   * <en>The stroke used for the label linking lines.</en>
   */
  private transient Stroke labelLinkStroke ;

  /**
   * <zh>label,ڱͼʾʾ.</zh>
   * <en>A flag that controls whether simple or extended labels are used.</en>
   */
  private boolean simpleLabels ;

  /**
   * <zh>εСǶ.</zh>
   * <en>The smallest arc angle that will get drawn.</en>
   */
  private double minimumArcAngleToDraw ;

  /**
   * <zh>Ƿ3D͵ıͼ.</zh>
   * <en>A flag that controls whether or not the pie is 3D.</en>
   */
  private boolean pie3D ;

  /**
   * <zh>3DЧ.</zh>
   * <en>The depth of 3D pie.</en>
   */
  private double depth3D ;

  /**
   * <zh>ǷʾΪԲͼ.</zh>
   * <en>A flag that controls whether or not the shape is ring.</en>
   */
  private boolean drawAsRing ;

  /**
   * <zh>ĳ뾶ռ.</zh>
   * <en>The length factor (ring-width/sector-radius-width).</en>
   */
  private double ringLengthFactor ;

  /**
   * <zh>.</zh>
   * <en>constructor.</en>
   *
   * @param dataset SingleRowDataset <zh>ݼ (Ϊnull).</zh>
   * <en>The dataset (null not permitted).</en>
   */
  public PieRender(SingleRowDataset dataset) {
    this(dataset, DEFAULT_PIE_IS_3D) ;
  }

  /**
   * <zh>.</zh>
   * <en>constructor.</en>
   *
   * @param dataset SingleRowDataset <zh>ݼ (Ϊnull).</zh>
   * <en>The dataset (null not permitted).</en>
   * @param is3D boolean <zh>Ƿ3D͵ıͼ.</zh>
   * <en>A flag that controls whether or not the pie is 3D.</en>
   */
  public PieRender(SingleRowDataset dataset, boolean is3D) {
    super(dataset) ;

    this.interiorGap = DEFAULT_INTERIOR_GAP ;
    this.circular = DEFAULT_IS_CIRCLE ;
    this.startAngle = DEFAULT_START_ANGLE ;

    this.pieRotation = DEFAULT_PIE_ROTATION ;
    this.sectionOutlinesVisible = DEFAULT_SECTION_OUTLINES_VISIBLE ;
    this.pieShadowPaint = DEFAULT_PIE_SHADOW_PAINT ;

    this.shadowXOffset = DEFAULT_SHADOW_X_OFFSET ;
    this.shadowYOffset = DEFAULT_SHADOW_Y_OFFSET ;
    this.explodePercentages = new HashMap() ;

    this.labelBackgroundPaint = DEFAULT_LABEL_BACKGROUND_PAINT ;
    this.labelOutlinePaint = DEFAULT_LABEL_OUTLINE_PAINT ;
    this.labelOutlineStroke = DEFAULT_LABEL_OUTLINE_STROKE ;

    this.labelShadowPaint = DEFAULT_LABEL_SHADOW_PAINT ;
    this.labelInPie = DEFAULT_LABEL_IN_PIE ;
    this.labelPadding = DEFAULT_LABEL_PADDING ;

    this.maximumLabelWidth = DEFAULT_MAXIMUM_LABEL_WIDTH ;
    this.labelGap = DEFAULT_LABEL_GAP ;
    this.labelLinksVisible = DEFAULT_LABEL_LINKS_VISIBLE ;

    this.labelLinkMargin = DEFAULT_LABEL_LINK_MARGIN ;
    this.labelLinkPaint = DEFAULT_LABEL_LINK_PAINT ;
    this.labelLinkStroke = DEFAULT_LABEL_LINK_STROKE ;
    this.simpleLabels = DEFAULT_SIMPLE_LABELS ;

    this.minimumArcAngleToDraw = DEFAULT_MINIMUM_ARC_ANGLE_TO_DRAW ;
    this.pie3D = is3D ;
    this.depth3D = DEFAULT_DEPTH_3D ;
    this.drawAsRing = DEFAULT_DRAW_AS_RING ;
    this.ringLengthFactor = RingChartShape.DEFAULT_LENGTH_FACTOR ;

    setLabelGenerator(new DefaultPieChartLabelGenerator()) ;
    setBackgroundPaint(null) ;
  }

  /**
   * <zh>ڲĿհ(ٷֱ).</zh>
   * <en>Returns the interior gap.</en>
   *
   * @return double <zh>հ.</zh>
   * <en>The gap.</en>
   */
  public double getInteriorGap() {
    return this.interiorGap ;
  }

  /**
   * <zh>ڲĿհ(ٷֱ),м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the interior gap and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param gapSize double <zh>հ,[0.0,1.0).</zh>
   * <en>The gap,in interval [0.0,1.0).</en>
   */
  public void setInteriorGap(double gapSize) {
    if (gapSize < 0 || gapSize >= 1.0) {
      throw new IllegalArgumentException("Argument 'gapSize' must be in interval [0.0,1.0).") ;
    }

    if (this.interiorGap != gapSize) {
      this.interiorGap = gapSize ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>ǷǿƱͼΪԲ.</zh>
   * <en>Returns the flag determining whether to draw an ellipse or a perfect circle.</en>
   *
   * @return boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  public boolean isCircular() {
    return this.circular ;
  }

  /**
   * <zh>ǷǿƱͼΪԲ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the flag determining whether to draw an ellipse or a perfect circle and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param flag boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  public void setCircular(boolean flag) {
    if (this.circular != flag) {
      this.circular = flag ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>ؿʼεĽǶ.</zh>
   * <en>Returns the starting angle.</en>
   *
   * @return double <zh>Ƕ.</zh>
   * <en>The angle.</en>
   */
  public double getStartAngle() {
    return this.startAngle ;
  }

  /**
   * <zh>ÿʼεĽǶ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the starting angle and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param angle double <zh>Ƕ.</zh>
   * <en>The angle.</en>
   */
  public void setStartAngle(double angle) {
    if (this.startAngle != angle) {
      this.startAngle = angle ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>ػƱͼķ.</zh>
   * <en>Returns the direction for the pie segments.</en>
   *
   * @return Rotation <zh>.</zh>
   * <en>The direction.</en>
   */
  public Rotation getPieRotation() {
    return this.pieRotation ;
  }

  /**
   * <zh>ûƱͼķ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the direction for the pie segments and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param rotation Rotation <zh> (Ϊnull).</zh>
   * <en>The direction (null not permitted).</en>
   */
  public void setPieRotation(Rotation rotation) {
    if (rotation == null) {
      throw new IllegalArgumentException("Null 'rotation' argument") ;
    }

    if (!this.pieRotation.equals(rotation)) {
      this.pieRotation = rotation ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>Ƿεı߿.</zh>
   * <en>Returns the flag that controls whether or not an outline is drawn for each
   * section in the render area.</en>
   *
   * @return boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  public boolean isSectionOutlinesVisible() {
    return this.sectionOutlinesVisible ;
  }

  /**
   * <zh>Ƿεı߿,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the flag that controls whether or not an outline is drawn for each
   * section in the render area and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param flag boolean
   */
  public void setSectionOutlinesVisible(boolean flag) {
    if (this.sectionOutlinesVisible != flag) {
      this.sectionOutlinesVisible = flag ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>ػƱͼӰpaint.</zh>
   * <en>Returns the pie shadow paint.</en>
   *
   * @return Paint <zh>paint (Ϊnull).</zh>
   * <en>The paint (maybe null).</en>
   */
  public Paint getPieShadowPaint() {
    return this.pieShadowPaint ;
  }

  /**
   * <zh>ûƱͼӰpaint,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the pie shadow paint and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param paint Paint <zh>paint (Ϊnull).</zh>
   * <en>The paint (null permitted).</en>
   */
  public void setPieShadowPaint(Paint paint) {
    if (!(this.pieShadowPaint == null && paint == null)) {
      this.pieShadowPaint = paint ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>xӰ.</zh>
   * <en>Returns the x-offset for the shadow effect.</en>
   *
   * @return double <zh>ƫֵ.</zh>
   * <en>The x-offset.</en>
   */
  public double getShadowXOffset() {
    return this.shadowXOffset ;
  }

  /**
   * <zh>xӰ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the x-offset for the shadow effect and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param offset double <zh>ƫֵ.</zh>
   * <en>The x-offset.</en>
   */
  public void setShadowXOffset(double offset) {
    if (offset < 0) {
      throw new IllegalArgumentException("Argument 'offset' must be >=0.") ;
    }

    if (this.shadowXOffset != offset) {
      this.shadowXOffset = offset ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>xӰ.</zh>
   * <en>Returns the y-offset for the shadow effect.</en>
   *
   * @return double <zh>ƫֵ.</zh>
   * <en>The y-offset.</en>
   */
  public double getShadowYOffset() {
    return this.shadowYOffset ;
  }

  /**
   * <zh>yӰ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the y-offset for the shadow effect and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param offset double <zh>ƫֵ.</zh>
   * <en>The y-offset.</en>
   */
  public void setShadowYOffset(double offset) {
    if (this.shadowYOffset != offset) {
      this.shadowYOffset = offset ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>ƫԲĵĴС(Բ뾶С).</zh>
   * <en>Returns the percentage amount to explode each pie section of the specified data.</en>
   *
   * @param index int <zh>ݼݵֵ.</zh>
   * <en>The data index in dataset.</en>
   * @return double <zh>ƫС.</zh>
   * <en>The offset.</en>
   */
  public double getExplodePercentages(int index) {
    double result = 0.0 ;
    if (this.explodePercentages != null) {
      Number percent = (Number)this.explodePercentages.get(new Integer(index)) ;
      if (percent != null) {
        result = percent.doubleValue() ;
      }
    }

    return result ;
  }

  /**
   * <zh>ƫԲĵĴС,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the percentage amount to explode each pie section and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param index int <zh>ݼݵֵ.</zh>
   * <en>The data index in dataset.</en>
   * @param percent double <zh>ƫٷֱ.</zh>
   * <en>The percent.</en>
   */
  public void setExplodePercent(int index, double percent) {
    if (index < 0) {
      throw new IllegalArgumentException("Argument 'index' must be >=0.") ;
    }

    this.explodePercentages.put(new Integer(index), new Double(percent)) ;
    fireRenderChanged() ;
  }

  /**
   * <zh>رǩıɫ.</zh>
   * <en>Returns the color used to draw the background of the section labels.</en>
   *
   * @return Paint <zh>paint.</zh>
   * <en>The paint.</en>
   */
  public Paint getLabelBackgroundPaint() {
    return this.labelBackgroundPaint ;
  }

  /**
   * <zh>ñǩıɫ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the color used to draw the background of the section labels and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param paint Paint <zh>paint (Ϊnull).</zh>
   * <en>The paint (null permitted).</en>
   */
  public void setLabelBackgroundPaint(Paint paint) {
    if (!(this.labelBackgroundPaint == null && paint == null)) {
      this.labelBackgroundPaint = paint ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>ػƱǩı߿paint.</zh>
   * <en>Returns the paint used to draw the outline of the section labels.</en>
   *
   * @return Paint <zh>paint.</zh>
   * <en>The paint.</en>
   */
  public Paint getLabelOutlinePaint() {
    return this.labelOutlinePaint ;
  }

  /**
   * <zh>ûƱǩı߿paint,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the paint used to draw the outline of the section labels and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param paint Paint <zh>paint (Ϊnull).</zh>
   * <en>The paint (null permitted).</en>
   */
  public void setLabelOutlinePaint(Paint paint) {
    if (!(this.labelBackgroundPaint == null && paint == null)) {
      this.labelOutlinePaint = paint ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>ػƱ߿stroke.</zh>
   * <en>Returns the stroke used to draw the outline of the section labels.</en>
   *
   * @return Stroke <zh>stroke.</zh>
   * <en>The stroke.</en>
   */
  public Stroke getLabelOutlineStroke() {
    return this.labelOutlineStroke ;
  }

  /**
   * <zh>ûƱ߿stroke,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the stroke used to draw the outline of the section labels and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param stroke Stroke <zh>stroke (Ϊnull).</zh>
   * <en>The stroke (null permitted).</en>
   */
  public void setLabelOutlineStroke(Stroke stroke) {
    if (!(this.labelOutlineStroke == null && stroke == null)) {
      this.labelOutlineStroke = stroke ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>ػƱǩӰpaint.</zh>
   * <en>Returns the paint used to draw the shadow for the section labels.</en>
   *
   * @return Paint <zh>paint.</zh>
   * <en>The paint.</en>
   */
  public Paint getLabelShadowPaint() {
    return this.labelShadowPaint ;
  }

  /**
   * <zh>ûƱǩӰpaint,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the paint used to draw the shadow for the section labels and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param paint Paint
   */
  public void setLabelShadowPaint(Paint paint) {
    if (!(this.labelShadowPaint == null && paint == null)) {
      this.labelShadowPaint = paint ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>رǩǷڱͼڲ.</zh>
   * <en>Returns the flag that controls whether or not label in pie.</en>
   *
   * @return boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  public boolean isLabelInPie() {
    return this.labelInPie ;
  }

  /**
   * <zh>ñǩǷڱͼڲ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the flag that controls whether or not label in pie and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param flag boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  public void setLabelInPie(boolean flag) {
    if (this.labelInPie != flag) {
      this.labelInPie = flag ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>رǩĲ.</zh>
   * <en>Returns the padding between the labels and the label outlines.</en>
   *
   * @return RectangleInsets <zh> (Ϊnull).</zh>
   * <en>The padding (never null).</en>
   */
  public RectangleInsets getLabelPadding() {
    return this.labelPadding ;
  }

  /**
   * <zh>ñǩĲ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the padding between the labels and the label outlines and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param padding RectangleInsets <zh> (Ϊnull).</zh>
   * <en>The padding (null not permitted).</en>
   */
  public void setLabelPadding(RectangleInsets padding) {
    if (padding == null) {
      throw new IllegalArgumentException("Null 'padding' argument") ;
    }

    this.labelPadding = padding ;
    fireRenderChanged() ;
  }

  /**
   * <zh>رǩ(İٷֱ).</zh>
   * <en>Returns the maximum label width as a percentage of the draw area width.</en>
   *
   * @return double <zh>.</zh>
   * <en>The width.</en>
   */
  public double getMaximumLabelWidth() {
    return this.maximumLabelWidth ;
  }

  /**
   * <zh>ñǩ(İٷֱ),м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the maximum label width as a percentage of the draw area width and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param maximumWidth double <zh>,>0.</zh>
   * <en>The width and must be >0.</en>
   */
  public void setMaximumLabelWidth(double maximumWidth) {
    if (maximumWidth <= 0) {
      throw new IllegalArgumentException("Argument 'maximumWidth' must be >0.") ;
    }

    if (this.maximumLabelWidth != maximumWidth) {
      this.maximumLabelWidth = maximumWidth ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>رǩrender߿ľ(ٷֱ).</zh>
   * <en>Returns the gap between the labels and the draw area
   *  as a percentage of the render area width.</en>
   *
   * @return double <zh>հ״С.</zh>
   * <en>The gap.</en>
   */
  public double getLabelGap() {
    return this.labelGap ;
  }

  /**
   * <zh>ñǩrender߿ľ(ٷֱ),м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets  the gap between the labels and the render area and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param gap double double <zh>հ,>=0.</zh>
   * <en>The gap and must be >0.</en>
   */
  public void setLabelGap(double gap) {
    if (gap < 0) {
      throw new IllegalArgumentException("Argument 'gap' must be >=0.") ;
    }

    if (this.labelGap != gap) {
      this.labelGap = gap ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>Ƿʾǩͱͼ֮.</zh>
   * <en>A flag that controls whether or not the label links are drawn.</en>
   *
   * @return boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  public boolean isLabelLinksVisible() {
    return this.labelLinksVisible ;
  }

  /**
   * <zh>Ƿʾǩͱͼ֮,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets whether or not the label links are drawn and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param flag boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  public void setLabelLinksVisible(boolean flag) {
    if (this.labelLinksVisible != flag) {
      this.labelLinksVisible = flag ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>ߵĿհ.</zh>
   * <en>Returns the link margin.</en>
   *
   * @return double <zh>հ.</zh>
   * <en>The margin.</en>
   */
  public double getLabelLinkMargin() {
    return this.labelLinkMargin ;
  }

  /**
   * <zh>ߵĿհ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the link margin and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param margin double <zh>հ,>=0.</zh>
   * <en>The margin and must be >=0.</en>
   */
  public void setLabelLinkMargin(double margin) {
    if (this.labelLinkMargin != margin) {
      this.labelLinkMargin = margin ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>ػߵpaint.</zh>
   * <en>Returns the paint used for the label linking lines.</en>
   *
   * @return Paint <zh>paint.</zh>
   * <en>The paint.</en>
   */
  public Paint getLabelLinkPaint() {
    return this.labelLinkPaint ;
  }

  /**
   * <zh>ûߵpaint,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the paint used for the label linking lines and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param paint Paint <zh>paint (Ϊnull).</zh>
   * <en>The paint (null not permitted).</en>
   */
  public void setLabelLinkPaint(Paint paint) {
    if (paint == null) {
      throw new IllegalArgumentException("Null 'paint' argument") ;
    }

    this.labelLinkPaint = paint ;
    fireRenderChanged() ;
  }

  /**
   * <zh>ػߵstroke.</zh>
   * <en>Returns the stroke used for the label linking lines.</en>
   *
   * @return Stroke <zh>stroke.</zh>
   * <en>The stroke.</en>
   */
  public Stroke getLabelLinkStroke() {
    return this.labelLinkStroke ;
  }

  /**
   * <zh>ûߵstroke,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the stroke used for the label linking lines and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param stroke Stroke <zh>stroke.</zh>
   * <en>The stroke.</en>
   */
  public void setLabelLinkStroke(Stroke stroke) {
    if (stroke == null) {
      throw new IllegalArgumentException("Null 'stroke' argument") ;
    }

    this.labelLinkStroke = stroke ;
    fireRenderChanged() ;
  }

  /**
   * <zh>ػlabel,ڱͼʾʾ.</zh>
   * <en>Returns the flag that controls whether simple or extended labels are used.</en>
   *
   * @return boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  public boolean isSimpleLabels() {
    return this.simpleLabels ;
  }

  /**
   * <zh>ûlabel,ڱͼʾʾ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the flag that controls whether simple or extended labels are used and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param flag boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  public void setSimpleLabels(boolean flag) {
    if (this.simpleLabels != flag) {
      this.simpleLabels = flag ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>ػεСǶ.</zh>
   * <en>Returns the smallest arc angle that will get drawn.</en>
   *
   * @return double <zh>Ƕ.</zh>
   * <en>The angle.</en>
   */
  public double getMinimumArcAngleToDraw() {
    return this.minimumArcAngleToDraw ;
  }

  /**
   * <zh>ûεСǶ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the smallest arc angle that will get drawn and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param angle double <zh>Ƕ,>=0.</zh>
   * <en>The angle and must be >=0.</en>
   */
  public void setMinimumArcAngleToDraw(double angle) {
    if (angle < 0) {
      throw new IllegalArgumentException("Argument 'angle' must be >=0") ;
    }

    if (this.minimumArcAngleToDraw != angle) {
      this.minimumArcAngleToDraw = angle ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>Ƿ3D͵ıͼ.</zh>
   * <en>Returns whether or not the pie is 3D.</en>
   *
   * @return boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  public boolean isPie3D() {
    return this.pie3D ;
  }

  /**
   * <zh>Ƿ3D͵ıͼ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the flag that controls whether or not the pie is 3D and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param flag boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  public void setPie3D(boolean flag) {
    if (this.pie3D != flag) {
      this.pie3D = flag ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>ػ3DЧ.</zh>
   * <en>Returns the depth of 3D pie.</en>
   *
   * @return double <zh>.</zh>
   * <en>The depth.</en>
   */
  public double getDepth3D() {
    return this.depth3D ;
  }

  /**
   * <zh>û3DЧ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the depth of 3D pie and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param depth3D double <zh>.</zh>
   * <en>The depth.</en>
   */
  public void setDepth3D(double depth3D) {
    if (depth3D >= 0 && this.depth3D != depth3D) {
      this.depth3D = depth3D ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>ǷʾΪԲͼ.</zh>
   * <en>Returns whether or not the shape is ring.</en>
   *
   * @return boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  public boolean isDrawAsRing() {
    return this.drawAsRing ;
  }

  /**
   * <zh>ǷʾΪԲͼ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the flag that controls whether or not the shape is ring and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param flag boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  public void setDrawAsRing(boolean flag) {
    if (this.drawAsRing != flag) {
      this.drawAsRing = flag ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>ػĳ뾶ռ.</zh>
   * <en>Returns the length factor.</en>
   *
   * @return double <zh>.</zh>
   * <en>The length factor.</en>
   */
  public double getRingLengthFactor() {
    return this.ringLengthFactor ;
  }

  /**
   * <zh>ûĳ뾶ռ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the length factor and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param factor double <zh>,(0,1].</zh>
   * <en>The length factor,in interval (0,1].</en>
   */
  public void setRingLengthFactor(double factor) {
    if (factor <= 0 || factor > 1) {
      throw new IllegalArgumentException("Argument 'factor' must be in interval (0,1].") ;
    }

    if (this.ringLengthFactor != factor) {
      this.ringLengthFactor = factor ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>еͼ.</zh>
   * <en>Draws all data items.</en>
   *
   * @param dataArea Rectangle2D <zh> (Ϊnull).</zh>
   * <en>The chart data area (null not permitted).</en>
   * @param renderShapes RenderShapeCollection <zh>renderͼμ (Ϊnull).</zh>
   * <en>Collects information about the render (null not permitted).</en>
   * @param alpha float <zh>alphaֵ,ڱ[0.0,1.0].</zh>
   * <en>The alpha transparency (in the range 0.0f to 1.0f, where 0.0f is fully transparent,
   * and 1.0f is fully opaque).</en>
   */
  public void drawAllItems(Rectangle2D dataArea, RenderShapeCollection renderShapes, float alpha) {
    if (dataArea == null) {
      throw new IllegalArgumentException("Null 'dataArea' argument.") ;
    }
    if (renderShapes == null) {
      throw new IllegalArgumentException("Null 'renderShapes' argument.") ;
    }
    if (alpha < 0.0f || alpha > 1.0f) {
      throw new IllegalArgumentException("The 'alpha' value must be in the range 0.0f to 1.0f.") ;
    }

    IMatrixDataset dataset = getDataset() ;
    if (dataset.isEmpty()) {
      return ;
    }

    int columns = dataset.getColumnCount() ;
    this.fillPaints = getFillPaints(columns) ;
    this.borderPaints = getBorderPaints(columns) ;
    this.borderStrokes = getBorderStrokes(columns) ;

    double labelWidth = 0.0 ;
    if (getLabelGenerator() != null && !this.simpleLabels) {
      labelWidth = this.labelGap + this.maximumLabelWidth + this.labelLinkMargin ;
    }

    double gapHorizontal = dataArea.getWidth() * (this.interiorGap + labelWidth) ;
    double gapVertical = dataArea.getHeight() * this.interiorGap ;

    double linkX = dataArea.getX() + gapHorizontal * 0.5 ;
    double linkY = dataArea.getY() + gapVertical * 0.5 ;
    double linkW = dataArea.getWidth() - gapHorizontal ;
    double linkH = dataArea.getHeight() - gapVertical ;

    // make the link area a square if the pie chart is to be circular...
    if (this.circular) {
      double min = Math.min(linkW, linkH) * 0.5 ;
      linkX = (linkX + linkX + linkW) * 0.5 - min ;
      linkY = (linkY + linkY + linkH) * 0.5 - min ;
      linkW = 2 * min ;
      linkH = 2 * min ;
    }

    Rectangle2D linkArea = new Rectangle2D.Double(linkX, linkY, linkW, linkH) ;
    double lm = 0.0 ;
    if (!this.simpleLabels) {
      lm = this.labelLinkMargin ;
    }

    double hh = linkArea.getWidth() * lm ;
    double vv = linkArea.getHeight() * lm ;
    Rectangle2D explodeArea = new Rectangle2D.Double(linkX + hh * 0.5, linkY + vv * 0.5,
                                                     linkW - hh, linkH - vv) ;

    double maximumExplodePercent = getMaximumExplodePercent() ;
    double percent = maximumExplodePercent / (1.0 + maximumExplodePercent) ;

    double h1 = explodeArea.getWidth() * percent ;
    double v1 = explodeArea.getHeight() * percent ;
    Rectangle2D pieArea = new Rectangle2D.Double(explodeArea.getX() + h1 * 0.5,
                                                 explodeArea.getY() + v1 * 0.5,
                                                 explodeArea.getWidth() - h1,
                                                 explodeArea.getHeight() - v1) ;

    AffineTransform transform = AffineTransform.getTranslateInstance(this.shadowXOffset,
                                                                     this.shadowYOffset) ;

    SingleRowDataset singleDataset = (SingleRowDataset) dataset ;
    double totalValue = DatasetUtil.calculateDatasetTotalForPieChart(singleDataset) ;
    int cols = singleDataset.getColumnCount() ;
    double angleTotal = this.startAngle ;
    int rotationFactor = this.pieRotation.getFactor() ;
    for (int i = 0 ; i < cols ; i++) {
      IChartData data = singleDataset.getData(0, i) ;
      NumberChartData numberData = (NumberChartData) data ;
      if (numberData != null) {
        double value = numberData.getValue() ;
        if (Double.isNaN(value) || value <= 0.0 || totalValue == 0.0) {
          renderShapes.addShadowShape(null) ;
          renderShapes.addDataItemShape(null) ;
          continue ;
        }

        double valuePercent = value / totalValue ;
        double extentAngle = 0 ;
        if (i != cols - 1) {
          extentAngle = rotationFactor * valuePercent * 360.0 ;
        } else { //ȷԲǹرյ
          extentAngle = rotationFactor * (360.0 - Math.abs(angleTotal - this.startAngle)) ;
        }

        if (this.minimumArcAngleToDraw >= Math.abs(extentAngle)) {
          renderShapes.addShadowShape(null) ;
          renderShapes.addDataItemShape(null) ;
          continue ;
        }

        double explode = getExplodePercentages(i) ;
        Rectangle2D arcBounds = getArcBounds(pieArea, angleTotal, extentAngle, explode) ;
        Arc2D.Double arc = new Arc2D.Double(arcBounds, angleTotal, extentAngle, Arc2D.PIE) ;

        if (this.pieShadowPaint != null) {
          Shape shadowShape = null ;
          if (this.drawAsRing) {
            double arcWidth = arc.getWidth() ;
            double arcHeight = arc.getHeight() ;
            double arcHoleLen = 1 - this.ringLengthFactor ;
            double halfFactor = this.ringLengthFactor * 0.5 ;
            Arc2D arcHole = new Arc2D.Double(arcWidth * halfFactor + arcBounds.getX(),
                                             arcHeight * halfFactor + arcBounds.getY(),
                                             arcWidth * arcHoleLen, arcHeight * arcHoleLen,
                                             angleTotal, extentAngle * 1.1, arc.getArcType()) ; //*1.1Ǳ֤㹻
            Area ringArea = new Area(arc) ;
            ringArea.subtract(new Area(arcHole)) ;

            shadowShape = transform.createTransformedShape(ringArea) ;
          } else {
            shadowShape = transform.createTransformedShape(arc) ;
          }

          GeneralPath path = new GeneralPath(shadowShape) ;
          GeneralPathChartShape shadowChartShape = new GeneralPathChartShape(path) ;
          shadowChartShape.setFillShape(true) ;
          shadowChartShape.setFillPaint(this.pieShadowPaint) ;
          shadowChartShape.setDrawBorder(false) ;

          renderShapes.addShadowShape(shadowChartShape) ;
        }

        ILabelGenerator labelGen = getLabelGenerator() ;
        String label = null ;
        if (labelGen != null) {
          label = labelGen.generateLabel(getDataset(), 0, i) ;
        } else {
          Object labelObj = singleDataset.getColKey(i) ;
          if (labelObj != null) {
            label = labelObj.toString() ;
          }
        }

        SectorChartShape sector = null ;
        if (this.drawAsRing) {
          sector = new RingChartShape(arc, linkArea, label) ;
        } else {
          sector = new SectorChartShape(arc, linkArea, label) ;
        }
        sector.setSectorShadowXOffset(this.shadowXOffset) ;
        sector.setSectorShadowXOffset(this.shadowYOffset) ;
        sector.setSectorShadowPaint(null) ;
        sector.setDrawBorder(this.sectionOutlinesVisible) ;
        sector.setSectorShadowXOffset(this.shadowXOffset) ;
        sector.setSectorShadowYOffset(this.shadowYOffset) ;

        sector.setLabelBackgroundPaint(this.labelBackgroundPaint) ;
        sector.setLabelOutlinePaint(this.labelOutlinePaint) ;
        sector.setLabelOutlineStroke(this.labelOutlineStroke) ;
        sector.setLabelShadowPaint(this.pie3D ? null : this.labelShadowPaint) ;
        sector.setLabelPadding(this.labelPadding) ;
        sector.setSector3D(false) ;

        IChartShape chartShape = createItemShape(0, i, fillPaints[i], borderPaints[i],
                                                 borderStrokes[i], alpha, sector) ;
        if (!this.simpleLabels) { //ڱͼƱǩ
          sector.setDrawText(false) ;
        }
        renderShapes.addDataItemShape(chartShape) ;

        if (this.pie3D && !this.drawAsRing) {
          SectorChartShape sector3D = new SectorChartShape(arc, linkArea, label) ;
          sector3D.setDrawBorder(false) ;
          sector3D.setFillShape(false) ;
          sector3D.setDrawText(false) ;
          sector3D.setSector3D(true) ;
          sector3D.setLabelShadowPaint(null) ;
          sector3D.setFillPaint(fillPaints[i]) ;
          sector3D.setSectorShadowPaint(null) ;
          sector3D.setBorderPaint(borderPaints[i]) ;
          sector3D.setBorderStroke(borderStrokes[i]) ;
          sector.setSectorShadowPaint(null) ;
          sector3D.setDepth3D(this.depth3D) ;

          renderShapes.addPie3DEffectShape(sector3D) ;
        }

        angleTotal += extentAngle ;
      } else {
        renderShapes.addShadowShape(null) ;
        renderShapes.addDataItemShape(null) ;
      }
    }

    if (!this.simpleLabels) { //ڱͼƱǩ
      drawLabels(totalValue, dataArea, pieArea, linkArea, renderShapes) ;
    }
  }

  /**
   * <zh>(arc)ľ,ҪǿǵЩarcƫԲĵ.</zh>
   * <en>Returns a rectangle that can be used to create a pie section.</en>
   *
   * @param unexploded <zh>ڲδƫƵεľ.</zh>
   * <en>The area inside which the unexploded pie sections are drawn.</en>
   * @param startAngle <zh>ʼĽǶ.</zh>
   * <en>The start angle.</en>
   * @param extent <zh>εչǶ.</zh>
   * <en>The extent of the arc.</en>
   * @param explodePercent <zh>ƫԲĵİٷֱ.</zh>
   * <en>The amount by which the pie section is exploded.</en>
   *
   * @return <zh>.</zh>
   * <en>The rectangle.</en>
   */
  protected Rectangle2D getArcBounds(Rectangle2D unexploded, double startAngle,
                                     double extent, double explodePercent) {
    if (explodePercent == 0.0) {
      return unexploded ;
    } else {
      Arc2D arc = new Arc2D.Double(unexploded, startAngle, extent, Arc2D.PIE) ;

      Point2D center = new Point2D.Double(arc.getCenterX(), arc.getCenterY()) ;
      Point2D middle = GeometryUtil.middlePoint(arc.getStartPoint(), arc.getEndPoint()) ;
      double selectedShiftDis = (unexploded.getWidth() * 0.5 + unexploded.getHeight() * 0.5)
                                * 0.5 * explodePercent ;
      Point2D extentPoint = GeometryUtil.extentPoint(center, middle, selectedShiftDis) ;

      double deltaX = extentPoint.getX() - center.getX() ;
      double deltaY = extentPoint.getY() - center.getY() ;
      double x = arc.getX() + deltaX ;
      double y = arc.getY() + deltaY ;

      return new Rectangle2D.Double(x, y, arc.getWidth(), arc.getHeight()) ;
    }
  }

  /**
   * <zh>ֻͼ,ͼε.</zh>
   * <en>Create a shape with no attribute specified.</en>
   *
   * @param rect Rectangle2D <zh>ͼľ (Ϊnull).</zh>
   * <en>The bar rectangle (null not permitted).</en>
   * @param text String <zh>Ҫʾ (Ϊnull).</zh>
   * <en>The text (null permitted).</en>
   * @param appendParam Object <zh>ӵĲ,ڴݶĶ (Ϊnull).</zh>
   * <en>The append parameter,used for special propose (null permitted).</en>
   *
   * @return IChartShape <zh>ͼ.</zh>
   * <en>The shape.</en>
   */
  protected IChartShape createItemShapeOnly(Rectangle2D rect, String text,
                                            Object appendParam) {
    if (appendParam == null) {
      throw new IllegalArgumentException("Null 'appendParam' argument.") ;
    }

    return (IChartShape) appendParam ;
  }

  /**
   * <zh>Ʊɫ.</zh>
   * <en>Draws a background for the data area.</en>
   *
   * @param dataArea Rectangle2D <zh> (Ϊnull).</zh>
   * <en>The chart data area (null not permitted).</en>
   * @param orientation ChartOrientation <zh>ͼƷ (Ϊnull).</zh>
   * <en>The chart draw orientation (null not permitted).</en>
   * @param renderShapes RenderShapeCollection <zh>renderͼμ (Ϊnull).</zh>
   * <en>Collects information about the render (null not permitted).</en>
   */
  public void drawBackground(Rectangle2D dataArea, ChartOrientation orientation,
                             RenderShapeCollection renderShapes) {
    if (dataArea == null) {
      throw new IllegalArgumentException("Null 'dataArea' argument.") ;
    }
    if (orientation == null) {
      throw new IllegalArgumentException("Null 'orientation' argument.") ;
    }
    if (renderShapes == null) {
      throw new IllegalArgumentException("Null 'renderShapes' argument.") ;
    }

    Paint p = getBackgroundPaint() ;
    if (p == null) {
      return ;
    }

    if (p instanceof GradientPaint) {
      GradientPaint gp = (GradientPaint) p ;
      if (orientation.equals(ChartOrientation.VERTICAL)) {
        p = new GradientPaint((float) dataArea.getCenterX(), (float) dataArea.getMaxY(),
                              gp.getColor1(), (float) dataArea.getCenterX(),
                              (float) dataArea.getMinY(), gp.getColor2()) ;
      } else if (orientation.equals(ChartOrientation.HORIZONTAL)) {
        p = new GradientPaint((float) dataArea.getMinX(), (float) dataArea.getCenterY(),
                              gp.getColor1(), (float) dataArea.getMaxX(),
                              (float) dataArea.getCenterY(), gp.getColor2()) ;
      }
    }

    //ɫͼ
    EllipseTextChartShape shape = new EllipseTextChartShape(dataArea, null) ;
    shape.setDrawBorder(false) ;
    shape.setFillPaint(p) ;
    shape.setFillShape(true) ;

    renderShapes.setBackgroundShape(shape) ;
  }

  /**
   * <zh>ƫԲĵĴС.</zh>
   * <en>Returns the maximum explode percent.</en>
   *
   * @return double <zh>İٷֱ.</zh>
   * <en>The maximum percent..</en>
   */
  public double getMaximumExplodePercent() {
    double result = 0.0 ;
    Iterator iterator = this.explodePercentages.values().iterator() ;
    while (iterator.hasNext()) {
      Double percent = (Double) iterator.next() ;
      result = Math.max(result, percent.doubleValue()) ;
    }

    return result ;
  }

  /**
   * <zh>Ʊͼⲿıǩ.</zh>
   * <en>Draws the labels for the pie sections.</en>
   *
   * @param totalValue double <zh>ͼݼݵĺ.</zh>
   * <en>The total value of pie dataset.</en>
   * @param dataArea Rectangle2D <zh>.</zh>
   * <en>The data area.</en>
   * @param pieArea Rectangle2D <zh>ͼ.</zh>
   * <en>The pie area.</en>
   * @param linkArea Rectangle2D <zh>ⲿǩ.</zh>
   * <en>The link area.</en>
   * @param renderShapes RenderShapeCollection <zh>renderͼμ.</zh>
   * <en>Collects information about the render.</en>
   */
  protected void drawLabels(double totalValue, Rectangle2D dataArea, Rectangle2D pieArea,
                            Rectangle2D linkArea, RenderShapeCollection renderShapes) {
    Hashtable leftKeys = new Hashtable() ;
    Hashtable rightKeys = new Hashtable() ;

    SingleRowDataset singleDataset = (SingleRowDataset) getDataset() ;
    int cols = singleDataset.getColumnCount() ;
    double runningTotal = 0.0 ;
    for (int i = 0 ; i < cols ; i++) {
      boolean include = true ;
      double v = 0.0 ;
      NumberChartData aData = (NumberChartData) singleDataset.getData(0, i) ;
      if (aData == null) {
        include = false ;
      } else {
        v = aData.getValue() ;
        include = v > 0.0 ;
      }

      if (include) {
        runningTotal += v ;
        double factor = this.pieRotation.getFactor() ;
        double mid = this.startAngle + (factor * ((runningTotal - v * 0.5) * 360) / totalValue) ;
        if (Math.cos(Math.toRadians(mid)) < 0.0) {
          leftKeys.put(new Integer(i), new Double(mid)) ;
        } else {
          rightKeys.put(new Integer(i), new Double(mid)) ;
        }
      }
    }

    drawLeftLabels(leftKeys, dataArea, linkArea, pieArea, renderShapes) ;
    drawRightLabels(rightKeys, dataArea, linkArea, pieArea, renderShapes) ;
  }

  /**
   * <zh>ߵıǩ.</zh>
   * <en>Draws the left labels.</en>
   *
   * @param leftKeys Hashtable <zh>ߵıǩ.</zh>
   * <en>A collection of left keys.</en>
   * @param dataArea Rectangle2D <zh>.</zh>
   * <en>The data area.</en>
   * @param linkArea Rectangle2D <zh>ⲿǩ.</zh>
   * <en>The link area.</en>
   * @param pieArea Rectangle2D <zh>ͼ.</zh>
   * <en>The pie area.</en>
   * @param renderShapes RenderShapeCollection <zh>renderͼμ.</zh>
   * <en>Collects information about the render.</en>
   */
  protected void drawLeftLabels(Hashtable leftKeys, Rectangle2D dataArea, Rectangle2D linkArea,
                                Rectangle2D pieArea, RenderShapeCollection renderShapes) {
    double lGap = dataArea.getWidth() * this.labelGap ;
    double verticalLinkRadius = linkArea.getHeight() * 0.5 ;
    ILabelGenerator labelGen = getLabelGenerator() ;
    PieLabelDistributor labelDistributor = new PieLabelDistributor() ;

    final double labelWidthExtent = 6, labelHeightExtent = 4 ; //ǩչȺ͸߶
    Enumeration keys = leftKeys.keys() ;
    while (keys.hasMoreElements()) {
      Integer key = (Integer) keys.nextElement() ;
      Double value = (Double) leftKeys.get(key) ;
      String label = null ;
      if (labelGen != null && isItemLabelVisible(0, key.intValue())) {
        label = labelGen.generateLabel(getDataset(), 0, key.intValue()) ;
      }

      if (!StringUtil.isEmpty(label)) {
        Rectangle2D textRect = TextUtil.getStringBound(getItemLabelFont(0, key.intValue()), label) ;
        RectangleInsets inset = new RectangleInsets(labelHeightExtent * 0.5, labelWidthExtent * 0.5,
                                                    labelHeightExtent * 0.5, labelWidthExtent * 0.5) ;
        inset.expandRect(textRect) ;

        double theta = Math.toRadians(value.doubleValue()) ;
        double baseY = pieArea.getCenterY() - Math.sin(theta) * verticalLinkRadius ;
        double hh = textRect.getHeight() ;

        PieLabelRecord record = new PieLabelRecord(key, theta, baseY, textRect, label, hh,
                                                   lGap * 0.5 + lGap * 0.5 * -Math.cos(theta), 0.9D
                                                   + getExplodePercentages(key.intValue())) ;
        labelDistributor.addPieLabelRecord(record) ;
      }
    }
    labelDistributor.distributeLabels(dataArea.getMinY(), dataArea.getHeight()) ;

    for (int i = 0 ; i < labelDistributor.getItemCount() ; i++) {
      PieLabelRecord record = labelDistributor.getPieLabelRecord(i) ;
      int colIdx = ((Integer) record.getKey()).intValue() ;
      if (!isItemLabelVisible(0, colIdx)) {
        continue ;
      }

      double anchorX = linkArea.getMinX() ;
      double targetX = anchorX - record.getGap() ;
      double targetY = record.getAllocatedY() ;

      Polygon linkLine = null ;
      if (this.labelLinksVisible) {
        double theta = record.getAngle() ;
        double linkX = pieArea.getCenterX() + Math.cos(theta)
                       * pieArea.getWidth() * 0.5 * record.getLinkPercent() ;
        double linkY = pieArea.getCenterY() - Math.sin(theta)
                       * pieArea.getHeight() * 0.5 * record.getLinkPercent() ;
        double elbowX = pieArea.getCenterX() + Math.cos(theta) * linkArea.getWidth() * 0.5 ;
        double elbowY = pieArea.getCenterY() - Math.sin(theta) * linkArea.getHeight() * 0.5 ;
        double anchorY = elbowY ;

        int[] xPoints = {(int) linkX, (int) elbowX, (int) anchorX, (int) targetX} ;
        int[] yPoints = {(int) linkY, (int) elbowY, (int) anchorY, (int) targetY} ;
        linkLine = new Polygon(xPoints, yPoints, xPoints.length) ; //ӱǩ
      }

      Rectangle2D labelRect = record.getLabelRect() ;
      double width = labelRect.getWidth() + labelWidthExtent ;
      double height = labelRect.getHeight() + labelHeightExtent ;

      Rectangle2D textRect = new Rectangle2D.Double(targetX - width, targetY - height * 0.5,
                                                    width + 4, height) ;
      PieOutsideLabelChartShape pieLabel = new PieOutsideLabelChartShape(textRect, record.getLabel(),
                                                                         linkLine) ;
      pieLabel.setForceShowAllText(true) ;
      pieLabel.setLinkPaint(this.labelLinkPaint) ;
      pieLabel.setLinkStroke(this.labelLinkStroke) ;

      renderShapes.addPieLabelShape(pieLabel) ;
    }
  }

  /**
   * <zh>ұߵıǩ.</zh>
   * <en>Draws the right labels.</en>
   *
   * @param rightKeys Hashtable <zh>ߵıǩ.</zh>
   * <en>A collection of left keys.</en>
   * @param dataArea Rectangle2D <zh>.</zh>
   * <en>The data area.</en>
   * @param linkArea Rectangle2D <zh>ⲿǩ.</zh>
   * <en>The link area.</en>
   * @param pieArea Rectangle2D <zh>ͼ.</zh>
   * <en>The pie area.</en>
   * @param renderShapes RenderShapeCollection <zh>renderͼμ.</zh>
   * <en>Collects information about the render.</en>
   */
  protected void drawRightLabels(Hashtable rightKeys, Rectangle2D dataArea, Rectangle2D linkArea,
                                 Rectangle2D pieArea, RenderShapeCollection renderShapes) {
    double lGap = dataArea.getWidth() * this.labelGap ;
    double verticalLinkRadius = linkArea.getHeight() * 0.5 ;
    ILabelGenerator labelGen = getLabelGenerator() ;
    PieLabelDistributor labelDistributor = new PieLabelDistributor() ;

    final double labelWidthExtent = 8, labelHeightExtent = 4 ; //ǩչȺ͸߶
    Enumeration keys = rightKeys.keys() ;
    while (keys.hasMoreElements()) {
      Integer key = (Integer) keys.nextElement() ;
      Double value = (Double) rightKeys.get(key) ;
      String label = null ;
      if (labelGen != null && isItemLabelVisible(0, key.intValue())) {
        label = labelGen.generateLabel(getDataset(), 0, key.intValue()) ;
      }

      if (!StringUtil.isEmpty(label)) {
        Rectangle2D textRect = TextUtil.getStringBound(getItemLabelFont(0, key.intValue()), label) ;
        RectangleInsets inset = new RectangleInsets(labelHeightExtent * 0.5, labelWidthExtent * 0.5,
                                                    labelHeightExtent * 0.5, labelWidthExtent * 0.5) ;
        inset.expandRect(textRect) ;

        double theta = Math.toRadians(value.doubleValue()) ;
        double baseY = pieArea.getCenterY() - Math.sin(theta) * verticalLinkRadius ;
        double hh = textRect.getHeight() ;

        PieLabelRecord record = new PieLabelRecord(key, theta, baseY, textRect, label, hh,
                                                   lGap * 0.5 + lGap * 0.5 * -Math.cos(theta), 0.9D
                                                   + getExplodePercentages(key.intValue())) ;
        labelDistributor.addPieLabelRecord(record) ;
      }
    }

    labelDistributor.distributeLabels(dataArea.getMinY(), dataArea.getHeight()) ;

    for (int i = 0 ; i < labelDistributor.getItemCount() ; i++) {
      PieLabelRecord record = labelDistributor.getPieLabelRecord(i) ;
      int colIdx = ((Integer) record.getKey()).intValue() ;
      if (!isItemLabelVisible(0, colIdx)) {
        continue ;
      }

      double anchorX = linkArea.getMaxX() ;
      double targetX = anchorX + record.getGap() ;
      double targetY = record.getAllocatedY() ;

      Polygon linkLine = null ;
      if (this.labelLinksVisible) {
        double theta = record.getAngle() ;
        double linkX = pieArea.getCenterX() + Math.cos(theta)
                       * pieArea.getWidth() * 0.5 * record.getLinkPercent() ;
        double linkY = pieArea.getCenterY() - Math.sin(theta)
                       * pieArea.getHeight() * 0.5 * record.getLinkPercent() ;
        double elbowX = pieArea.getCenterX() + Math.cos(theta) * linkArea.getWidth() * 0.5 ;
        double elbowY = pieArea.getCenterY() - Math.sin(theta) * linkArea.getHeight() * 0.5 ;
        double anchorY = elbowY ;

        int[] xPoints = {(int) linkX, (int) elbowX, (int) anchorX, (int) targetX} ;
        int[] yPoints = {(int) linkY, (int) elbowY, (int) anchorY, (int) targetY} ;
        linkLine = new Polygon(xPoints, yPoints, xPoints.length) ; //ӱǩ
      }

      Rectangle2D labelRect = record.getLabelRect() ;
      double width = labelRect.getWidth() + labelWidthExtent ;
      double height = labelRect.getHeight() + labelHeightExtent ;

      Rectangle2D textRect = new Rectangle2D.Double(targetX, targetY - height * 0.5,
                                                    width + 4, height) ;
      PieOutsideLabelChartShape pieLabel = new PieOutsideLabelChartShape(textRect, record.getLabel(),
                                                                         linkLine) ;
      pieLabel.setForceShowAllText(true) ;
      pieLabel.setLinkPaint(this.labelLinkPaint) ;
      pieLabel.setLinkStroke(this.labelLinkStroke) ;

      renderShapes.addPieLabelShape(pieLabel) ;
    }
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Tests this instance for equality with an arbitrary object.</en>
   *
   * @param obj  the object (null permitted).
   *
   * @return A boolean.
   */
  public boolean equals(Object obj) {
    if (obj == this) {
      return true ;
    }
    if (!(obj instanceof PieRender)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    PieRender that = (PieRender) obj ;
    if (this.interiorGap != that.interiorGap) {
      return false ;
    }
    if (this.circular != that.circular) {
      return false ;
    }
    if (this.startAngle != that.startAngle) {
      return false ;
    }
    if (!EqualsUtil.equals(this.pieRotation, that.pieRotation)) {
      return false ;
    }
    if (this.sectionOutlinesVisible != that.sectionOutlinesVisible) {
      return false ;
    }
    if (!EqualsUtil.equals(this.pieShadowPaint, that.pieShadowPaint)) {
      return false ;
    }
    if (this.shadowXOffset != that.shadowXOffset) {
      return false ;
    }
    if (this.shadowYOffset != that.shadowYOffset) {
      return false ;
    }
    if (!EqualsUtil.equals(this.explodePercentages, that.explodePercentages)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.labelBackgroundPaint, that.labelBackgroundPaint)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.labelOutlinePaint, that.labelOutlinePaint)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.labelOutlineStroke, that.labelOutlineStroke)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.labelShadowPaint, that.labelShadowPaint)) {
      return false ;
    }
    if (this.labelInPie != that.labelInPie) {
      return false ;
    }
    if (!EqualsUtil.equals(this.labelPadding, that.labelPadding)) {
      return false ;
    }
    if (this.maximumLabelWidth != that.maximumLabelWidth) {
      return false ;
    }
    if (this.labelGap != that.labelGap) {
      return false ;
    }
    if (this.labelLinksVisible != that.labelLinksVisible) {
      return false ;
    }
    if (this.labelLinkMargin != that.labelLinkMargin) {
      return false ;
    }
    if (!EqualsUtil.equals(this.labelLinkPaint, that.labelLinkPaint)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.labelLinkStroke, that.labelLinkStroke)) {
      return false ;
    }
    if (this.simpleLabels != that.simpleLabels) {
      return false ;
    }
    if (this.minimumArcAngleToDraw != that.minimumArcAngleToDraw) {
      return false ;
    }
    if (this.pie3D != that.pie3D) {
      return false ;
    }
    if (this.depth3D != that.depth3D) {
      return false ;
    }
    if (this.drawAsRing != that.drawAsRing) {
      return false ;
    }
    if (this.ringLengthFactor != that.ringLengthFactor) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return int <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    double result = super.hashCode() ;
    result = 37 * result + this.interiorGap ;
    result = 37 * result + HashcodeUtil.hashCode(this.circular) ;
    result = 37 * result + this.startAngle ;
    result = 37 * result + HashcodeUtil.hashCode(this.pieRotation) ;
    result = 37 * result + HashcodeUtil.hashCode(this.sectionOutlinesVisible) ;
    result = 37 * result + HashcodeUtil.hashCode(this.pieShadowPaint) ;
    result = 37 * result + this.shadowXOffset ;
    result = 37 * result + this.shadowYOffset ;
    result = 37 * result + HashcodeUtil.hashCode(this.explodePercentages) ;
    result = 37 * result + HashcodeUtil.hashCode(this.labelBackgroundPaint) ;
    result = 37 * result + HashcodeUtil.hashCode(this.labelOutlinePaint) ;
    result = 37 * result + HashcodeUtil.hashCode(this.labelOutlineStroke) ;
    result = 37 * result + HashcodeUtil.hashCode(this.labelShadowPaint) ;
    result = 37 * result + HashcodeUtil.hashCode(this.labelInPie) ;
    result = 37 * result + HashcodeUtil.hashCode(this.labelPadding) ;
    result = 37 * result + this.maximumLabelWidth ;
    result = 37 * result + this.labelGap ;
    result = 37 * result + HashcodeUtil.hashCode(this.labelLinksVisible) ;
    result = 37 * result + this.labelLinkMargin ;
    result = 37 * result + HashcodeUtil.hashCode(this.labelLinkPaint) ;
    result = 37 * result + HashcodeUtil.hashCode(this.labelLinkStroke) ;
    result = 37 * result + HashcodeUtil.hashCode(this.simpleLabels) ;
    result = 37 * result + this.minimumArcAngleToDraw ;
    result = 37 * result + HashcodeUtil.hashCode(this.pie3D) ;
    result = 37 * result + this.depth3D ;
    result = 37 * result + HashcodeUtil.hashCode(this.drawAsRing) ;
    result = 37 * result + 100 * this.ringLengthFactor ;

    return (int) result ;
  }

  /**
   * <zh>ؿ¡Ķ.</zh>
   * <en>Creates and returns a copy of this object.</en>
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the Cloneable interface. Subclasses
   *               that override the clone method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object.</en>
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "PieRender" ;
  }

  /**
   * Provides serialization support.
   *
   * @param stream  the output stream.
   *
   * @throws IOException  if there is an I/O error.
   */
  private void writeObject(ObjectOutputStream stream) throws IOException {
    stream.defaultWriteObject() ;

    SerialUtil.writePaint(stream, this.pieShadowPaint) ;
    SerialUtil.writePaint(stream, this.labelBackgroundPaint) ;
    SerialUtil.writePaint(stream, this.labelOutlinePaint) ;
    SerialUtil.writePaint(stream, this.labelShadowPaint) ;
    SerialUtil.writePaint(stream, this.labelLinkPaint) ;

    SerialUtil.writeStroke(stream, this.labelOutlineStroke) ;
    SerialUtil.writeStroke(stream, this.labelLinkStroke) ;
  }

  /**
   * Provides serialization support.
   *
   * @param stream  the input stream.
   *
   * @throws IOException  if there is an I/O error.
   * @throws ClassNotFoundException  if there is a classpath problem.
   */
  private void readObject(ObjectInputStream stream) throws IOException, ClassNotFoundException {
    stream.defaultReadObject() ;

    this.pieShadowPaint = SerialUtil.readPaint(stream) ;
    this.labelBackgroundPaint = SerialUtil.readPaint(stream) ;
    this.labelOutlinePaint = SerialUtil.readPaint(stream) ;
    this.labelShadowPaint = SerialUtil.readPaint(stream) ;
    this.labelLinkPaint = SerialUtil.readPaint(stream) ;

    this.labelOutlineStroke = SerialUtil.readStroke(stream) ;
    this.labelLinkStroke = SerialUtil.readStroke(stream) ;
  }

}
