package com.ijchart.xychart.render.xy ;

import java.awt.Shape ;
import java.awt.geom.Line2D ;
import java.awt.geom.Rectangle2D ;
import java.io.Serializable ;

import com.ijchart.xychart.axis.value.AbstractValueAxis ;
import com.ijchart.xychart.chartinfo.RenderShapeCollection ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.constant.LegendType ;
import com.ijchart.xychart.data.IChartData ;
import com.ijchart.xychart.data.YIntervalChartData ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.shape.DblHeadMatchstickChartShape ;
import com.ijchart.xychart.shape.IChartShape ;
import com.ijchart.xychart.ui.RectangleEdge ;
import com.ijchart.xychart.ui.supplier.IDrawingSupplier ;
import com.ijchart.xychart.util.HashcodeUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>yֵͼ.</zh>
 * <en>Draw Y-Interval chart shapes.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class YIntervalRenderer extends AbstractXYRender implements Cloneable, Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = 9068816734821305548L ;

  /**
   * <zh>ȱʡеͼǷʾ.</zh>
   * <en>A default flag indicating whether or not shapes are drawn for all points.</en>
   */
  public static final boolean DEFAULT_SHAPES_VISIBLE = true ;

  /**
   * <zh>еͼǷʾ.</zh>
   * <en>A flag indicating whether or not shapes are drawn for all points.</en>
   */
  private boolean shapesVisible ;

  /**
   * <zh>.</zh>
   * <en>constructor.</en>
   *
   * @param dataset IMatrixDataset <zh>ݼ (Ϊnull).</zh>
   * <en>The dataset (null not permitted).</en>
   */
  public YIntervalRenderer(IMatrixDataset dataset) {
    super(dataset) ;

    this.shapesVisible = DEFAULT_SHAPES_VISIBLE ;
    setBaseItemDrawBorder(true) ;
    setAutoSelectBorderPaint(true) ;
  }

  /**
   * <zh>еͼǷʾĲֵ.</zh>
   * <en>Returns the base 'shape visible' attribute.</en>
   *
   * @return boolean <zh>ֵ.</zh>
   * <en>The base flag.</en>
   */
  public boolean isShapesVisible() {
    return this.shapesVisible ;
  }

  /**
   * <zh>еͼǷʾĲֵ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the base 'shapes visible' flag and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param flag boolean <zh>ֵ.</zh>
   * <en>The flag.</en>
   */
  public void setShapesVisible(boolean flag) {
    if (this.shapesVisible != flag) {
      this.shapesVisible = flag ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>еͼ.</zh>
   * <en>Draws all data items.</en>
   *
   * @param xAxis AbstractValueAxis <zh>xֵ (Ϊnull).</zh>
   * <en>The x value axis (null not permitted).</en>
   * @param xAxisEdge RectangleEdge <zh>xλ(Ϊnull).</zh>
   * <en>The x axis edge.</en>
   * @param yAxis AbstractValueAxis <zh>yֵ (Ϊnull).</zh>
   * <en>The y value axis (null not permitted).</en>
   * @param yAxisEdge RectangleEdge <zh>yλ (Ϊnull).</zh>
   * <en>The y axis edge (null not permitted).</en>
   * @param dataArea Rectangle2D <zh> (Ϊnull).</zh>
   * <en>The chart data area (null not permitted).</en>
   * @param orientation ChartOrientation <zh>ͼķ (Ϊnull).</zh>
   * <en>The chart orientation (null not permitted).</en>
   * @param renderShapes RenderShapeCollection <zh>renderͼμ (Ϊnull).</zh>
   * <en>Collects information about the render (null not permitted).</en>
   * @param alpha float <zh>alphaֵ,ڱ[0.0,1.0].</zh>
   * <en>The alpha transparency (in the range 0.0f to 1.0f, where 0.0f is fully transparent,
   * and 1.0f is fully opaque).</en>
   */
  public void drawAllItems(AbstractValueAxis xAxis, RectangleEdge xAxisEdge,
                           AbstractValueAxis yAxis, RectangleEdge yAxisEdge,
                           Rectangle2D dataArea, ChartOrientation orientation,
                           RenderShapeCollection renderShapes, float alpha) {
    if (xAxis == null) {
      throw new IllegalArgumentException("Null 'xAxis' argument.") ;
    }
    if (xAxisEdge == null) {
      throw new IllegalArgumentException("Null 'xAxisEdge' argument.") ;
    }
    if (yAxis == null) {
      throw new IllegalArgumentException("Null 'yAxis' argument.") ;
    }
    if (yAxisEdge == null) {
      throw new IllegalArgumentException("Null 'yAxisEdge' argument.") ;
    }
    if (dataArea == null) {
      throw new IllegalArgumentException("Null 'dataArea' argument.") ;
    }
    if (orientation == null) {
      throw new IllegalArgumentException("Null 'orientation' argument.") ;
    }
    if (renderShapes == null) {
      throw new IllegalArgumentException("Null 'renderShapes' argument.") ;
    }
    if (alpha < 0.0f || alpha > 1.0f) {
      throw new IllegalArgumentException("The 'alpha' value must be in the range 0.0f to 1.0f.") ;
    }

    IMatrixDataset dataset = getDataset() ;
    if (dataset.isEmpty()) {
      return ;
    }

    int rows = dataset.getRowCount() ;
    this.fillPaints = getFillPaints(rows) ;
    getBorderPaints(rows) ;
    this.borderStrokes = getBorderStrokes(rows) ;

    //ߺͼ
    IDrawingSupplier supplier = getDrawingSupplier() ;
    for (int i = 0 ; i < rows ; i++) {
      Shape shape = supplier.getNextShape() ;
      if (!this.shapesVisible) {
        shape = null ;
      }

      int currRowCols = dataset.getColumnCount(i) ;
      for (int j = 0 ; j < currRowCols - 1 ; j++) {
        IChartData currData = dataset.getData(i, j) ;
        if (currData != null && currData instanceof YIntervalChartData) {
          double currX = ((YIntervalChartData) currData).getX() ;
          double currYLow = ((YIntervalChartData) currData).getYLow() ;
          double currYHigh = ((YIntervalChartData) currData).getYHigh() ;
          if (Double.isNaN(currX) || Double.isNaN(currYLow) || Double.isNaN(currYHigh)) {
            renderShapes.addDataItemShape(null) ;
            continue ;
          }

          double currXPos = xAxis.valueToJava2D(dataArea, xAxisEdge, currX, false) ;
          double currYLowPos = yAxis.valueToJava2D(dataArea, yAxisEdge, currYLow, false) ;
          double currYHighPos = yAxis.valueToJava2D(dataArea, yAxisEdge, currYHigh, false) ;

          //ֵ᷶Χֵ
          if (currXPos == AbstractValueAxis.INVALID_COORDINATE_VALUE
              || currYLowPos == AbstractValueAxis.INVALID_COORDINATE_VALUE
              || currYHighPos == AbstractValueAxis.INVALID_COORDINATE_VALUE) {
            renderShapes.addDataItemShape(null) ;
            continue ;
          }

          Line2D line = null ;
          if (orientation.equals(ChartOrientation.VERTICAL)) { //ֱ
            line = new Line2D.Double(currXPos, currYLowPos, currXPos, currYHighPos) ;
          } else if (orientation.equals(ChartOrientation.HORIZONTAL)) { //ˮƽ
            line = new Line2D.Double(currYLowPos, currXPos, currYHighPos, currXPos) ;
          }

          DblHeadMatchstickChartShape matchShape = new DblHeadMatchstickChartShape(shape, line) ;
          IChartShape chartShape = createItemShape(orientation, yAxis, i, j, null,
                                                   fillPaints[i], fillPaints[i],
                                                   borderStrokes[i], alpha, matchShape) ;

          renderShapes.addDataItemShape(chartShape) ;
        } else {
          renderShapes.addDataItemShape(null) ;
        }
      }
    }
  }

  /**
   * <zh>ͼ.</zh>
   * <en>Return the legend type.</en>
   *
   * @return LegendType <zh>ͼ.</zh>
   * <en>The legend type.</en>
   */
  public LegendType getLegendType() {
    return LegendType.ROW ;
  }

  /**
   * <zh>ֻͼ,ͼε.</zh>
   * <en>Create a shape with no attribute specified.</en>
   *
   * @param rect Rectangle2D <zh>ͼľ (Ϊnull).</zh>
   * <en>The bar rectangle (null not permitted).</en>
   * @param text String <zh>Ҫʾ (Ϊnull).</zh>
   * <en>The text (null permitted).</en>
   * @param appendParam Object <zh>ӵĲ,ڴݶĶ (Ϊnull).</zh>
   * <en>The append parameter,used for special propose (null permitted).</en>
   *
   * @return IChartShape <zh>ͼ.</zh>
   * <en>The shape.</en>
   */
  protected IChartShape createItemShapeOnly(Rectangle2D rect, String text,
                                            Object appendParam) {
    if (appendParam == null) {
      throw new IllegalArgumentException("Null 'appendParam' argument.") ;
    }

    return (IChartShape) appendParam ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Tests this instance for equality with an arbitrary object.</en>
   *
   * @param obj  the object (null permitted).
   *
   * @return A boolean.
   */
  public boolean equals(Object obj) {
    if (obj == this) {
      return true ;
    }
    if (!(obj instanceof XYLineAndShapeRender)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    YIntervalRenderer that = (YIntervalRenderer) obj ;
    if (this.shapesVisible != that.shapesVisible) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return int <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    int result = super.hashCode() ;
    result = result * 37 + HashcodeUtil.hashCode(this.shapesVisible) ;

    return result ;

  }

  /**
   * <zh>ؿ¡Ķ.</zh>
   * <en>Creates and returns a copy of this object.</en>
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the Cloneable interface. Subclasses
   *               that override the clone method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object.</en>
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "YIntervalRenderer" ;
  }

}
