package com.ijchart.xychart.shape ;

import java.awt.Font ;
import java.awt.Graphics2D ;
import java.awt.Shape ;
import java.awt.geom.AffineTransform ;
import java.awt.geom.Point2D ;
import java.awt.geom.Rectangle2D ;
import java.io.IOException ;
import java.io.ObjectInputStream ;
import java.io.ObjectOutputStream ;
import java.io.Serializable ;
import java.text.BreakIterator ;
import java.util.ArrayList ;
import java.util.List ;

import com.ijchart.xychart.constant.HorizontalAlignment ;
import com.ijchart.xychart.constant.IAngle ;
import com.ijchart.xychart.constant.TextDirection ;
import com.ijchart.xychart.constant.TextPosition ;
import com.ijchart.xychart.constant.VerticalAlignment ;
import com.ijchart.xychart.ui.Graphics2DState ;
import com.ijchart.xychart.ui.RectangleInsets ;
import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.HashcodeUtil ;
import com.ijchart.xychart.util.SerialUtil ;
import com.ijchart.xychart.util.StringUtil ;
import com.ijchart.xychart.util.StrokeUtil ;
import com.ijchart.xychart.util.TextUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>һڲⲿһЩ,Ϊ.</zh>
 * <en>Draw a text inside or outside a Rectangle,with or not have a border.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public abstract class AbstractRectangleTextChartShape extends AbstractTextChartShape implements
  Cloneable, Serializable {

  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = -4114142933809897610L ;

  /**
   * <zh>Զеȱʡֵ.</zh>
   * <en>The default auto wrap.</en>
   */
  public static final boolean DEFAULT_DRAW_AUTO_WRAP = true ;

  /**
   * <zh>ǩȱʡʾ,Ĭȫʾ.</zh>
   * <en>The default maximum number of lines for text (default is all).</en>
   */
  public static final int DEFAULT_TEXT_MAX_SHOW_LINE = Integer.MAX_VALUE ;

  /**
   * <zh>ȱʡǷǿͼڲַ.</zh>
   * <en>A default boolean flag controls whether or not the inside text is no wrap.</en>
   */
  public static final boolean DEFAULT_FORCE_INSIDE_TEXT_NO_WRAP = false ;

  /**
   * <zh>ȱʡǷǿʾеַ,ַ̫,ں"."ַ.</zh>
   * <en>A default boolean flag controls whether or not
   *  draw text even if it's too long to show.</en>
   */
  public static final boolean DEFAULT_FORCE_SHOW_ALL = false ;

  /**
   * <zh>ֵĿд.</zh>
   * <en>The text draw area.</en>
   */
  private transient Rectangle2D drawArea ;

  /**
   * <zh>ƵַһʾʱǷԶ.</zh>
   * <en>A boolean flag controls whether or not the string is auto wrap when text can not be drawn
   *  in one line.</en>
   */
  private boolean autoWrap ;

  /**
   * <zh>Ƿǿͼڲַ.</zh>
   * <en>A boolean flag controls whether or not the inside text is no wrap.</en>
   */
  private boolean forceInsideTextNoWrap ;

  /**
   * <zh>Ƿǿʾеַ,ַ̫,ں"."ַ.</zh>
   * <en>A boolean flag controls whether or not draw text even if it's too long to show.</en>
   */
  private boolean forceShowAll ;

  /**
   * <zh>ǩʾ.</zh>
   * <en>The maximum number of lines for text.</en>
   */
  private int textMaxShowLine ;

  /**
   * <zh>drawAreaʾtext.</zh>
   * <en>Draw text in area drawArea.</en>
   *
   * @param drawArea Rectangle2D <zh> (Ϊnull).</zh>
   * <en>The draw area (null not permitted).</en>
   * @param text String <zh>Ҫʾ (Ϊnull).</zh>
   * <en>The text (null permitted).</en>
   */
  protected AbstractRectangleTextChartShape(Rectangle2D drawArea, String text) {
    this(drawArea, text, null) ;
  }

  /**
   * <zh>ֵĿд.</zh>
   * <en>Returns the text draw area.</en>
   *
   * @return Rectangle2D <zh>εֿд (Ϊnull).</zh>
   * <en>The text draw area (never null).</en>
   */
  public Rectangle2D getDrawArea() {
    return this.drawArea ;
  }

  /**
   * <zh>ֿд,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the text draw area,and sends a {@link com.ijchart.xychart.event.ChartShapeChangeEvent}
   *  to all registered listeners.</en>
   *
   * @param drawArea Rectangle2D <zh>εֿд (Ϊnull).</zh>
   * <en>The text draw area (null not permitted).</en>
   */
  public void setDrawArea(Rectangle2D drawArea) {
    if (drawArea == null) {
      throw new IllegalArgumentException("Null 'drawArea' argument.") ;
    }
    this.drawArea = drawArea ;
    fireChartShapeChanged() ;
  }

  /**
   * <zh>صַһʾʱǷԶ.</zh>
   * <en>Returns a boolean flag controls whether or not the string is auto wrap when text can not be
   *  drawn in one line.</en>
   * @return boolean
   */
  public boolean isAutoWrap() {
    return autoWrap ;
  }

  /**
   * <zh>õַһʾʱǷԶ,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets a boolean flag controls whether or not the string is auto wrap when text can not be
   *  drawn in one line,and sends a {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all
   * registered listeners.</en>
   *
   * @param autoWrap A boolean.
   */
  public void setAutoWrap(boolean autoWrap) {
    if (this.autoWrap != autoWrap) {
      this.autoWrap = autoWrap ;
      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>Ƿǿͼڲַ.</zh>
   * <en>Returns whether or not the inside text is no wrap.</en>
   *
   * @return boolean
   */
  public boolean isForceInsideTextNoWrap() {
    return this.forceInsideTextNoWrap ;
  }

  /**
   * <zh>Ƿǿͼڲַ,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets whether or not the inside text is no wrap,and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param flag boolean
   */
  public void setForceInsideTextNoWrap(boolean flag) {
    if (this.forceInsideTextNoWrap != flag) {
      this.forceInsideTextNoWrap = flag ;
      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>رǩʾ.</zh>
   * <en>Returns the maximum number of lines for text.</en>
   *
   * @return int <zh>ʾ,ڵ1.</zh>
   * <en>The maximum number of lines for text,must greater or equal to 1 .</en>
   */
  public int getTextMaxShowLine() {
    return this.textMaxShowLine ;
  }

  /**
   * <zh>ñǩʾ,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the maximum number of lines for text,and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param textMaxShowLine int <zh>ʾ,ڵ1.</zh>
   * <en>The maximum number of lines for text,must greater or equal to 1 .</en>
   */
  public void setTextMaxShowLine(int textMaxShowLine) {
    if (textMaxShowLine < 1) {
      throw new IllegalArgumentException("Must be textMaxShowLine>0;") ;
    }

    if (this.textMaxShowLine != textMaxShowLine) {
      this.textMaxShowLine = textMaxShowLine ;
      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>Ƿǿʾеַ,ַ̫,ں"."ַ.</zh>
   * <en>Returns whether or not draw text even if it's too long to show.</en>
   *
   * @return boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  public boolean isForceShowAll() {
    return this.forceShowAll ;
  }

  /**
   * <zh>Ƿǿʾеַ,ַ̫,ں"."ַ,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the boolean flag whether or not draw text even if it's too long to show,and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param flag boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  public void setForceShowAllText(boolean flag) {
    if (this.forceShowAll != flag) {
      this.forceShowAll = flag ;
      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>drawAreaʾtext.</zh>
   * <en>Draw text in area drawArea.</en>
   *
   * @param drawArea Rectangle2D <zh> (Ϊnull).</zh>
   * <en>The draw area (null not permitted).</en>
   * @param text String <zh>Ҫʾ (Ϊnull).</zh>
   * <en>The text (null permitted).</en>
   * @param font <zh> (Ϊnull),fontΪnull,
   * Ϊ{@link com.ijchart.xychart.shape.IChartShape#DEFAULT_TEXT_FONT}.</zh>
   * <en>The specified font (null permitted).If font is null,
   * {@link com.ijchart.xychart.shape.IChartShape#DEFAULT_TEXT_FONT} is used.</en>
   */
  protected AbstractRectangleTextChartShape(Rectangle2D drawArea, String text, Font font) {
    if (drawArea == null) {
      throw new IllegalArgumentException("Null 'drawArea' argument.") ;
    }

    setText(text) ;
    this.drawArea = (Rectangle2D) drawArea.clone() ;
    if (font == null) {
      setTextFont(DEFAULT_TEXT_FONT) ;
    } else {
      setTextFont(font) ;
    }

    this.forceInsideTextNoWrap = DEFAULT_FORCE_INSIDE_TEXT_NO_WRAP ;
    this.autoWrap = DEFAULT_DRAW_AUTO_WRAP ;
    this.textMaxShowLine = DEFAULT_TEXT_MAX_SHOW_LINE ;
    this.forceShowAll = DEFAULT_FORCE_SHOW_ALL ;
  }

  /**
   * <zh>ֲ.</zh>
   * <en>Draw text.</en>
   * @param g2 Graphics2D <zh>ͼ豸(Ļӡ) (Ϊnull)</zh>
   * <en>the graphics device (such as the screen or the printer) (null not permitted).</en>
   * @param area Rectangle2D <zh> (Ϊnull).</zh>
   * <en>The draw area (null not permitted).</en>
   */
  protected void drawText(Graphics2D g2, Rectangle2D area) {
    if (g2 == null) {
      throw new IllegalArgumentException("Null 'g2' argument.") ;
    }
    if (area == null) {
      throw new IllegalArgumentException("Null 'area' argument.") ;
    }

    String[] strArr = null ;
    TextPosition position = getTextPosition() ;
    if (position.equals(TextPosition.NONE)) {
      return ;
    }

    if (position.equals(TextPosition.OUTSIDE_TOP)
        || position.equals(TextPosition.OUTSIDE_BOTTOM)
        || position.equals(TextPosition.OUTSIDE_LEFT)
        || position.equals(TextPosition.OUTSIDE_RIGHT)) { //ⲿ,Ҫֹ
      strArr = splitStringManual() ;
    } else { //ڲ,Զ
      TextDirection direction = getTextDirection() ;
      if (direction.equals(TextDirection.HORIZONTAL)) {
        strArr = splitString(g2, area) ;
      } else if (direction.equals(TextDirection.VERTICAL)) {
        strArr = splitString(g2, new Rectangle2D.Double(0, 0, area.getHeight(), area.getWidth())) ;
      }
    }

    if (strArr == null || strArr.length == 0) {
      return ;
    }

    if (strArr.length > textMaxShowLine) {
      String[] tmpArr = new String[textMaxShowLine] ;
      System.arraycopy(strArr, 0, tmpArr, 0, textMaxShowLine) ;
      strArr = tmpArr ;
    }

    //save old state
    Graphics2DState state = new Graphics2DState(g2) ;
    state.saveOldState() ;

    g2.setPaint(getTextPaint()) ;
    g2.setFont(getTextFont()) ;

    double totalHeight = 0 ; //ַܸ߶
    Rectangle2D[] boundArr = new Rectangle2D[strArr.length] ;
    for (int i = 0 ; i < strArr.length ; i++) {
      boundArr[i] = TextUtil.getStringBoundWithFm(g2, strArr[i]) ;
      totalHeight += boundArr[i].getHeight() ;
    }

    //ͼⲿ
    if (position.equals(TextPosition.OUTSIDE_TOP)) {
      TextDirection direction = getTextDirection() ;

      if (direction.equals(TextDirection.HORIZONTAL)) { //ˮƽ
        float height = -1 ;
        HorizontalAlignment hAlign = getTextHorizontalAlign() ;
        if (hAlign.equals(HorizontalAlignment.LEFT)) {
          for (int i = strArr.length - 1 ; i >= 0 ; i--) {
            double positionY = height - (boundArr[i].getHeight() - Math.abs(boundArr[i].getY())) ;
            g2.drawString(strArr[i], 0, (float) positionY) ;
            height -= boundArr[i].getHeight() ;
          }
        } else if (hAlign.equals(HorizontalAlignment.CENTER)) {
          for (int i = strArr.length - 1 ; i >= 0 ; i--) {
            double positionY = height - (boundArr[i].getHeight() - Math.abs(boundArr[i].getY())) ;
            float textX = (float) (0.5 * (area.getWidth() - boundArr[i].getWidth())) ;
            g2.drawString(strArr[i], textX, (float) positionY) ;
            height -= boundArr[i].getHeight() ;
          }
        } else if (hAlign.equals(HorizontalAlignment.RIGHT)) {
          for (int i = strArr.length - 1 ; i >= 0 ; i--) {
            double positionY = height - (boundArr[i].getHeight() - Math.abs(boundArr[i].getY())) ;
            float textX = (float) (area.getWidth() - boundArr[i].getWidth()) ;
            g2.drawString(strArr[i], textX, (float) positionY) ;
            height -= boundArr[i].getHeight() ;
          }
        }
      } else { //ֱ
        g2.rotate(IAngle.ANGLE_UP_90) ;

        double height = 0 ;
        VerticalAlignment vAlign = getTextVerticalAlign() ;
        if (vAlign.equals(VerticalAlignment.TOP)) { //ֱ˶
          height = 0 ;
        } else if (vAlign.equals(VerticalAlignment.CENTER)) {
          height = 0.5 * (area.getWidth() - totalHeight) ;
        } else if (vAlign.equals(VerticalAlignment.BOTTOM)) {
          height = area.getWidth() - totalHeight ;
        }

        for (int i = 0 ; i < strArr.length ; i++) {
          g2.drawString(strArr[i], 0, (float) (height + Math.abs(boundArr[i].getY()))) ;
          height += boundArr[i].getHeight() ;
        }

        g2.rotate(IAngle.ANGLE_DOWN_90) ;
      }
    } else if (position.equals(TextPosition.OUTSIDE_BOTTOM)) {
      TextDirection direction = getTextDirection() ;

      if (direction.equals(TextDirection.HORIZONTAL)) { //ˮƽ
        float height = (float) (this.drawArea.getHeight()) ;
        HorizontalAlignment hAlign = getTextHorizontalAlign() ;
        if (hAlign.equals(HorizontalAlignment.LEFT)) {
          for (int i = 0 ; i < strArr.length ; i++) {
            double positionY = height + Math.abs(boundArr[i].getY()) ;
            g2.drawString(strArr[i], 0, (float) positionY) ;
            height += boundArr[i].getHeight() ;
          }
        } else if (hAlign.equals(HorizontalAlignment.CENTER)) {
          for (int i = 0 ; i < strArr.length ; i++) {
            double positionY = height + Math.abs(boundArr[i].getY()) ;
            float textX = (float) (0.5 * (area.getWidth() - boundArr[i].getWidth())) ;
            g2.drawString(strArr[i], textX, (float) positionY) ;
            height += boundArr[i].getHeight() ;
          }
        } else if (hAlign.equals(HorizontalAlignment.RIGHT)) {
          for (int i = 0 ; i < strArr.length ; i++) {
            double positionY = height + Math.abs(boundArr[i].getY()) ;
            float textX = (float) (area.getWidth() - boundArr[i].getWidth()) ;
            g2.drawString(strArr[i], textX, (float) positionY) ;
            height += boundArr[i].getHeight() ;
          }
        }
      } else { //ֱ
        double height = 0 ;
        VerticalAlignment vAlign = getTextVerticalAlign() ;
        if (vAlign.equals(VerticalAlignment.TOP)) { //ֱ˶
          height = 0 ;
        } else if (vAlign.equals(VerticalAlignment.CENTER)) {
          height = 0.5 * (area.getWidth() - totalHeight) ;
        } else if (vAlign.equals(VerticalAlignment.BOTTOM)) {
          height = area.getWidth() - totalHeight ;
        }

        g2.translate(0, this.drawArea.getHeight()) ;
        for (int i = 0 ; i < strArr.length ; i++) {
          g2.translate(0, boundArr[i].getWidth()) ;
          g2.rotate(IAngle.ANGLE_UP_90) ;

          g2.drawString(strArr[i], 0, (float) (height + Math.abs(boundArr[i].getY()))) ;
          height += boundArr[i].getHeight() ;

          g2.rotate(IAngle.ANGLE_DOWN_90) ;
          g2.translate(0, -boundArr[i].getWidth()) ;
        }
        g2.translate(0, -this.drawArea.getHeight()) ;

      }

    } else if (position.equals(TextPosition.OUTSIDE_LEFT)) { //ͼⲿ
      TextDirection direction = getTextDirection() ;

      if (direction.equals(TextDirection.HORIZONTAL)) { //ˮƽ
        double beginY = 0 ; //ʼ߶
        if (getTextVerticalAlign().equals(VerticalAlignment.TOP)) {
          beginY = 0 ;
        } else if (getTextVerticalAlign().equals(VerticalAlignment.CENTER)) {
          beginY = (area.getHeight() - totalHeight) / 2 ;
        } else if (getTextVerticalAlign().equals(VerticalAlignment.BOTTOM)) {
          beginY = area.getHeight() - totalHeight ;
        }

        for (int i = 0 ; i < strArr.length ; i++) {
          g2.drawString(strArr[i], (float) ( -boundArr[i].getWidth() - 2),
                        (float) (beginY + Math.abs(boundArr[i].getY()))) ;
          beginY += boundArr[i].getHeight() ;
        }

      } else { //ֱ
        g2.translate(0, area.getHeight()) ;

        VerticalAlignment vAlign = getTextVerticalAlign() ;
        float height = 0 ;
        for (int i = strArr.length - 1 ; i >= 0 ; i--) {
          double beginY = 0 ;
          if (vAlign.equals(VerticalAlignment.TOP)) { //ֱ˶
            beginY = 0 ;
          } else if (vAlign.equals(VerticalAlignment.CENTER)) {
            beginY = 0.5 * (boundArr[i].getWidth() - area.getHeight()) ;
          } else if (vAlign.equals(VerticalAlignment.BOTTOM)) {
            beginY = boundArr[i].getWidth() - area.getHeight() ;
          }

          g2.translate(0, beginY) ;
          g2.rotate(IAngle.ANGLE_UP_90) ;
          double positionY = height - (boundArr[i].getHeight() - Math.abs(boundArr[i].getY())) ;
          g2.drawString(strArr[i], 0, (float) positionY) ;
          height -= boundArr[i].getHeight() ;
          g2.rotate(IAngle.ANGLE_DOWN_90) ;
          g2.translate(0, -beginY) ;
        }

        g2.translate( -area.getWidth(), -area.getHeight()) ;
      }
    } else if (position.equals(TextPosition.OUTSIDE_RIGHT)) { //ͼⲿҲ
      TextDirection direction = getTextDirection() ;

      if (direction.equals(TextDirection.HORIZONTAL)) { //ˮƽ
        double beginY = 0 ; //ʼ߶
        if (getTextVerticalAlign().equals(VerticalAlignment.TOP)) {
          beginY = 0 ;
        } else if (getTextVerticalAlign().equals(VerticalAlignment.CENTER)) {
          beginY = (area.getHeight() - totalHeight) / 2 ;
        } else if (getTextVerticalAlign().equals(VerticalAlignment.BOTTOM)) {
          beginY = area.getHeight() - totalHeight ;
        }

        for (int i = 0 ; i < strArr.length ; i++) {
          g2.drawString(strArr[i], (float) area.getWidth(),
                        (float) (beginY + Math.abs(boundArr[i].getY()))) ;
          beginY += boundArr[i].getHeight() ;
        }

      } else { //ֱ
        g2.translate(area.getWidth(), area.getHeight()) ;

        VerticalAlignment vAlign = getTextVerticalAlign() ;
        float height = 0 ;
        for (int i = 0 ; i < strArr.length ; i++) {
          double beginY = 0 ;
          if (vAlign.equals(VerticalAlignment.TOP)) { //ֱ˶
            beginY = 0 ;
          } else if (vAlign.equals(VerticalAlignment.CENTER)) {
            beginY = 0.5 * (boundArr[i].getWidth() - area.getHeight()) ;
          } else if (vAlign.equals(VerticalAlignment.BOTTOM)) {
            beginY = boundArr[i].getWidth() - area.getHeight() ;
          }

          g2.translate(0, beginY) ;
          g2.rotate(IAngle.ANGLE_UP_90) ;
          g2.drawString(strArr[i], 0, (float) (height + Math.abs(boundArr[i].getY()))) ;
          height += boundArr[i].getHeight() ;
          g2.rotate(IAngle.ANGLE_DOWN_90) ;
          g2.translate(0, -beginY) ;
        }

        g2.translate( -area.getWidth(), -area.getHeight()) ;
      }
    } else if (position.equals(TextPosition.INSIDE)) { //ͼڲ
      TextDirection direction = getTextDirection() ;
      if (direction.equals(TextDirection.HORIZONTAL)) { //ˮƽ
        area.setRect(area.getX(), area.getY(), area.getWidth() + 0.5, area.getHeight() + 0.5) ; //Ŵȸ߶

        double beginY = 0 ; //ʼ߶
        if (getTextVerticalAlign().equals(VerticalAlignment.TOP)) {
          beginY = 0 ;
        } else if (getTextVerticalAlign().equals(VerticalAlignment.CENTER)) {
          beginY = (area.getHeight() - totalHeight) / 2 ;
        } else if (getTextVerticalAlign().equals(VerticalAlignment.BOTTOM)) {
          beginY = area.getHeight() - totalHeight - 2 ;
        }

        if (beginY < 0) {
          beginY = 0 ;
        }

        for (int i = 0 ; i < strArr.length ; i++) {
          double beginX = 0 ; //ʼ
          HorizontalAlignment horizontalAlignment = getTextHorizontalAlign() ;
          if (horizontalAlignment.equals(HorizontalAlignment.LEFT)) {
            beginX = 0 ;
          } else if (horizontalAlignment.equals(HorizontalAlignment.CENTER)) {
            beginX = (area.getWidth() - boundArr[i].getWidth()) / 2 ;
          } else if (horizontalAlignment.equals(HorizontalAlignment.RIGHT)) {
            beginX = area.getWidth() - boundArr[i].getWidth() ;
          }
          if (beginX < 0 && !this.forceInsideTextNoWrap && !this.forceShowAll) {
            beginX = 0 ;
          }

          if (beginY + boundArr[i].getHeight() > area.getHeight() && i > 0) {
            break ;
          } else {
            if (i != strArr.length - 1) {
              if (beginY + boundArr[i].getHeight() + boundArr[i + 1].getHeight() > area.getHeight()
                  && !forceShowAll) {
                if (boundArr[i].getWidth() < area.getWidth()) { //ַ
                  String tmp = strArr[i] + "..." ;
                  for (int m = 0 ; m < strArr[i].length() ; m++) {
                    if (TextUtil.getStringBoundWithFm(g2, tmp).getWidth() < area.getWidth()) {
                      break ;
                    }
                    tmp = strArr[i].substring(0, strArr[i].length() - m) + "..." ;
                  }
                  strArr[i] = tmp ;

                  if (TextUtil.getStringBoundWithFm(g2, strArr[i]).getWidth() > area.getWidth()) {
                    strArr[i] = strArr[i].substring(0, strArr[i].length() - 1) ;
                  }
                  if (TextUtil.getStringBoundWithFm(g2, strArr[i]).getWidth() > area.getWidth()) {
                    strArr[i] = strArr[i].substring(0, strArr[i].length() - 1) ;
                  }
                } else {
                  if (strArr[i].length() > 3) {
                    strArr[i] = strArr[i].substring(0, strArr[i].length() - 2) + "..." ;
                  } else {
                    if (strArr[i].length() > 1) {
                      strArr[i] = strArr[i].substring(0, strArr[i].length() - 1) + "." ;
                    }
                  }
                }
              }
            }

            g2.drawString(strArr[i], (float) beginX, (float) (beginY + Math.abs(boundArr[i].getY()))) ;
            beginY += boundArr[i].getHeight() ;

          }
        }
      } else if (direction.equals(TextDirection.VERTICAL)) {
        g2.translate(area.getWidth() * 0.5 + 0.5 * totalHeight, area.getHeight()) ;

        VerticalAlignment vAlign = getTextVerticalAlign() ;
        float height = 0 ;
        for (int i = strArr.length - 1 ; i >= 0 ; i--) {
          double beginY = 0 ;
          if (vAlign.equals(VerticalAlignment.TOP)) { //ֱ˶
            beginY = 0 ;
          } else if (vAlign.equals(VerticalAlignment.CENTER)) {
            beginY = 0.5 * (boundArr[i].getWidth() - area.getHeight()) ;
          } else if (vAlign.equals(VerticalAlignment.BOTTOM)) {
            beginY = boundArr[i].getWidth() - area.getHeight() ;
          }

          g2.translate(0, beginY) ;
          g2.rotate(IAngle.ANGLE_UP_90) ;
          double positionY = height - (boundArr[i].getHeight() - Math.abs(boundArr[i].getY())) ;
          g2.drawString(strArr[i], 0, (float) positionY) ;
          height -= boundArr[i].getHeight() ;
          g2.rotate(IAngle.ANGLE_DOWN_90) ;
          g2.translate(0, -beginY) ;
        }

        g2.translate( -area.getWidth(), -area.getHeight()) ;

      }
    }

    state.restoreOldState() ;
  }

  /**
   * <zh>rectʾ.</zh>
   * <en>Draw text on rectangle area.</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(Ļӡ) (Ϊnull)</zh>
   * <en>the graphics device (such as the screen or the printer) (null not permitted).</en>
   * @param drawArea Rectangle2D <zh> (Ϊnull).</zh>
   * <en>The draw area (null not permitted).</en>
   */
  protected void drawTextWithTrimMarginRect(Graphics2D g2, Rectangle2D drawArea) {
    if (g2 == null) {
      throw new IllegalArgumentException("Null 'g2' argument.") ;
    }
    if (drawArea == null) {
      throw new IllegalArgumentException("Null 'drawArea' argument.") ;
    }

    if (isDrawText()) {
      float strokeWidth = 0 ;
      if (isDrawBorder()) {
        strokeWidth = StrokeUtil.getStrokeWidth(getBorderStroke()) ;
      }

      RectangleInsets margin = getMargin() ;
      TextPosition position = getTextPosition() ;
      if (position.equals(TextPosition.NONE)) {
        return ;
      }

      //ͼⲿ
      if (position.equals(TextPosition.OUTSIDE_TOP)
          || position.equals(TextPosition.OUTSIDE_BOTTOM)
          || position.equals(TextPosition.OUTSIDE_LEFT)
          || position.equals(TextPosition.OUTSIDE_RIGHT)) {
        drawText(g2, getDrawArea()) ;
      } else if (position.equals(TextPosition.INSIDE)) { //ͼڲ
        g2.translate(drawArea.getX(), drawArea.getY()) ;
        drawArea.setRect(0, 0, drawArea.getWidth() - strokeWidth * 2,
                         drawArea.getHeight() - strokeWidth * 2) ;
        g2.translate(strokeWidth, strokeWidth) ;

        Rectangle2D trimArea = margin.getTrimArea(drawArea) ;
        trimArea.setRect(0, 0, trimArea.getWidth(), trimArea.getHeight()) ;

        //translate margin area
        g2.translate(margin.getLeft(), margin.getTop()) ;

        drawText(g2, trimArea) ;
      }
    }
  }

  /**
   * <zh>ƶͼ,X᷽ƫtx,Y᷽ƫty,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Concatenates the current Graphics2D Transform with a translation transform.and sends
   * a {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.
   * Subsequent rendering is translated by the specified
   * distance relative to the previous position.
   * This is equivalent to calling transform(T), where T is an
   * AffineTransform represented by the following matrix:
   * <pre>
   *		[   1    0    tx  ]
   *		[   0    1    ty  ]
   *		[   0    0    1   ]
   * </pre>.</en>
   *
   * @param tx double <zh>X᷽ľ.</zh>
   * <en>The distance to translate along the x-axis.</en>
   * @param ty double <zh>Y᷽ľ.</zh>
   * <en>The distance to translate along the y-axis.</en>
   */
  public void translate(double tx, double ty) {
    if (!(tx == 0 && ty == 0)) {
      this.drawArea.setRect(drawArea.getX() + tx, drawArea.getY() + ty,
                            drawArea.getWidth(), drawArea.getHeight()) ;

      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>ͼϽǵ.</zh>
   * <en>Returns the top left corner coordinates of this shape.</en>
   *
   * @return Point2D <zh>ֵ.</zh>
   * <en>The coordinate.</en>
   */
  public Point2D getTopLeftPoint() {
    return new Point2D.Double(this.drawArea.getX(), this.drawArea.getY()) ;
  }

  /**
   * <zh>ʱͼεĵ.</zh>
   * <en>Returns the lock coordinates of this shape.</en>
   *
   * @return Point2D <zh>ֵ.</zh>
   * <en>The coordinate.</en>
   */
  public Point2D getLockPoint() {
    Rectangle2D area = getDrawArea() ;
    Rectangle2D newArea = new Rectangle2D.Double(0, 0, area.getWidth(), area.getHeight()) ;

    AffineTransform transform = AffineTransform.getTranslateInstance(area.getX(), area.getY()) ;
    transform.rotate(getRotate()) ;
    Shape rotateShape = transform.createTransformedShape(newArea) ;
    Rectangle2D rotateBound = rotateShape.getBounds2D() ;

    return new Point2D.Double(rotateBound.getCenterX(), rotateBound.getCenterY()) ;
  }

  /**
   * <zh>ַһʾ,Ϊ.</zh>
   * <en>Split a line to multiple line for drawing in the canvas.</en>
   *
   * @param g2 Graphics2D<zh>ͼ.</zh>
   * <en>The specified Graphics context.</en>
   * @param area Rectangle2D <zh>ֵ.</zh>
   * <en>The area of draw text.</en>
   * @return String[] <zh>ָΪеַ.</zh>
   * <en>The splited string array.</en>
   */
  protected String[] splitString(Graphics2D g2, Rectangle2D area) {
    String text = getText() ;
    if (StringUtil.isEmptyWithTrim(text)) {
      return null ;
    }

    if (!this.autoWrap || this.forceInsideTextNoWrap) { //Զ
      return new String[] {text} ;
    }

    if (this.forceShowAll || text.indexOf("\n") >= 0) {
      return text.split("\n") ;
    }

    double drawWidth = area.getWidth() ;

    Graphics2DState state = new Graphics2DState(g2) ;
    state.saveOldState() ;

    g2.setFont(getTextFont()) ;
    double strWidth = TextUtil.getStringBoundWithFm(g2, text).getWidth() ;
    if (strWidth < drawWidth) {
      return new String[] {text} ;
    }

    //ÿһַĿ
    double[] charWidth = new double[text.length()] ;
    for (int i = 0 ; i < text.length() ; i++) {
      charWidth[i] = TextUtil.getStringBound(g2, text, i, i + 1).getWidth() ;
    }

    List words = new ArrayList() ; //ʼ
    List wordsWidth = new ArrayList() ; //ÿʵĿ
    BreakIterator iter = BreakIterator.getWordInstance() ;
    iter.setText(text) ;
    int start = iter.first() ;
    for (int end = iter.next() ; end != BreakIterator.DONE ; start = end, end = iter.next()) {
      words.add(text.substring(start, end)) ;

      int width = 0 ;
      for (int i = start ; i < end ; i++) {
        width += charWidth[i] ;
      }
      wordsWidth.add(new Integer(width)) ;
    }

    //ѭÿһ
    int charIndex = 0 ; //ַλ
    int begin = 0 ; //ʵĿʼ
    double lineCharWidth = 0 ; //ÿַĿ֮
    List lineList = new ArrayList() ;

    for (int i = 0 ; i < words.size() ; i++) {
      String tmpStr = (String) words.get(i) ;
      for (int j = 0 ; j < tmpStr.length() ; j++) {
        if (tmpStr.charAt(j) == '\n') { //з
          lineList.add(text.substring(begin, charIndex)) ;
          lineCharWidth = 0 ;
          charIndex++ ;
          begin = charIndex ;
          continue ;
        }

        lineCharWidth += charWidth[charIndex] ;
        //һַСڻƿһʷǿհַ,жϼһʵĿǷ񳬱
        if (j == tmpStr.length() - 1 && lineCharWidth < drawWidth) {
          if (i < words.size() - 1 && !isWhitespace((String) words.get(i + 1))
              && lineCharWidth + ((Integer) wordsWidth.get(i + 1)).intValue() > drawWidth) {
            lineList.add(text.substring(begin, charIndex + 1)) ;
            lineCharWidth = 0 ;
            begin = charIndex + 1 ;
          }
        } else { // ǵʵһַ
          if (lineCharWidth > drawWidth) { //Խ
            if (begin == charIndex) { //֤һַ
              lineList.add(text.substring(begin, begin + 1)) ;
              lineCharWidth = 0 ;
              begin += 1 ;
            } else {
              lineList.add(text.substring(begin, charIndex)) ;
              lineCharWidth = charWidth[charIndex] ;
              begin = charIndex ;
            }
          }
        }

        charIndex++ ;
      }
    }

    if (begin < text.length()) {
      lineList.add(text.substring(begin)) ;
    }
    state.restoreOldState() ;

//    double len = 0 ;
//    int start = 0, end = 0 ;
//    for (int i = 0 ; i < charWidth.length ; i++) {
//      if (charWidth[i] >= drawWidth) { //СһַĿ
//        stringList.add(text.substring(i, i + 1)) ;
//      } else {
//        len += charWidth[i] ;
//        if (len > drawWidth) { //Χ
//          end = i - 1 ;
//
//          boolean in = false ;
//          if (end < start) {
//            end = start + 1 ;
//            in = true ;
//          }
//
//          if (end > text.length() - 1) {
//            end = text.length() - 1 ;
//          }
//
//          stringList.add(text.substring(start, end + 1)) ;
//          if (in) {
//            start = i + 1 ;
//            len = 0 ;
//          } else {
//            start = i ;
//            len = charWidth[i] ;
//          }
//
//        }
//
//        if (i == charWidth.length - 1) { //һַ
//          stringList.add(text.substring(start)) ;
//        }
//      }
//    }



    return (String[]) lineList.toArray(new String[0]) ;
  }

  /**
   * <zh>жַеÿַǷǿհַ.</zh>
   * <en>Determines if the characters of a string are white space according to Java.</en>
   *
   * @param str String
   * @return boolean
   */
  private static boolean isWhitespace(String str) {
    if (str != null) {
      char[] arr = str.toCharArray() ;
      for (int i = 0 ; i < arr.length ; i++) {
        if (!Character.isWhitespace(arr[i])) {
          return false ;
        }
      }
    }

    return true ;
  }

  /**
   * <zh>ַкַ'\n',򲻻.</zh>
   * <en>Split string if the string contains characters '\n'.</en>
   *
   * @return String[] <zh>ָΪеַ.</zh>
   * <en>The splited string array.</en>
   */
  private String[] splitStringManual() {
    String text = getText() ;
    if (StringUtil.isEmptyWithTrim(text)) {
      return null ;
    }

    if (!autoWrap) { //Զ
      return new String[] {text} ;
    }

    if (text.indexOf("\n") > 0) { //ֶ
      return text.split("\n") ;
    }

    return new String[] {text} ;
  }

  /**
   * <zh>tooltipͼӦƫƵλ.</zh>
   * <en>Returns the translate point of tooltip shape.</en>
   *
   * @return Point2D <zh> (Ϊnull).</zh>
   * <en>The translate point (null permitted).</en>
   */
  protected Point2D getTooltipShapeTranslatePoint() {
    if (this.drawArea != null) {
      return new Point2D.Double(this.drawArea.getX(), this.drawArea.getY()) ;
    }

    return null ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Indicates whether some other object is "equal to" this one.</en>
   *
   * @param obj <zh>ҪȽϵĶ</zh>
   * <en>the reference object with which to compare.</en>
   * @return <zh>ȷtrue,򷵻false.</zh>
   * <en>true if this object is the same as the obj argument;
   * false otherwise.</en>
   */
  public boolean equals(Object obj) {
    if (this == obj) {
      return false ;
    }
    if (!(obj instanceof AbstractRectangleTextChartShape)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    AbstractRectangleTextChartShape that = (AbstractRectangleTextChartShape) obj ;
    if (!EqualsUtil.equals(this.drawArea, that.drawArea)) {
      return false ;
    }
    if (this.autoWrap != that.autoWrap) {
      return false ;
    }
    if (this.forceInsideTextNoWrap != that.forceInsideTextNoWrap) {
      return false ;
    }
    if (this.forceShowAll != that.forceShowAll) {
      return false ;
    }
    if (this.textMaxShowLine != that.textMaxShowLine) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return int <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    int hashCode = super.hashCode() ;
    hashCode = 37 * hashCode + HashcodeUtil.hashCode(this.drawArea) ;
    hashCode = 37 * hashCode + HashcodeUtil.hashCode(this.autoWrap) ;
    hashCode = 37 * hashCode + HashcodeUtil.hashCode(this.forceInsideTextNoWrap) ;
    hashCode = 37 * hashCode + HashcodeUtil.hashCode(this.forceShowAll) ;
    hashCode = 37 * hashCode + this.textMaxShowLine ;

    return hashCode ;
  }

  /**
   * <zh>ؿ¡Ķ.</zh>
   * <en>Creates and returns a copy of this object.</en>
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the Cloneable interface. Subclasses
   *               that override the clone method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object. </en>
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "[drawArea=" + this.drawArea
      + ",autoWrap=" + this.autoWrap
      + ",forceInsideTextNoWrap=" + this.forceInsideTextNoWrap
      + ",forceShowAll=" + this.forceShowAll
      + ",textMaxShowLine=" + this.textMaxShowLine
      + "]" ;
  }

  /**
   * <zh>л</zh>
   * <en>Provides serialization support</en>
   * @param stream <zh></zh>
   * <en>the output stream</en>
   * @throws IOException  if there is an I/O error.
   */
  private void writeObject(ObjectOutputStream stream) throws IOException {
    stream.defaultWriteObject() ;

    SerialUtil.writeShape(stream, this.drawArea) ;
  }

  /**
   * Provides serialization support.
   *
   * @param stream  the input stream.
   *
   * @throws IOException  if there is an I/O error.
   * @throws ClassNotFoundException  if there is a classpath problem.
   */
  private void readObject(ObjectInputStream stream) throws IOException, ClassNotFoundException {
    stream.defaultReadObject() ;

    this.drawArea = (Rectangle2D) SerialUtil.readShape(stream) ;
  }

}
