package com.ijchart.xychart.shape ;

import java.awt.BasicStroke ;
import java.awt.Color ;
import java.awt.Font ;
import java.awt.Graphics2D ;
import java.awt.Paint ;
import java.awt.Shape ;
import java.awt.Stroke ;
import java.awt.geom.Point2D ;
import java.awt.geom.Rectangle2D ;

import com.ijchart.xychart.constant.ChartShapeDrawType ;
import com.ijchart.xychart.constant.GradientPaintTransformType ;
import com.ijchart.xychart.constant.ShapeType ;
import com.ijchart.xychart.data.IChartData ;
import com.ijchart.xychart.listener.IChartShapeChangeListener ;
import com.ijchart.xychart.shape.change.IShapeChange ;
import com.ijchart.xychart.ui.RectangleInsets ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ͼʹõĸͼεĽӿ,ͼеĸͼζһͼζ
 * </zh>
 * <en>Described in the chart for the use of graphical interfaces,
 * for a variety of graphic charts are seen as a separate graphic objects</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public interface IChartShape {

  /**
   * <zh>ȱʡ</zh>
   * <en>Default draw outline is false</en>
   */
  boolean DEFAULT_DRAW_BORDER = false ;

  /**
   * <zh>ȱʡͼ</zh>
   * <en>Default fill shape is true</en>
   */
  boolean DEFAULT_FILL_SHAPE = false ;

  /**
   * <zh>ȱʡ䱳alphaֵ.</zh>
   * <en>Thedefault alpha value for fill background.</en>
   */
  float DEFAULT_FILL_ALPHA = 1.0F ;

  /**
   * <zh>Paint</zh>
   * <en>The default content Paint</en>
   */
  Paint DEFAULT_CONTENT_PAINT = Color.BLACK ;

  /**
   * <zh>ȱʡPaint</zh>
   * <en>The default fill Paint</en>
   */
  Paint DEFAULT_FILL_PAINT = Color.WHITE ;

  /**
   * <zh>ȱʡPaint</zh>
   * <en>The default border Paint</en>
   */
  Paint DEFAULT_BORDER_PAINT = Color.GRAY ;

  /**
   * <zh>ȱʡ.</zh>
   * <en>Default margin.</en>
   */
  RectangleInsets DEFAULT_MARGIN = RectangleInsets.INSETS_ZERO ;

  /**
   * <zh>ȱʡGradientPaint任.</zh>
   * <en>The defeault GradientPaint transform type.</en>
   */
  GradientPaintTransformType DEFAULT_GRADIENT_PAINT_TRANSFORM_TYPE =
    GradientPaintTransformType.HORIZONTAL ;

  /**
   * <zh>ȱʡStroke</zh>
   * <en>The default border Stroke</en>
   */
  Stroke DEFAULT_BORDER_STROKE = new BasicStroke(1.0f) ;

  /**
   * <zh>ѡвֵľεĿ.</zh>
   * <en>The width of selected area.</en>
   */
  double DEFAULT_SELECT_RECT_WIDTH = 4 ;

  /**
   * <zh>ѡвֵľεAlphaֵ.</zh>
   * <en>The alpha value of selected area.</en>
   */
  float DEFAULT_SELECT_RECT_ALPHA = 0.5F ;

  /**
   * <zh>ѡвֵɫ.</zh>
   * <en>The fill paint of selected area.</en>
   */
  Paint DEFAULT_SELECT_FILL_PAINT = Color.BLACK ;

  /**
   * <zh>ȱʡ.</zh>
   * <en>The default font. </en>
   */
  Font DEFAULT_TEXT_FONT = new Font("SansSerif", Font.PLAIN, 12) ;

  /**
   * <zh>tooltipͼεı߳.</zh>
   * <en>The default tooltip shape width.</en>
   */
  double DEFAULT_TOOLTIP_SHAPE_WIDTH = 2 ;

  /**
   * <zh>Ƿѡеȱʡֵ.</zh>
   * <en>The default value of shape selected.</en>
   */
  boolean DEFAULT_SELECTED = false ;

  /**
   * <zh>Բȵͼ.</zh>
   * <en>The circle shape of a hyperlink AREA in an image MAP.</en>
   */
  String HOTSPOT_TYPE_CIRCLE = "circ" ;

  /**
   * <zh>ȵͼ.</zh>
   * <en>The polygon shape of a hyperlink AREA in an image MAP.</en>
   */
  String HOTSPOT_TYPE_POLYGON = "poly" ;

  /**
   * <zh>ȵͼ.</zh>
   * <en>The rectangle shape of a hyperlink AREA in an image MAP.</en>
   */
  String HOTSPOT_TYPE_RECTANGLE = "rect" ;

  /**
   * <zh>ͼεı.</zh>
   * <en>Returns the ID.</en>
   *
   * @return int <zh>ͼεı.</zh>
   * <en>The shape ID.</en>
   */
  int getId() ;

  /**
   * <zh>ͼεı,һchart,ͼεIDΨһ.</zh>
   * <en>Sets an ID for this shape,the shape ID is unique and continuous in a chart.</en>
   * @param shapeId int
   */
  void setId(int shapeId) ;

  /**
   * <zh>ǷƵı־</zh>
   * <en>A flag that controls whether or not the shape border is visible</en>
   *
   * @return boolean <zh>Ƿtrue,򷵻false</zh>
   * <en>A boolean flag</en>
   */
  boolean isDrawBorder() ;

  /**
   * <zh>Ƿͼε</zh>
   * <en>Sets a flag that controls whether or not the shape border is visible</en>
   *
   * @param flag boolean <zh>true,false</zh>
   * <en>true,draw the border of the shape;false otherwise</en>
   */
  void setDrawBorder(boolean flag) ;

  /**
   * <zh>߻</zh>
   * <en>Get the ChartShape border Stroke</en>
   *
   * @return Stroke <zh>߻</zh>
   * <en>The border Stroke</en>
   */
  Stroke getBorderStroke() ;

  /**
   * <zh>ͼߵĻ</zh>
   * <en>Sets the shape border Stroke</en>
   *
   * @param stroke Stroke <zh>ͼ߻(Ϊnull)</zh>
   * <en>The ChartShape border Stroke(null permitted).</en>
   */
  void setBorderStroke(Stroke stroke) ;

  /**
   * <zh>ɫ</zh>
   * <en>Get the ChartShape border Paint</en>
   *
   * @return Paint <zh>ɫ</zh>
   * <en>The border Paint</en>
   */
  Paint getBorderPaint() ;

  /**
   * <zh>ͼߵɫ</zh>
   * <en>Sets the shape border Paint</en>
   *
   * @param paint Paint <zh>ͼɫ(Ϊnull)</zh>
   * <en>The ChartShape border Paint(null permitted).</en>
   */
  void setBorderPaint(Paint paint) ;

  /**
   * <zh>ǷƵı־</zh>
   * <en>A flag that controls whether or not the shape is filled</en>
   *
   * @return boolean <zh>Ƿtrue,򷵻false</zh>
   * <en>A boolean flag</en>
   */
  boolean isFillShape() ;

  /**
   * <zh>Ƿͼڲ</zh>
   * <en>Sets a flag that controls whether or not the shape is filled</en>
   *
   * @param bFillShape boolean <zh>trueͼڲ,falseͼڲ</zh>
   * <en>true,fill the shape;false otherwise</en>
   */
  void setFillShape(boolean bFillShape) ;

  /**
   * <zh>ͼڲɫ</zh>
   * <en>Get the ChartShape fill paint</en>
   *
   * @return Paint <zh>ͼڲɫ</zh>
   * <en>The ChartShape fill paint</en>
   */
  Paint getFillPaint() ;

  /**
   * <zh>ͼڲɫ</zh>
   * <en>Sets the shape fill Paint</en>
   *
   * @param paint Paint <zh>ͼڲõPaint(Ϊnull)</zh>
   * <en>The ChartShape fill Paint(null not permitted).</en>
   */
  void setFillPaint(Paint paint) ;

  /**
   * <zh>䱳alphaֵ,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>The alpha value for fill background,and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param alpha float <zh>alphaֵ,ڱ[0.0,1.0].</zh>
   * <en>The alpha transparency (in the range 0.0f to 1.0f, where 0.0f is fully transparent,
   * and 1.0f is fully opaque).</en>
   */
  void setFillAlpha(float alpha) ;

  /**
   * <zh>䱳alphaֵ.</zh>
   * <en>Returns the alpha value for fill background.</en>
   * @return float <zh>alphaֵ.</zh>
   * <en>The alpha value.</en>
   */
  float getFillAlpha() ;

  /**
   * <zh>ͼڲɫչ</zh>
   * <en>Get the ChartShape GradientPaintTransformType type</en>
   *
   * @return GradientPaintTransformType
   */
  GradientPaintTransformType getGradientPaintTransformType() ;

  /**
   * <zh>ͼڲɫչ,PaintΪGradientPaint</zh>
   * <en>Sets the ChartShape GradientPaintTransformType type,use only when fill
   *
   * Paint is GradientPaint</en>
   * @param transformType GradientPaintTransformType (null permitted)
   */
  void setGradientPaintTransformType(GradientPaintTransformType transformType) ;

  /**
   * <zh>tooltipַ.</zh>
   * <en>Returns the tooltip text.</en>
   *
   * @return String <zh>tooltipַ (Ϊnull).</zh>
   * <en>The tooltip text (maybe null).</en>
   */
  String getTooltipText() ;

  /**
   * <zh>tooltipַ.</zh>
   * <en>Sets the tooltip text.</en>
   *
   * @param tooltip String <zh>tooltipַ (Ϊnull).</zh>
   * <en>The tooltip text (null permitted).</en>
   */
  void setTooltipText(String tooltip) ;

  /**
   * <zh>URLַ.</zh>
   * <en>Returns the URL text.</en>
   *
   * @return String <zh>URLַ (Ϊnull).</zh>
   * <en>The URL text (maybe null).</en>
   */
  String getURL() ;

  /**
   * <zh>URLַ.</zh>
   * <en>Sets the URL text.</en>
   *
   * @param url String <zh>URLַ (Ϊnull).</zh>
   * <en>The URL text (null permitted).</en>
   */
  void setURL(String url) ;

  /**
   * <zh>ͼ豸(Ļӡ)ϻƴͼ</zh>
   * <en>Draw the ChartShape on a Java 2d graphics device(such as the screen or the printer).</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(Ļӡ) (Ϊnull)</zh>
   * <en>The graphics device (such as the screen or the printer) (null not permitted).</en>
   */
  void draw(Graphics2D g2) ;

  /**
   * <zh>ص(x,y)Ƿͼ.</zh>
   * <en>Tests if a specified Point2D is inside the boundary of the ChartShape.</en>
   *
   * @param x double <zh>(xλ).</zh>
   * <en>the x position.</en>
   * @param y double <zh>(yλ).</zh>
   * <en>the y position.</en>
   * @return boolean <zh>򷵻true,򷵻false.</zh>
   * <en>Returns true if this shape contains point(x,y),otherwise return false.</en>
   */
  boolean contains(double x, double y) ;

  /**
   * <zh>صpointǷͼ.</zh>
   * <en>Tests if a specified Point2D is inside the boundary of the ChartShape.</en>
   *
   * @param point Point2D <zh>ֵ,.(Ϊnull).</zh>
   * <en>The position,maybe a mouse point (null not permitted).</en>
   * @return boolean <zh>򷵻true,򷵻false.</zh>
   * <en>Returns true if this shape contains point(x,y),otherwise return false.</en>
   */
  boolean contains(Point2D point) ;

  /**
   * <zh>ͼε.</zh>
   * <en>The type for describe this shape.</en>
   * @return ShapeType <zh>{@link com.ijchart.xychart.constant.ShapeType}ඨĳ
   * (Ϊnull).</zh>
   * <en>The constant value in class {@link com.ijchart.xychart.constant.ShapeType}.</en>
   */
  ShapeType getShapeType() ;

  /**
   * <zh>ͼλתǶ,λΪ.</zh>
   * <en>Return the angle of rotation in radians.</en>
   *
   * @return double <zh>ͼλתǶ,λΪ.</zh>
   * <en>The angle of rotation in radians.</en>
   */
  double getRotate() ;

  /**
   * <zh>ͼλתǶ,λΪ.任ľʾ:
   * <pre>
   *		[   cos(theta)    -sin(theta)    0   ]
   *		[   sin(theta)     cos(theta)    0   ]
   *		[       0              0         1   ]
   * </pre>
   * </zh>
   * <en>Concatenates the current Graphics2D Transform with a rotation
   *  transform.Subsequent rendering is rotated by the specified radians relative to the previous
   *  origin.This is equivalent to calling transform(R), where R is an
   *  AffineTransform represented by the following matrix:
   * <pre>
   *		[   cos(theta)    -sin(theta)    0   ]
   *		[   sin(theta)     cos(theta)    0   ]
   *		[       0              0         1   ]
   * </pre>
   * Rotating with a positive angle theta rotates points on the positive
   * x axis toward the positive y axis.</en>
   *
   * @param theta <zh>תĽǶ,λΪ.</zh>
   * <en>the angle of rotation in radians.</en>
   */
  void setRotate(double theta) ;

  /**
   * <zh>ͼǷѡ.</zh>
   * <en>Returns whether or not this shape is selected.</en>
   *
   * @return boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  boolean isSelected() ;

  /**
   * <zh>ͼǷѡ.</zh>
   * <en>Sets a flag controls whether or not the shape is selected.</en>
   *
   * @param flag boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  void setSelected(boolean flag) ;

  /**
   * <zh>ԭʼtooltipͼ,ͼûпתضͼεӰ.</zh>
   * <en>Returns the original tooltip shape.</en>
   *
   * @return Shape <zh>ͼ (Ϊnull).</zh>
   * <en>A shape (null permitted).</en>
   */
  Shape getOriginTooltipShape() ;

  /**
   * <zh>ԭʼtooltipͼ.</zh>
   * <en>Sets the original tooltip shape.</en>
   *
   * @param shape Shape <zh>ͼ (Ϊnull).</zh>
   * <en>A shape (null permitted).</en>
   */
  void setOriginTooltipShape(Shape shape) ;

  /**
   * <zh>tooltipͼ,ͼҪתضͼεӰ.</zh>
   * <en>Returns the tooltip shape.</en>
   *
   * @return Shape <zh>ͼ (Ϊnull).</zh>
   * <en>A shape (null permitted).</en>
   */
  Shape getTooltipShape() ;

  /**
   * <zh>ƶͼ,X᷽ƫtx,Y᷽ƫty,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Concatenates the current Graphics2D Transform with a translation transform.and sends
   * a {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.
   * Subsequent rendering is translated by the specified
   * distance relative to the previous position.
   * This is equivalent to calling transform(T), where T is an
   * AffineTransform represented by the following matrix:
   * <pre>
   *		[   1    0    tx  ]
   *		[   0    1    ty  ]
   *		[   0    0    1   ]
   * </pre>.</en>
   *
   * @param tx double <zh>X᷽ľ.</zh>
   * <en>The distance to translate along the x-axis.</en>
   * @param ty double <zh>Y᷽ľ.</zh>
   * <en>The distance to translate along the y-axis.</en>
   */
  void translate(double tx, double ty) ;

  /**
   * <zh>ͼϽǵ(ͼεת).</zh>
   * <en>Returns the top left corner coordinates of this shape (not consider rotation).</en>
   *
   * @return Point2D <zh>ֵ.</zh>
   * <en>The coordinate.</en>
   */
  Point2D getTopLeftPoint() ;

  /**
   * <zh>ʱͼεĵ.</zh>
   * <en>Returns the lock coordinates of this shape.</en>
   *
   * @return Point2D <zh>ֵ.</zh>
   * <en>The coordinate.</en>
   */
  Point2D getLockPoint() ;

  /**
   * <zh>IChartData.</zh>
   * <en>Returns the IChartData object.</en>
   *
   * @return IChartData <zh>(Ϊnull).</zh>
   * <en>(maybe null).</en>
   */
  IChartData getData() ;

  /**
   * <zh>IChartData.</zh>
   * <en>Sets the IChartData object.</en>
   *
   * @param data IChartData <zh>(Ϊnull).</zh>
   * <en>(null permitted).</en>
   */
  void setData(IChartData data) ;

  /**
   * <zh>ͼε߿.</zh>
   * <en>Returns a Rectangle2D that completely encloses the Shape.</en>
   *
   * @return Rectangle2D <zh>߿.</zh>
   * <en>A Rectangle2D that completely encloses the Shape.</en>
   */
  Rectangle2D getBound() ;

  /**
   * <zh>עͼ{@link IChartShape}仯ļ߶.</zh>
   * <en>Registers an object for notification of changes to the {@link IChartShape}</en>
   *
   * @param listener <zh>עļ.</zh>
   * <en>the object to register.</en>
   */
  void addChangeListener(IChartShapeChangeListener listener) ;

  /**
   * <zh>ɾעͼ{@link IChartShape}仯߶.</zh>
   * <en>Deregisters an object for notification of changes to the {@link IChartShape}.</en>
   *
   * @param listener <zh>Ҫɾļ߶</zh>
   * <en>the object to deregister.</en>
   */
  void removeChangeListener(IChartShapeChangeListener listener) ;

  /**
   * <zh>еͼ{@link IChartShape}仯߶.</zh>
   * <en>Deregisters all object for notification of changes to the {@link IChartShape}.</en>
   */
  void removeAllChangeListener() ;

  /**
   * <zh>ͼζڲı,ֲܴ¼,ͨ˷֪ͨͼζı.</zh>
   * <en>If the data object of {@link IChartShape} changed and no event
   * triggered,set {@link IChartShape} changed by this method.</en>
   */
  void forceChartShapeChanged() ;

  /**
   * <zh>ͼchartеĻ.</zh>
   * <en>Returns the chart shape draw type.</en>
   *
   * @return ChartShapeDrawType <zh>(Ϊnull).</zh>
   * <en>(never null).</en>
   */
  ChartShapeDrawType getShapeDrawType() ;

  /**
   * <zh>ͼchartеĻ.</zh>
   * <en>Returns the chart shape draw type.</en>
   *
   * @param drawType ChartShapeDrawType <zh>(null not permitted).</zh>
   * <en>(null not permitted).</en>
   */
  void setShapeDrawType(ChartShapeDrawType drawType) ;

  /**
   * <zh>ͼǷɼ.</zh>
   * <en>Returns whether or not the shape is visible.</en>
   *
   * @return boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  boolean isVisible() ;

  /**
   * <zh>ͼǷɼ.</zh>
   * <en>Sets whether or not the shape is visible.</en>
   *
   * @param flag boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  void setVisible(boolean flag) ;

  /**
   * <zh>{@link IShapeChange}.</zh>
   * <en>Sets the {@link IShapeChange} object.</en>
   *
   * @param change IShapeChange <zh>{@link IShapeChange} (Ϊnull).</zh>
   * <en>{@link IShapeChange} object (null permitted).</en>
   */
  void setShapeChange(IShapeChange change) ;

  /**
   * <zh>{@link IShapeChange}.</zh>
   * <en>Returns the {@link IShapeChange} object.</en>
   *
   * @return IShapeChange <zh>{@link IShapeChange} (Ϊnull).</zh>
   * <en>{@link IShapeChange} object (maybe null).</en>
   */
  IShapeChange getShapeChange() ;

  /**
   * <zh>γȵͼ.</zh>
   * <en>Returns the shape type of hotspot.</en>
   *
   * @return String <zh>ͼ,ֵpolyrect.</zh>
   * <en>The shape type of hotspot,the value can be poly or rect.</en>
   */
  String getHotspotType() ;

  /**
   * <zh>γȵ.</zh>
   * <en>Returns a string containing the coordinates for a given shape.This
   * string is intended for use in an image map.</en>
   *
   * @param clipArea Shape <zh>ͼμ (Ϊnull).</zh>
   * <en>The clip area.</en>
   *
   * @return String <zh>ȵ (Ϊnull).</zh>
   * <en>The coordinates of hotspot (maybe null).</en>
   */
  String getHotspotPolyCoords(Shape clipArea) ;
}
