package com.ijchart.xychart.shape ;

import java.awt.AlphaComposite ;
import java.awt.BasicStroke ;
import java.awt.Color ;
import java.awt.Font ;
import java.awt.Graphics2D ;
import java.awt.Paint ;
import java.awt.Polygon ;
import java.awt.Shape ;
import java.awt.Stroke ;
import java.awt.geom.AffineTransform ;
import java.awt.geom.Rectangle2D ;
import java.io.IOException ;
import java.io.ObjectInputStream ;
import java.io.ObjectOutputStream ;
import java.io.Serializable ;

import com.ijchart.xychart.constant.HorizontalAlignment ;
import com.ijchart.xychart.constant.ShapeType ;
import com.ijchart.xychart.constant.VerticalAlignment ;
import com.ijchart.xychart.ui.Graphics2DState ;
import com.ijchart.xychart.util.AnchorUtil ;
import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.HashcodeUtil ;
import com.ijchart.xychart.util.SerialUtil ;
import com.ijchart.xychart.util.StrokeUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ͼ֮ıǩͼ,ɱǩһ.</zh>
 * <en>The outside label shape of pie,composite by a label and a polygon.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class PieOutsideLabelChartShape extends AbstractRectangleTextChartShape implements
  Cloneable, Serializable {

  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = -1970743519999857014L ;

  /**
   * <zh>ȱʡĻƱǩӰpaint.</zh>
   * <en>The default section label shadow paint.</en>
   */
  public static final Paint DEFAULT_LABEL_SHADOW_PAINT = new Color(151, 151, 151, 128) ;

  /**
   * <zh>ȱʡıǩxӰ.</zh>
   * <en>The default x-offset for the label shadow effect.</en>
   */
  public static final double DEFAULT_LABEL_SHADOW_X_OFFSET = 3F ;

  /**
   * <zh>ȱʡıǩyӰ.</zh>
   * <en>The default y-offset for the label shadow effect.</en>
   */
  public static final double DEFAULT_LABEL_SHADOW_Y_OFFSET = 3F ;

  /**
   * <zh>ȱʡĻͼεpaint.</zh>
   * <en>The default paint used to draw sector shadow.</en>
   */
  public static final Paint DEFAULT_SECTOR_SHADOW_PAINT = Color.gray ;

  /**
   * <zh>ȱʡĻߵpaint.</zh>
   * <en>The default link paint.</en>
   */
  public static final Paint DEFAULT_LINK_PAINT = Color.black ;

  /**
   * <zh>ȱʡĻߵstroke.</zh>
   * <en>The default link stroke.</en>
   */
  public static final Stroke DEFAULT_LINK_STROKE = new BasicStroke(0.5F) ;

  /**
   * <zh>ƱǩӰpaint.</zh>
   * <en> The paint used to draw the shadow for the section labels.</en>
   */
  private transient Paint labelShadowPaint ;

  /**
   * <zh>ǩxӰ.</zh>
   * <en>The x-offset for the label shadow effect.</en>
   */
  private double labelShadowXOffset ;

  /**
   * <zh>ǩyӰ.</zh>
   * <en>The y-offset for the label shadow effect.</en>
   */
  private double labelShadowYOffset ;

  /**
   * <zh>ǩ.</zh>
   * <en>The polygon.</en>
   */
  private Polygon link ;

  /**
   * <zh>ߵpaint.</zh>
   * <en>The link paint.</en>
   */
  private transient Paint linkPaint ;

  /**
   * <zh>ߵstroke.</zh>
   * <en>The link stroke.</en>
   */
  private transient Stroke linkStroke ;

  /**
   * <zh>drawAreaʾtext.</zh>
   * <en>Draw text in area drawArea.</en>
   *
   * @param labelArea Rectangle2D <zh>ǩ (Ϊnull).</zh>
   * <en>The label area (null not permitted).</en>
   * @param text String <zh>Ҫʾ (Ϊnull).</zh>
   * <en>The text (null permitted).</en>
   * @param link Polygon <zh>ǩ (Ϊnull).</zh>
   * <en>the polygon (null permitted).</en>
   */
  public PieOutsideLabelChartShape(Rectangle2D labelArea, String text, Polygon link) {
    this(labelArea, text, null, link) ;
  }

  /**
   * <zh>drawAreaʾtext.</zh>
   * <en>Draw text in area drawArea.</en>
   *
   * @param labelArea Rectangle2D <zh>ǩ (Ϊnull).</zh>
   * <en>The label area (null not permitted).</en>
   * @param text String <zh>Ҫʾ (Ϊnull).</zh>
   * <en>The text (null permitted).</en>
   * @param font <zh> (Ϊnull),fontΪnull,
   * Ϊ{@link com.ijchart.xychart.shape.IChartShape#DEFAULT_TEXT_FONT}.</zh>
   * <en>The specified font (null permitted).If font is null,
   * {@link com.ijchart.xychart.shape.IChartShape#DEFAULT_TEXT_FONT} is used.</en>
   * @param link Polygon <zh>ǩ (Ϊnull).</zh>
   * <en>the polygon (null permitted).</en>
   */
  public PieOutsideLabelChartShape(Rectangle2D labelArea, String text, Font font, Polygon link) {
    super(labelArea, text, font) ;

    this.labelShadowPaint = DEFAULT_LABEL_SHADOW_PAINT ;
    this.labelShadowXOffset = DEFAULT_LABEL_SHADOW_X_OFFSET ;
    this.labelShadowYOffset = DEFAULT_LABEL_SHADOW_Y_OFFSET ;

    this.link = link ;
    this.linkPaint = DEFAULT_LINK_PAINT ;
    this.linkStroke = DEFAULT_LINK_STROKE ;

    setDrawBorder(true) ;
    setFillShape(true) ;
    setTextHorizontalAlign(HorizontalAlignment.CENTER) ;
    setTextVerticalAlign(VerticalAlignment.CENTER) ;
  }

  /**
   * <zh>ػͼεpaint.</zh>
   * <en>Returns the paint used to draw sector shadow.</en>
   *
   * @return Paint <zh>paint (Ϊnull).</zh>
   * <en>The paint (maybe null).</en>
   */
  public Paint getLabelShadowPaint() {
    return this.labelShadowPaint ;
  }

  /**
   * <zh>ûͼεpaint,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the paint used to draw sector shadow and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param paint Paint <zh>paint (Ϊnull).</zh>
   * <en>The paint (null permitted).</en>
   */
  public void setLabelShadowPaint(Paint paint) {
    if (!(this.labelShadowPaint == null && paint == null)) {
      this.labelShadowPaint = paint ;
      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>رǩxӰ.</zh>
   * <en>Returns the x-offset for the label shadow effect.</en>
   *
   * @return double <zh>ƫֵ.</zh>
   * <en>The x-offset.</en>
   */
  public double getLabelShadowXOffset() {
    return this.labelShadowXOffset ;
  }

  /**
   * <zh>ñǩxӰ,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the x-offset for the label shadow effect and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param offset double <zh>ƫֵ.</zh>
   * <en>The x-offset.</en>
   */
  public void setLabelShadowXOffset(double offset) {
    if (offset < 0) {
      throw new IllegalArgumentException("Argument 'offset' must be >=0.") ;
    }

    if (this.labelShadowXOffset != offset) {
      this.labelShadowXOffset = offset ;
      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>رǩxӰ.</zh>
   * <en>Returns the y-offset for the label shadow effect.</en>
   *
   * @return double <zh>ƫֵ.</zh>
   * <en>The y-offset.</en>
   */
  public double getLabelShadowYOffset() {
    return this.labelShadowYOffset ;
  }

  /**
   * <zh>ñǩyӰ,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the y-offset for the label shadow effect and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param offset double <zh>ƫֵ.</zh>
   * <en>The y-offset.</en>
   */
  public void setLabelShadowYOffset(double offset) {
    if (this.labelShadowYOffset != offset) {
      this.labelShadowYOffset = offset ;
      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>رǩ.</zh>
   * <en>Returns the polygon.</en>
   *
   * @return Polygon <zh>ǩ.</zh>
   * <en>The polygon.</en>
   */
  public Polygon getLink() {
    return this.link ;
  }

  /**
   * <zh>ñǩ,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the polygon and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param link Polygon <zh>ǩ (Ϊnull).</zh>
   * <en>The polygon (null permitted).</en>
   */
  public void setLink(Polygon link) {
    if (!(this.link == null && link == null)) {
      this.link = link ;
      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>ػߵpaint.</zh>
   * <en>Returns the link paint.</en>
   *
   * @return Paint <zh>paint.</zh>
   * <en>The paint.</en>
   */
  public Paint getLinkPaint() {
    return this.linkPaint ;
  }

  /**
   * <zh>ûߵpaint,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the link paint and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param paint Paint <zh>paint (Ϊnull).</zh>
   * <en>The paint (null not permitted).</en>
   */
  public void setLinkPaint(Paint paint) {
    if (paint == null) {
      throw new IllegalArgumentException("Null 'paint' argument") ;
    }

    this.linkPaint = paint ;
    fireChartShapeChanged() ;
  }

  /**
   * <zh>ػߵstroke.</zh>
   * <en>Returns the link stroke.</en>
   *
   * @return Stroke <zh>stroke.</zh>
   * <en>The stroke.</en>
   */
  public Stroke getLinkStroke() {
    return this.linkStroke ;
  }

  /**
   * <zh>ûߵstroke,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the link stroke and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param stroke Stroke <zh>stroke (Ϊnull).</zh>
   * <en>The stroke (null not permitted).</en>
   */
  public void setLinkStroke(Stroke stroke) {
    if (stroke == null) {
      throw new IllegalArgumentException("Null 'stroke' argument") ;
    }

    this.linkStroke = stroke ;
    fireChartShapeChanged() ;
  }

  /**
   * <zh>ͼ豸(Ļӡ)ϻƴͼ</zh>
   * <en>Draw the ChartShape on a Java 2d graphics device(such as the screen or the printer).</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(Ļӡ) (Ϊnull)</zh>
   * <en>The graphics device (such as the screen or the printer) (null not permitted).</en>
   */
  public void draw(Graphics2D g2) {
    if (g2 == null) {
      throw new IllegalArgumentException("Null 'g2' argument.") ;
    }
    if (!isVisible()) {
      return ;
    }

    Graphics2DState state = new Graphics2DState(g2) ;
    state.saveOldState() ;

    Rectangle2D drawArea = getDrawArea() ;
    Rectangle2D cloneArea = new Rectangle2D.Double(0, 0, drawArea.getWidth(), drawArea.getHeight()) ;
    double areaX = drawArea.getX() ;
    double areaY = drawArea.getY() ;

    g2.translate(areaX, areaY) ;
    g2.rotate(getRotate()) ;
    beforeDraw(g2) ;

    //Ӱ
    if (this.labelShadowPaint != null) {
      g2.translate(this.labelShadowXOffset, this.labelShadowYOffset) ;
      Paint oldPaint = g2.getPaint() ;
      g2.setPaint(this.labelShadowPaint) ;
      g2.fill(cloneArea) ;

      g2.setPaint(oldPaint) ;
      g2.translate( -this.labelShadowXOffset, -this.labelShadowYOffset) ;
    }

    drawBackground(g2, cloneArea) ;
    float strokeWidth = 0 ;
    if (isDrawBorder()) {
      strokeWidth = StrokeUtil.getStrokeWidth(getBorderStroke()) ;
    }
    drawBorder(g2, new Rectangle2D.Double(strokeWidth / 2, strokeWidth / 2,
                                          drawArea.getWidth() - strokeWidth,
                                          drawArea.getHeight() - strokeWidth)) ;

    drawTextWithTrimMarginRect(g2, cloneArea) ;
    if (this.link != null) {
      int[] xPoints = this.link.xpoints ;
      int[] yPoints = this.link.ypoints ;

      int[] cloneXPoints = new int[xPoints.length] ;
      int[] cloneYPoints = new int[yPoints.length] ;
      System.arraycopy(xPoints, 0, cloneXPoints, 0, xPoints.length) ;
      System.arraycopy(yPoints, 0, cloneYPoints, 0, yPoints.length) ;
      int minLen = Math.min(xPoints.length, yPoints.length) ;

      g2.setPaint(this.linkPaint) ;
      g2.setStroke(this.linkStroke) ;

      if (xPoints != null && yPoints != null) {
        for (int i = 0 ; i < minLen ; i++) {
          cloneXPoints[i] -= areaX ;
          cloneYPoints[i] -= areaY ;
        }

        g2.drawPolyline(cloneXPoints, cloneYPoints, minLen) ;
      }
    }

    afterDraw(g2) ;
    g2.rotate( -getRotate()) ;
    state.restoreOldState() ;
    drawSelect(g2) ;
  }

  /**
   * <zh>ͼ豸(Ļӡ)ϻͼεѡв,Ϊͼοܻص,ԻͼʱҪȻеͼκ,
   * ٵô˷ѡв.</zh>
   * <en>Draw the selected ChartShape on a Java 2d graphics device(such as the screen or the
   * printer).</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(Ļӡ) (Ϊnull)</zh>
   * <en>the graphics device (such as the screen or the printer) (null not permitted).</en>
   */
  protected void drawSelect(Graphics2D g2) {
    if (g2 == null) {
      throw new IllegalArgumentException("Null 'g2' argument.") ;
    }

    if (isSelected()) {
      Graphics2DState state = new Graphics2DState(g2) ;
      state.saveOldState() ;
      g2.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, getSelectRectAlpha())) ;

      Rectangle2D drawArea = getDrawArea() ;
      double rectWidth = getSelectRectWidth() ;
      g2.translate(drawArea.getX(), drawArea.getY()) ;
      g2.rotate(getRotate()) ;

      g2.setPaint(getSelectAnchorPaint()) ;
      //ê
      Rectangle2D newArea = new Rectangle2D.Double(0, 0, drawArea.getWidth(),
                                                   drawArea.getHeight()) ;
      AnchorUtil.drawAnchor(g2, newArea, rectWidth) ;

      g2.rotate( -getRotate()) ;
      state.restoreOldState() ;
    }

  }

  /**
   * <zh>任ͼ.</zh>
   * <en>Calculate the transformed shape.</en>
   */
  /**
   * <zh>任ͼ.</zh>
   * <en>Calculate the transformed shape.</en>
   */
  protected void calculateTransformedShape() {
    if (transformedShape == null) {
      AffineTransform transform = new AffineTransform() ;
      transform.translate(getDrawArea().getX(), getDrawArea().getY()) ;
      transform.rotate(getRotate()) ;
      Rectangle2D newArea = new Rectangle2D.Double(0, 0, getDrawArea().getWidth(),
                                                   getDrawArea().getHeight()) ;

      transformedShape = transform.createTransformedShape(newArea) ;
    }
  }

  /**
   * <zh>ȵͼ.</zh>
   * <en>Returns the hotspot shape.</en>
   *
   * @return Shape <zh>ȵͼ (Ϊnull).</zh>
   * <en>The hotspot shape (maybe null).</en>
   */
  protected Shape getHotspotShape() {
    calculateTransformedShape() ;
    return transformedShape ;
  }

  /**
   * <zh>ͼε.</zh>
   * <en>The type for describe this shape.</en>
   * @return ShapeType <zh>{@link com.ijchart.xychart.constant.ShapeType}ඨĳ
   * (Ϊnull).</zh>
   * <en>The constant value in class {@link com.ijchart.xychart.constant.ShapeType}.</en>
   */
  public ShapeType getShapeType() {
    return ShapeType.SHAPE_PIE_OUTSIDE_LABEL ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Indicates whether some other object is "equal to" this one.</en>
   *
   * @param obj <zh>ҪȽϵĶ</zh>
   * <en>the reference object with which to compare.</en>
   * @return <zh>ȷtrue,򷵻false.</zh>
   * <en>true if this object is the same as the obj argument;
   * false otherwise.</en>
   */
  public boolean equals(Object obj) {
    if (this == obj) {
      return false ;
    }
    if (!(obj instanceof PieOutsideLabelChartShape)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    PieOutsideLabelChartShape that = (PieOutsideLabelChartShape) obj ;
    if (!EqualsUtil.equals(this.labelShadowPaint, that.labelShadowPaint)) {
      return false ;
    }
    if (this.labelShadowXOffset != that.labelShadowXOffset) {
      return false ;
    }
    if (this.labelShadowYOffset != that.labelShadowYOffset) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return int <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    double hashCode = super.hashCode() ;

    hashCode = 37 * hashCode + HashcodeUtil.hashCode(this.labelShadowPaint) ;
    hashCode = 37 * hashCode + this.labelShadowXOffset ;
    hashCode = 37 * hashCode + this.labelShadowYOffset ;

    return (int) hashCode ;
  }

  /**
   * <zh>ؿ¡Ķ.</zh>
   * <en>Creates and returns a copy of this object.</en>
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the Cloneable interface. Subclasses
   *               that override the clone method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object. </en>
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "PieOutsideLabelChartShape" ;
  }

  /**
   * Provides serialization support.
   *
   * @param stream  the output stream.
   *
   * @throws IOException  if there is an I/O error.
   */
  private void writeObject(ObjectOutputStream stream) throws IOException {
    stream.defaultWriteObject() ;

    SerialUtil.writePaint(stream, this.labelShadowPaint) ;
    SerialUtil.writePaint(stream, this.linkPaint) ;
    SerialUtil.writeStroke(stream, this.linkStroke) ;
  }

  /**
   * Provides serialization support.
   *
   * @param stream  the input stream.
   *
   * @throws IOException  if there is an I/O error.
   * @throws ClassNotFoundException  if there is a classpath problem.
   */
  private void readObject(ObjectInputStream stream) throws IOException, ClassNotFoundException {
    stream.defaultReadObject() ;

    this.labelShadowPaint = SerialUtil.readPaint(stream) ;
    this.linkPaint = SerialUtil.readPaint(stream) ;
    this.linkStroke = SerialUtil.readStroke(stream) ;
  }
}
