package com.ijchart.xychart.shape ;

import java.awt.AlphaComposite ;
import java.awt.BasicStroke ;
import java.awt.Color ;
import java.awt.Graphics2D ;
import java.awt.Paint ;
import java.awt.Shape ;
import java.awt.Stroke ;
import java.awt.geom.AffineTransform ;
import java.awt.geom.GeneralPath ;
import java.awt.geom.Point2D ;
import java.awt.geom.Rectangle2D ;
import java.io.IOException ;
import java.io.ObjectInputStream ;
import java.io.ObjectOutputStream ;
import java.io.Serializable ;

import com.ijchart.xychart.constant.ShapeType ;
import com.ijchart.xychart.ui.Graphics2DState ;
import com.ijchart.xychart.util.AnchorUtil ;
import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.GraphicsUtil ;
import com.ijchart.xychart.util.HashcodeUtil ;
import com.ijchart.xychart.util.SerialUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>һڲʾ,һһͷ.</zh>
 * <en>Draw a text inside a Rectangle and an arrow.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class TextAndPointerChartShape extends AbstractRectangleTextChartShape implements Cloneable,
  Serializable {

  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = 1569931309907294416L ;

  /**
   * <zh>ȱʡĻƼͷpaint.</zh>
   * <en>The default arrow paint.</en>
   */
  public static final Paint DEFAULT_ARROW_PAINT = Color.black ;

  /**
   * <zh>ȱʡĻƼͷstroke.</zh>
   * <en>The default arrow stroke.</en>
   */
  public static final Stroke DEFAULT_ARROW_STROKE = new BasicStroke(1) ;

  /**
   * <zh>ͷͼ.</zh>
   * <en>The arrow shape.</en>
   */
  private transient Shape arrowShape ;

  /**
   * <zh>Ƽͷpaint.</zh>
   * <en>The arrow paint.</en>
   */
  private transient Paint arrowPaint ;

  /**
   * <zh>Ƽͷstroke.</zh>
   * <en>The arrow stroke.</en>
   */
  private transient Stroke arrowStroke ;

  /**
   * <zh>drawAreaʾtext.</zh>
   * <en>Draw text in area drawArea.</en>
   *
   * @param drawArea Rectangle2D <zh> (Ϊnull).</zh>
   * <en>The draw area (null not permitted).</en>
   * @param text String <zh>Ҫʾ (Ϊnull).</zh>
   * <en>The text (null permitted).</en>
   * @param arrowShape Shape <zh>ͷͼ (Ϊnull).</zh>
   * <en>The arrow shape (null not permitted).</en>
   */
  public TextAndPointerChartShape(Rectangle2D drawArea, String text, Shape arrowShape) {
    this(drawArea, text, arrowShape, DEFAULT_ARROW_PAINT, DEFAULT_ARROW_STROKE) ;
  }

  /**
   * <zh>drawAreaʾtext.</zh>
   * <en>Draw text in area drawArea.</en>
   *
   * @param drawArea Rectangle2D <zh> (Ϊnull).</zh>
   * <en>The draw area (null not permitted).</en>
   * @param text String <zh>Ҫʾ (Ϊnull).</zh>
   * <en>The text (null permitted).</en>
   * @param arrowShape Shape <zh>ͷͼ (Ϊnull).</zh>
   * <en>The arrow shape (null not permitted).</en>
   * @param arrowPaint Paint <zh>Ƽͷpaint (Ϊnull).</zh>
   * <en>The arrow paint (null not permitted).</en>
   * @param arrowStroke Stroke <zh>Ƽͷstroke (Ϊnull).</zh>
   * <en>The arrow stroke (null not permitted).</en>
   */
  public TextAndPointerChartShape(Rectangle2D drawArea, String text, Shape arrowShape,
                                  Paint arrowPaint, Stroke arrowStroke) {
    super(drawArea, text) ;
    if (arrowShape == null) {
      throw new IllegalArgumentException("Null 'arrowShape' argument.") ;
    }
    if (arrowPaint == null) {
      throw new IllegalArgumentException("Null 'arrowPaint' argument.") ;
    }
    if (arrowStroke == null) {
      throw new IllegalArgumentException("Null 'arrowStroke' argument.") ;
    }

    this.arrowShape = arrowShape ;
    this.arrowPaint = arrowPaint ;
    this.arrowStroke = arrowStroke ;
  }

  /**
   * <zh>ؼͷͼ.</zh>
   * <en>Returns the arrow shape.</en>
   *
   * @return Shape <zh>ͼ (Ϊnull).</zh>
   * <en>The arrow shape (never null).</en>
   */
  public Shape getArrowShape() {
    return this.arrowShape ;
  }

  /**
   * <zh>üͷͼ,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the arrow shape and sends
   * a {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param shape Shape <zh>ͷͼ (Ϊnull).</zh>
   * <en>The arrow shape (null not permitted).</en>
   */
  public void setArrowShape(Shape shape) {
    if (shape == null) {
      throw new IllegalArgumentException("Null 'shape' argument.") ;
    }

    this.arrowShape = shape ;
    fireChartShapeChanged() ;
  }

  /**
   * <zh>ػƼͷpaint.</zh>
   * <en>Returns the arrow paint.</en>
   *
   * @return Paint <zh>paint (Ϊnull).</zh>
   * <en>The arrow paint (never null).</en>
   */
  public Paint getArrowPaint() {
    return this.arrowPaint ;
  }

  /**
   * <zh>ûƼͷpaint,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the arrow paint and sends
   * a {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param paint Paint <zh>paint (Ϊnull).</zh>
   * <en>The arrow paint (null not permitted).</en>
   */
  public void setArrowPaint(Paint paint) {
    if (paint == null) {
      throw new IllegalArgumentException("Null 'paint' argument.") ;
    }

    this.arrowPaint = paint ;
    fireChartShapeChanged() ;
  }

  /**
   * <zh>ػƼͷstroke.</zh>
   * <en>Returns the arrow stroke.</en>
   *
   * @return Stroke <zh>stroke (Ϊnull).</zh>
   * <en>The arrow stroke (never null).</en>
   */
  public Stroke getArrowStroke() {
    return this.arrowStroke ;
  }

  /**
   * <zh>ûƼͷstroke,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the arrow stroke and sends
   * a {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param stroke Stroke <zh>stroke (Ϊnull).</zh>
   * <en>The arrow stroke (null not permitted).</en>
   */
  public void setArrowStroke(Stroke stroke) {
    if (stroke == null) {
      throw new IllegalArgumentException("Null 'stroke' argument.") ;
    }

    this.arrowStroke = stroke ;
    fireChartShapeChanged() ;
  }

  /**
   * <zh>ͼ豸(Ļӡ)ϻƴͼ</zh>
   * <en>Draw the ChartShape on a Java 2d graphics device(such as the screen or the printer).</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(Ļӡ) (Ϊnull)</zh>
   * <en>The graphics device (such as the screen or the printer) (null not permitted).</en>
   */
  public void draw(Graphics2D g2) {
    if (g2 == null) {
      throw new IllegalArgumentException("Null 'g2' argument.") ;
    }
    if (!isVisible()) {
      return ;
    }

    Graphics2DState state = new Graphics2DState(g2) ;
    state.saveOldState() ;

    Rectangle2D cloneArea = new Rectangle2D.Double(0, 0, getDrawArea().getWidth(),
                                                   getDrawArea().getHeight()) ;

    double translateX = getDrawArea().getX() ;
    double translateY = getDrawArea().getY() ;
    g2.translate(translateX, translateY) ;
    g2.rotate(getRotate()) ;
    beforeDraw(g2) ;
//    g2.setClip(cloneArea) ;

    drawTextWithTrimMarginRect(g2, cloneArea) ;
    Shape newArrowShape = GraphicsUtil.translateShapeByPath(this.arrowShape, (float) translateX,
                                                            (float) translateY) ;
    g2.setPaint(this.arrowPaint) ;
    g2.setStroke(this.arrowStroke) ;
    g2.draw(newArrowShape) ;
    g2.fill(newArrowShape) ;

    afterDraw(g2) ;
    g2.rotate( -getRotate()) ;
    state.restoreOldState() ;
    drawSelect(g2) ;
  }

  /**
   * <zh>ͼ豸(Ļӡ)ϻͼεѡв,Ϊͼοܻص,ԻͼʱҪȻеͼκ,
   * ٵô˷ѡв.</zh>
   * <en>Draw the selected ChartShape on a Java 2d graphics device(such as the screen or the
   * printer).</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(Ļӡ) (Ϊnull)</zh>
   * <en>the graphics device (such as the screen or the printer) (null not permitted).</en>
   */
  protected void drawSelect(Graphics2D g2) {
    if (g2 == null) {
      throw new IllegalArgumentException("Null 'g2' argument.") ;
    }

    if (isSelected()) {
      Graphics2DState state = new Graphics2DState(g2) ;
      state.saveOldState() ;

      Rectangle2D rect = getDrawArea() ;
      double translateX = rect.getX() ;
      double translateY = rect.getY() ;
      g2.translate(translateX, translateY) ;
      g2.rotate(getRotate()) ;
//    g2.setClip(cloneArea) ;
      g2.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, getSelectRectAlpha())) ;

      double rectWidth = getSelectRectWidth() ;
      g2.setPaint(getSelectAnchorPaint()) ;
      //ê
      AnchorUtil.drawAnchor(g2, rect, rectWidth) ;

      g2.rotate( -getRotate()) ;
      state.restoreOldState() ;
    }
  }

  /**
   * <zh>ʱͼεĵ.</zh>
   * <en>Returns the lock coordinates of this shape.</en>
   *
   * @return Point2D <zh>ֵ.</zh>
   * <en>The coordinate.</en>
   */
  public Point2D getLockPoint() {
    if (getDrawArea() == null) {
      return null ;
    }

    double translateX = getDrawArea().getX() ;
    double translateY = getDrawArea().getY() ;

    Shape newArrayShape = GraphicsUtil.translateShapeByPath(this.arrowShape, (float) translateX,
                                                            (float) translateY) ;

    AffineTransform transform = AffineTransform.getTranslateInstance(translateX, translateY) ;
    transform.rotate(getRotate()) ;
    Shape rotateShape = transform.createTransformedShape(newArrayShape) ;
    Rectangle2D rotateBound = rotateShape.getBounds2D() ;

    return new Point2D.Double(rotateBound.getCenterX(), rotateBound.getCenterY()) ;
  }

  /**
   * <zh>任ͼ.</zh>
   * <en>Calculate the transformed shape.</en>
   */
  protected void calculateTransformedShape() {
    if (transformedShape == null) {
      double translateX = getDrawArea().getX() ;
      double translateY = getDrawArea().getY() ;

      AffineTransform transform = new AffineTransform() ;
      transform.translate(translateX, translateY) ;
      transform.rotate(getRotate()) ;
      Rectangle2D newArea = new Rectangle2D.Double(0, 0, getDrawArea().getWidth(),
                                                   getDrawArea().getHeight()) ;

      GeneralPath path = new GeneralPath(newArea) ;
      Shape newArrayShape = GraphicsUtil.translateShapeByPath(this.arrowShape, (float) translateX,
                                                              (float) translateY) ;
      path.append(newArrayShape, false) ;

      transformedShape = transform.createTransformedShape(path) ;
    }
  }

  /**
   * <zh>ȵͼ.</zh>
   * <en>Returns the hotspot shape.</en>
   *
   * @return Shape <zh>ȵͼ (Ϊnull).</zh>
   * <en>The hotspot shape (maybe null).</en>
   */
  protected Shape getHotspotShape() {
    double translateX = getDrawArea().getX() ;
    double translateY = getDrawArea().getY() ;

    AffineTransform transform = new AffineTransform() ;
    transform.translate(translateX, translateY) ;
    transform.rotate(getRotate()) ;

    Shape shape = GraphicsUtil.translateShapeByPath(this.arrowShape, (float) translateX,
                                                    (float) translateY) ;
    return transform.createTransformedShape(shape) ;
  }

  /**
   * <zh>ͼε.</zh>
   * <en>The type for describe this shape.</en>
   * @return ShapeType <zh>{@link com.ijchart.xychart.constant.ShapeType}ඨĳ
   * (Ϊnull).</zh>
   * <en>The constant value in class {@link com.ijchart.xychart.constant.ShapeType}.</en>
   */
  public ShapeType getShapeType() {
    return ShapeType.SHAPE_TEXT_ARROW ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Indicates whether some other object is "equal to" this one.</en>
   *
   * @param obj <zh>ҪȽϵĶ</zh>
   * <en>the reference object with which to compare.</en>
   * @return <zh>ȷtrue,򷵻false.</zh>
   * <en>true if this object is the same as the obj argument;
   * false otherwise.</en>
   */
  public boolean equals(Object obj) {
    if (this == obj) {
      return false ;
    }
    if (!(obj instanceof TextAndPointerChartShape)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    TextAndPointerChartShape that = (TextAndPointerChartShape) obj ;
    if (!EqualsUtil.equals(this.arrowShape, that.arrowShape)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.arrowPaint, that.arrowPaint)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.arrowStroke, that.arrowStroke)) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return int <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    int hashCode = super.hashCode() ;
    hashCode = hashCode * 37 + HashcodeUtil.hashCode(this.arrowShape) ;
    hashCode = hashCode * 37 + HashcodeUtil.hashCode(this.arrowPaint) ;
    hashCode = hashCode * 37 + HashcodeUtil.hashCode(this.arrowStroke) ;

    return hashCode ;
  }

  /**
   * <zh>ؿ¡Ķ.</zh>
   * <en>Creates and returns a copy of this object.</en>
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the Cloneable interface. Subclasses
   *               that override the clone method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object. </en>
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "TextAndPointerChartShape" ;
  }

  /**
   * <zh>л</zh>
   * <en>Provides serialization support</en>
   *
   * @param stream <zh></zh>
   * <en>the output stream</en>
   * @throws IOException  if there is an I/O error.
   */
  private void writeObject(ObjectOutputStream stream) throws IOException {
    stream.defaultWriteObject() ;

    SerialUtil.writeShape(stream, this.arrowShape) ;
    SerialUtil.writePaint(stream, this.arrowPaint) ;
    SerialUtil.writeStroke(stream, this.arrowStroke) ;
  }

  /**
   * Provides serialization support.
   *
   * @param stream  the input stream.
   *
   * @throws IOException  if there is an I/O error.
   * @throws ClassNotFoundException  if there is a classpath problem.
   */
  private void readObject(ObjectInputStream stream) throws IOException, ClassNotFoundException {
    stream.defaultReadObject() ;

    this.arrowShape = SerialUtil.readShape(stream) ;
    this.arrowPaint = SerialUtil.readPaint(stream) ;
    this.arrowStroke = SerialUtil.readStroke(stream) ;
  }
}
