package com.ijchart.xychart.ui.supplier ;

import java.awt.BasicStroke ;
import java.awt.Color ;
import java.awt.Paint ;
import java.awt.Polygon ;
import java.awt.Shape ;
import java.awt.Stroke ;
import java.awt.geom.Ellipse2D ;
import java.awt.geom.Rectangle2D ;
import java.io.IOException ;
import java.io.ObjectInputStream ;
import java.io.ObjectOutputStream ;

import com.ijchart.xychart.constant.ChartColor ;
import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.HashcodeUtil ;
import com.ijchart.xychart.util.SerialUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ӿ{@link DefaultDrawingSupplier}ȱʡʵ.</zh>
 * <en>The default implementation of interface {@link DefaultDrawingSupplier}.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class DefaultDrawingSupplier implements IDrawingSupplier {
  /**
   * <zh>ȱʡPaint.</zh>
   * <en>The default paint sequence.</en>
   */
  public static final Paint[] DEFAULT_PAINT_SEQUENCE = new Paint[] {Color.white} ;

  /**
   * <zh>ȱʡĻ߿Paint.</zh>
   * <en>The default border paint sequence. </en>
   */
  public static final Paint[] DEFAULT_BORDER_PAINT_SEQUENCE = new Paint[] {Color.lightGray} ;

  /**
   * <zh>ȱʡͼεPaint.</zh>
   * <en>The default fill paint sequence.</en>
   */
  public static final Paint[] DEFAULT_FILL_PAINT_SEQUENCE = ChartColor.PAINT_ARRAY ;

  /**
   * <zh>ȱʡStroke.</zh>
   * <en>The default stroke sequence. </en>
   */
  public static final Stroke[] DEFAULT_STROKE_SEQUENCE =
    new Stroke[] {new BasicStroke(1.0f, BasicStroke.CAP_SQUARE, BasicStroke.JOIN_BEVEL)} ;

  /**
   * <zh>ȱʡĻ߿Stroke.</zh>
   * <en>The default border stroke sequence. </en>
   */
  public static final Stroke[] DEFAULT_BORDER_STROKE_SEQUENCE =
    new Stroke[] {new BasicStroke(1.0f, BasicStroke.CAP_SQUARE, BasicStroke.JOIN_BEVEL)} ;

  /**
   * <zh>ȱʡͼ(Shape).</zh>
   * <en>The default shape sequence. </en>
   */
  public static final Shape[] DEFAULT_SHAPE_SEQUENCE = createStandardSeriesShapes() ;

  /**
   * <zh>ȱʡǷõı־.</zh>
   * <en>A default boolean flag controls whether or not can reset.</en>
   */
  public static final boolean DEFAULT_CAN_RESET = true ;

  /**
   * <zh>Paint.</zh>
   * <en>The paint sequence. </en>
   */
  private transient Paint[] paintSequence ;

  /**
   * <zh>ɵPaint.</zh>
   * <en>The old paint index.</en>
   */
  private int oldPaintIndex ;

  /**
   * <zh>Paint.</zh>
   * <en>The current paint index.</en>
   */
  private int paintIndex ;

  /**
   * <zh>߿Paint.</zh>
   * <en>The border paint sequence. </en>
   */
  private transient Paint[] borderPaintSequence ;

  /**
   * <zh>ɵĻ߿Paint.</zh>
   * <en>The old border paint index. </en>
   */
  private int oldBorderPaintIndex ;

  /**
   * <zh>߿Paint.</zh>
   * <en>The current border paint index. </en>
   */
  private int borderPaintIndex ;

  /**
   * <zh>ͼεPaint.</zh>
   * <en>The fill paint sequence. </en>
   */
  private transient Paint[] fillPaintSequence ;

  /**
   * <zh>ɵͼεPaint.</zh>
   * <en>The old fill paint index.</en>
   */
  private int oldFillPaintIndex ;

  /**
   * <zh>ͼεPaint.</zh>
   * <en>The current fill paint index.</en>
   */
  private int fillPaintIndex ;

  /**
   * <zh>Stroke.</zh>
   * <en>The stroke sequence. </en>
   */
  private transient Stroke[] strokeSequence ;

  /**
   * <zh>ɵStroke.</zh>
   * <en>The old stroke index. </en>
   */
  private int oldStrokeIndex ;

  /**
   * <zh>Stroke.</zh>
   * <en>The current stroke index. </en>
   */
  private int strokeIndex ;

  /**
   * <zh>߿Stroke.</zh>
   * <en>The border stroke sequence.</en>
   */
  private transient Stroke[] borderStrokeSequence ;

  /**
   * <zh>ɵ߿Stroke.</zh>
   * <en>The old border stroke index.</en>
   */
  private int oldBorderStrokeIndex ;

  /**
   * <zh>߿Stroke.</zh>
   * <en>The current border stroke index.</en>
   */
  private int borderStrokeIndex ;

  /**
   * <zh>ͼ.</zh>
   * <en>The shape sequence.</en>
   */
  private transient Shape[] shapeSequence ;

  /**
   * <zh>ɵͼ.</zh>
   * <en>The old shape index.</en>
   */
  private int oldShapeIndex ;

  /**
   * <zh>ͼ.</zh>
   * <en>The current shape index.</en>
   */
  private int shapeIndex ;

  /**
   * <zh>ɵѾùĸ.</zh>
   * <en>The old count has been used.</en>
   */
  private int oldPass ;

  /**
   * <zh>Ѿùĸ.</zh>
   * <en>The count has been used.</en>
   */
  private int pass ;

  /**
   * <zh>Ƿõı־.</zh>
   * <en>A boolean flag controls whether or not can reset.</en>
   */
  private boolean canReset ;

  /**
   * <zh>ȱʡĹ.</zh>
   * <en>Creates a new supplier, with default sequences for fill paint, border
   * paint, stroke and shapes.</en>
   */
  public DefaultDrawingSupplier() {
    this(DEFAULT_PAINT_SEQUENCE, DEFAULT_FILL_PAINT_SEQUENCE,
         DEFAULT_BORDER_PAINT_SEQUENCE,
         DEFAULT_STROKE_SEQUENCE,
         DEFAULT_BORDER_STROKE_SEQUENCE,
         DEFAULT_SHAPE_SEQUENCE) ;
  }

  /**
   * <zh>.</zh>
   * <en>Creates a new supplier (null not permitted).</en>
   *
   * @param paintSequence Paint[] <zh>Paint (Ϊnull).</zh>
   * <en>The paint sequence (null not permitted).</en>
   * @param borderPaintSequence Paint[] <zh>߿Paint (Ϊnull).</zh>
   * <en>The border paint sequence (null not permitted).</en>
   * @param strokeSequence Stroke[] <zh>Stroke (Ϊnull).</zh>
   * <en>The stroke sequence (null not permitted).</en>
   * @param borderStrokeSequence Stroke[] <zh>߿Stroke (Ϊnull).</zh>
   * <en>The border stroke sequence (null not permitted).</en>
   * @param shapeSequence Shape[] <zh>ͼ (Ϊnull).</zh>
   * <en>The shape sequence (null not permitted).</en>
   */
  public DefaultDrawingSupplier(Paint[] paintSequence, Paint[] borderPaintSequence,
                                Stroke[] strokeSequence, Stroke[] borderStrokeSequence,
                                Shape[] shapeSequence) {
    this(paintSequence, DEFAULT_FILL_PAINT_SEQUENCE, borderPaintSequence, strokeSequence,
         borderStrokeSequence, shapeSequence) ;
  }

  /**
   * <zh>.</zh>
   * <en>Creates a new supplier (null not permitted).</en>
   *
   * @param paintSequence Paint[] <zh>Paint (Ϊnull).</zh>
   * <en>The paint sequence (null not permitted).</en>
   * @param fillPaintSequence Paint[] <zh>ͼεPaint (Ϊnull).</zh>
   * <en>The fill paint sequence (null not permitted).</en>
   * @param borderPaintSequence Paint[] <zh>߿Paint (Ϊnull).</zh>
   * <en>The border paint sequence (null not permitted).</en>
   * @param strokeSequence Stroke[] <zh>Stroke (Ϊnull).</zh>
   * <en>The stroke sequence (null not permitted).</en>
   * @param borderStrokeSequence Stroke[] <zh>߿Stroke (Ϊnull).</zh>
   * <en>The border stroke sequence (null not permitted).</en>
   * @param shapeSequence Shape[] <zh>ͼ (Ϊnull).</zh>
   * <en>The shape sequence (null not permitted).</en>
   */
  public DefaultDrawingSupplier(Paint[] paintSequence, Paint[] fillPaintSequence,
                                Paint[] borderPaintSequence, Stroke[] strokeSequence,
                                Stroke[] borderStrokeSequence, Shape[] shapeSequence) {
    if (paintSequence == null) {
      throw new IllegalArgumentException("Null 'paintSequence' argument.") ;
    }
    if (fillPaintSequence == null) {
      throw new IllegalArgumentException("Null 'fillPaintSequence' argument.") ;
    }
    if (borderPaintSequence == null) {
      throw new IllegalArgumentException("Null 'borderPaintSequence' argument.") ;
    }
    if (strokeSequence == null) {
      throw new IllegalArgumentException("Null 'strokeSequence' argument.") ;
    }
    if (borderStrokeSequence == null) {
      throw new IllegalArgumentException("Null 'borderStrokeSequence' argument.") ;
    }
    if (shapeSequence == null) {
      throw new IllegalArgumentException("Null 'shapeSequence' argument.") ;
    }

    this.paintSequence = paintSequence ;
    this.fillPaintSequence = fillPaintSequence ;
    this.borderPaintSequence = borderPaintSequence ;
    this.strokeSequence = strokeSequence ;
    this.borderStrokeSequence = borderStrokeSequence ;
    this.shapeSequence = shapeSequence ;
    this.pass = 0 ;
    this.canReset = DEFAULT_CAN_RESET ;
  }

  /**
   * <zh>һPaint.</zh>
   * <en>Returns the next paint in a sequence maintained by the supplier.</en>
   *
   * @return Paint <zh>Paint (Ϊnull).</zh>
   * <en>The paint (never null).</en>
   */
  public Paint getNextPaint() {
    Paint paint = this.paintSequence[(this.pass + this.paintIndex) % this.paintSequence.length] ;
    this.paintIndex++ ;

    return paint ;
  }

  /**
   * <zh>һ߿Paint.</zh>
   * <en>Returns the next border paint in a sequence maintained by the supplier.</en>
   *
   * @return Paint <zh>Paint (Ϊnull).</zh>
   * <en>The paint (never null).</en>
   */
  public Paint getNextBorderPaint() {
    Paint paint = this.borderPaintSequence[(this.pass + this.borderPaintIndex)
                  % this.borderPaintSequence.length] ;
    this.borderPaintIndex++ ;

    return paint ;
  }

  /**
   * <zh>һͼεPaint.</zh>
   * <en>Returns the next fill paint in a sequence maintained by the supplier.</en>
   *
   * @return Paint <zh>Paint (Ϊnull).</zh>
   * <en>The paint (never null).</en>
   */

  public Paint getNextFillPaint() {
    Paint paint = this.fillPaintSequence[(this.pass + this.fillPaintIndex)
                  % this.fillPaintSequence.length] ;
    this.fillPaintIndex++ ;

    return paint ;
  }

  /**
   * <zh>һStroke.</zh>
   * <en>Returns the next Stroke object in a sequence maintained by the supplier.</en>
   *
   * @return Stroke <zh>Stroke (Ϊnull).</zh>
   * <en>The Stroke (never null).</en>
   */
  public Stroke getNextStroke() {
    Stroke stroke = this.strokeSequence[(this.pass + this.strokeIndex) % this.strokeSequence.length] ;
    this.strokeIndex++ ;

    return stroke ;
  }

  /**
   * <zh>һƱ߿Stroke.</zh>
   * <en>Returns the next Stroke object in a sequence maintained by the supplier.</en>
   *
   * @return Stroke <zh>Stroke (Ϊnull).</zh>
   * <en>The Stroke (never null).</en>
   */
  public Stroke getNextBorderStroke() {
    Stroke stroke = this.borderStrokeSequence[(this.pass + this.borderStrokeIndex++)
                    % this.borderStrokeSequence.length] ;
    this.borderStrokeIndex++ ;

    return stroke ;
  }

  /**
   * <zh>һShape.</zh>
   * <en>Returns the next Shape object in a sequence maintained by the supplier.</en>
   *
   * @return Shape <zh>Shape (Ϊnull).</zh>
   * <en>The shape (never null).</en>
   */
  public Shape getNextShape() {
    Shape shape = this.shapeSequence[(this.pass + this.shapeIndex) % this.shapeSequence.length] ;
    this.shapeIndex++ ;

    return shape ;
  }

  /**
   * <zh>һϵͼε,Щͼοͼϵʾ.</zh>
   * <en>Creates an array of standard shapes to display for the items in series on charts.</en>
   *
   * @return Shape[] <zh>ͼ (Ϊnull).</zh>
   * <en>The shape array (never null).</en>
   */
  public static Shape[] createStandardSeriesShapes() {
    Shape[] result = new Shape[10] ;

    final double size = 6.0 ;
    final double delta = 3.0 ;
    int[] xpoints = null ;
    int[] ypoints = null ;

    // square
    result[0] = new Rectangle2D.Double( -delta, -delta, size, size) ;
    // circle
    result[1] = new Ellipse2D.Double( -delta, -delta, size, size) ;

    // up-pointing triangle
    xpoints = intArray(0.0, delta, -delta) ;
    ypoints = intArray( -delta, delta, delta) ;
    result[2] = new Polygon(xpoints, ypoints, 3) ;

    // diamond
    xpoints = intArray(0.0, delta, 0.0, -delta) ;
    ypoints = intArray( -delta, 0.0, delta, 0.0) ;
    result[3] = new Polygon(xpoints, ypoints, 4) ;

    // horizontal rectangle
    result[4] = new Rectangle2D.Double( -delta, -1.5, size, 3) ;

    // down-pointing triangle
    xpoints = intArray( -delta, +delta, 0.0) ;
    ypoints = intArray( -delta, -delta, delta) ;
    result[5] = new Polygon(xpoints, ypoints, 3) ;

    // horizontal ellipse
    result[6] = new Ellipse2D.Double( -delta, -1.5, size, 3) ;

    // right-pointing triangle
    xpoints = intArray( -delta, delta, -delta) ;
    ypoints = intArray( -delta, 0.0, delta) ;
    result[7] = new Polygon(xpoints, ypoints, 3) ;

    // vertical rectangle
    result[8] = new Rectangle2D.Double( -1.5, -delta, 3, size) ;

    // left-pointing triangle
    xpoints = intArray( -delta, delta, delta) ;
    ypoints = intArray(0.0, -delta, +delta) ;
    result[9] = new Polygon(xpoints, ypoints, 3) ;

    return result ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Tests this object for equality with another object.</en>
   *
   * @param obj  the object (null permitted).
   *
   * @return A boolean.
   */
  public boolean equals(Object obj) {
    if (obj == this) {
      return true ;
    }

    if (!(obj instanceof DefaultDrawingSupplier)) {
      return false ;
    }

    DefaultDrawingSupplier that = (DefaultDrawingSupplier) obj ;

    if (!EqualsUtil.equals(this.paintSequence, that.paintSequence)) {
      return false ;
    }
    if (this.paintIndex != that.paintIndex) {
      return false ;
    }
    if (!EqualsUtil.equals(this.borderPaintSequence, that.borderPaintSequence)) {
      return false ;
    }
    if (this.borderPaintIndex != that.borderPaintIndex) {
      return false ;
    }
    if (!EqualsUtil.equals(this.strokeSequence, that.strokeSequence)) {
      return false ;
    }
    if (this.strokeIndex != that.strokeIndex) {
      return false ;
    }
    if (!EqualsUtil.equals(this.borderStrokeSequence, that.borderStrokeSequence)) {
      return false ;
    }
    if (this.borderStrokeIndex != that.borderStrokeIndex) {
      return false ;
    }
    if (!EqualsUtil.equals(this.shapeSequence, that.shapeSequence)) {
      return false ;
    }
    if (this.shapeIndex != that.shapeIndex) {
      return false ;
    }
    if (this.pass != that.pass) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object</en>
   *
   * @return int <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    int result = HashcodeUtil.hashCode(this.paintSequence) ;
    result = 29 * result + this.paintIndex ;
    result = 29 * result + HashcodeUtil.hashCode(this.borderPaintSequence) ;
    result = 29 * result + this.borderPaintIndex ;
    result = 29 * result + HashcodeUtil.hashCode(this.fillPaintSequence) ;
    result = 29 * result + this.fillPaintIndex ;
    result = 29 * result + HashcodeUtil.hashCode(this.strokeSequence) ;
    result = 29 * result + this.strokeIndex ;
    result = 29 * result + HashcodeUtil.hashCode(this.borderStrokeSequence) ;
    result = 29 * result + this.borderStrokeIndex ;
    result = 29 * result + HashcodeUtil.hashCode(this.shapeSequence) ;
    result = 29 * result + this.shapeIndex ;
    result = 29 * result + this.pass ;

    return result ;
  }

  /**
   * <zh>л.</zh>
   * <en>Handles serialization.</en>
   *
   * @param stream  the output stream.
   *
   * @throws IOException if there is an I/O problem.
   */
  private void writeObject(ObjectOutputStream stream) throws IOException {
    stream.defaultWriteObject() ;

    int paintCount = this.paintSequence.length ;
    stream.writeInt(paintCount) ;
    for (int i = 0 ; i < paintCount ; i++) {
      SerialUtil.writePaint(stream, this.paintSequence[i]) ;
    }

    int borderPaintCount = this.borderPaintSequence.length ;
    stream.writeInt(borderPaintCount) ;
    for (int i = 0 ; i < borderPaintCount ; i++) {
      SerialUtil.writePaint(stream, this.borderPaintSequence[i]) ;
    }

    int strokeCount = this.strokeSequence.length ;
    stream.writeInt(strokeCount) ;
    for (int i = 0 ; i < strokeCount ; i++) {
      SerialUtil.writeStroke(stream, this.strokeSequence[i]) ;
    }

    int borderStrokeCount = this.borderStrokeSequence.length ;
    stream.writeInt(borderStrokeCount) ;
    for (int i = 0 ; i < borderStrokeCount ; i++) {
      SerialUtil.writeStroke(stream, this.borderStrokeSequence[i]) ;
    }

    int shapeCount = this.shapeSequence.length ;
    stream.writeInt(shapeCount) ;
    for (int i = 0 ; i < shapeCount ; i++) {
      SerialUtil.writeShape(stream, this.shapeSequence[i]) ;
    }

  }

  /**
   * <zh>л.</zh>
   * <en>Restores a serialized object.</en>
   *
   * @param stream  the input stream.
   *
   * @throws IOException if there is an I/O problem.
   * @throws ClassNotFoundException if there is a problem loading a class.
   */
  private void readObject(ObjectInputStream stream) throws IOException, ClassNotFoundException {
    stream.defaultReadObject() ;

    int paintCount = stream.readInt() ;
    this.paintSequence = new Paint[paintCount] ;
    for (int i = 0 ; i < paintCount ; i++) {
      this.paintSequence[i] = SerialUtil.readPaint(stream) ;
    }

    int borderPaintCount = stream.readInt() ;
    this.borderPaintSequence = new Paint[borderPaintCount] ;
    for (int i = 0 ; i < borderPaintCount ; i++) {
      this.borderPaintSequence[i] = SerialUtil.readPaint(stream) ;
    }

    int strokeCount = stream.readInt() ;
    this.strokeSequence = new Stroke[strokeCount] ;
    for (int i = 0 ; i < strokeCount ; i++) {
      this.strokeSequence[i] = SerialUtil.readStroke(stream) ;
    }

    int borderStrokeCount = stream.readInt() ;
    this.borderStrokeSequence = new Stroke[borderStrokeCount] ;
    for (int i = 0 ; i < borderStrokeCount ; i++) {
      this.borderStrokeSequence[i] = SerialUtil.readStroke(stream) ;
    }

    int shapeCount = stream.readInt() ;
    this.shapeSequence = new Shape[shapeCount] ;
    for (int i = 0 ; i < shapeCount ; i++) {
      this.shapeSequence[i] = SerialUtil.readShape(stream) ;
    }

  }

  /**
   * Helper method to avoid lots of explicit casts in getShape().  Returns
   * an array containing the provided doubles cast to ints.
   *
   * @param a  x
   * @param b  y
   * @param c  z
   *
   * @return int[3] with converted params.
   */
  private static int[] intArray(double a, double b, double c) {
    return new int[] {(int) a, (int) b, (int) c} ;
  }

  /**
   * Helper method to avoid lots of explicit casts in getShape().  Returns
   * an array containing the provided doubles cast to ints.
   *
   * @param a  x
   * @param b  y
   * @param c  z
   * @param d  t
   *
   * @return int[4] with converted params.
   */
  private static int[] intArray(double a, double b, double c, double d) {
    return new int[] {(int) a, (int) b, (int) c, (int) d} ;
  }

  /**
   * <zh>Ƿñൽʼ״̬.</zh>
   * <en>Whether or not can reset.</en>
   *
   * @return boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  public boolean isCanReset() {
    return this.canReset ;
  }

  /**
   * <zh>Ƿñൽʼ״̬.</zh>
   * <en>Sets whether or not can reset.</en>
   *
   * @param flag boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  public void setCanReset(boolean flag) {
    this.canReset = flag ;
  }

  /**
   * <zh>ñൽʼ״̬.</zh>
   * <en>Reset this class to first initiate state.</en>
   */
  public void reset() {
    if (this.canReset) {
      this.paintIndex = 0 ;
      this.borderPaintIndex = 0 ;
      this.fillPaintIndex = 0 ;
      this.strokeIndex = 0 ;
      this.borderStrokeIndex = 0 ;
      this.shapeIndex = 0 ;
      this.pass = 0 ;

      this.oldPaintIndex = 0 ;
      this.oldBorderPaintIndex = 0 ;
      this.oldFillPaintIndex = 0 ;
      this.oldStrokeIndex = 0 ;
      this.oldBorderStrokeIndex = 0 ;
      this.oldShapeIndex = 0 ;
      this.oldPass = 0 ;
    }
  }

  /**
   * <zh>浱ǰ״̬.</zh>
   * <en>Save current state.</en>
   */
  public void saveState() {
    this.oldPaintIndex = this.paintIndex ;
    this.oldBorderPaintIndex = this.borderPaintIndex ;
    this.oldFillPaintIndex = this.fillPaintIndex ;
    this.oldStrokeIndex = this.strokeIndex ;
    this.oldBorderStrokeIndex = this.borderStrokeIndex ;
    this.oldShapeIndex = this.shapeIndex ;
    this.oldPass = this.pass ;
  }

  /**
   * <zh>ָǰ״̬Ϊɵ״̬.</zh>
   * <en>Restore state to old.</en>
   */
  public void restoreState() {
    this.paintIndex = this.oldPaintIndex ;
    this.borderPaintIndex = this.oldBorderPaintIndex ;
    this.fillPaintIndex = this.oldFillPaintIndex ;
    this.strokeIndex = this.oldStrokeIndex ;
    this.borderStrokeIndex = this.oldBorderStrokeIndex ;
    this.shapeIndex = this.oldShapeIndex ;
    this.pass = this.oldPass ;
  }

  /**
   * <zh>¡.</zh>
   * <en>Returns a clone.</en>
   *
   * @return A clone.
   *
   * @throws CloneNotSupportedException if a component of the supplier does
   *                                    not support cloning.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }
}
