package com.ijchart.xychart.util ;

import java.awt.Shape ;
import java.awt.geom.AffineTransform ;
import java.awt.geom.GeneralPath ;
import java.awt.geom.PathIterator ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ͼλ(Graphics2D)صĹ.</zh>
 * <en>Nice static helpers for working with Graphics2D.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public abstract class GraphicsUtil {

  /**
   * <zh>ͼεÿƫ(-translateX,-translateY).</zh>
   * <en>Move the shape's coordinate to (-translateX, -translateY).</en>
   *
   * @param shape Shape <zh>Ҫƶͼ.</zh>
   * <en>The shape which need move coordinate.</en>
   * @param translateX <zh>µԭxֵ.</zh>
   * <en>The new x-coordinate of origin.</en>
   * @param translateY <zh>µԭyֵ.</zh>
   * <en>The new y-coordinate of origin.</en>
   *
   * @return Shape <zh>µͼ.</zh>
   * <en>The new shape.</en>
   */
  public static Shape translateShapeByPath(Shape shape, float translateX, float translateY) {
    PathIterator pathIterator = shape.getPathIterator(null) ;
    GeneralPath newShape = new GeneralPath() ;
    float[] coordinates = new float[6] ;

    while (!pathIterator.isDone()) {
      int type = pathIterator.currentSegment(coordinates) ;

      switch (type) {
      case PathIterator.SEG_MOVETO:
        newShape.moveTo(coordinates[0] - translateX, coordinates[1] - translateY) ;
        break ;
      case PathIterator.SEG_LINETO:
        newShape.lineTo(coordinates[0] - translateX, coordinates[1] - translateY) ;
        break ;
      case PathIterator.SEG_QUADTO:
        newShape.quadTo(coordinates[0] - translateX, coordinates[1] - translateY,
                        coordinates[2] - translateX, coordinates[3] - translateY) ;
        break ;
      case PathIterator.SEG_CUBICTO:
        newShape.curveTo(coordinates[0] - translateX, coordinates[1] - translateY,
                         coordinates[2] - translateX, coordinates[3] - translateY,
                         coordinates[4] - translateX, coordinates[5] - translateY) ;
        break ;
      case PathIterator.SEG_CLOSE:
        newShape.closePath() ;
        break ;
      default:
        break ;
      }

      pathIterator.next() ;
    }

    return newShape ;
  }

  /**
   * <zh>ͼεÿƫ(-translateX,-translateY).</zh>
   * <en>Move the shape's coordinate to (-translateX, -translateY).</en>
   *
   * @param shape Shape <zh>Ҫƶͼ.</zh>
   * <en>The shape which need move coordinate.</en>
   * @param translateX <zh>µԭxֵ.</zh>
   * <en>The new x-coordinate of origin.</en>
   * @param translateY <zh>µԭyֵ.</zh>
   * <en>The new y-coordinate of origin.</en>
   *
   * @return Shape <zh>µͼ.</zh>
   * <en>The new shape.</en>
   */
  public static Shape translateShapeByTransform(Shape shape, float translateX, float translateY) {
    AffineTransform transform = new AffineTransform() ;
    transform.translate(translateX, translateY) ;

    return transform.createTransformedShape(shape) ;
  }
}
