package com.ijchart.xychart.web ;

import java.io.BufferedOutputStream ;
import java.io.ByteArrayOutputStream ;
import java.io.IOException ;
import java.text.SimpleDateFormat ;
import java.util.Calendar ;
import java.util.Locale ;
import java.util.TimeZone ;

import javax.servlet.ServletException ;
import javax.servlet.http.HttpServlet ;
import javax.servlet.http.HttpServletRequest ;
import javax.servlet.http.HttpServletResponse ;
import javax.servlet.http.HttpSession ;

import com.ijchart.xychart.client.ChartPanel ;
import com.ijchart.xychart.client.IJChart ;
import com.ijchart.xychart.constant.ImageType ;
import com.ijchart.xychart.util.StringUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ҳʾͼƬ,ʹǰӦwebļ(web.xml):<br></zh>
 * <en>Servlet used for streaming charts to the client browser from the temporary
 * directory.You need to add this servlet and mapping to your deployment
 * descriptor (web.xml) in order to get it to work.The syntax is as follows:<br></en>
 * &lt;servlet&gt;<br>
 * &nbsp;&nbsp;&nbsp;&nbsp;&lt;servlet-name&gt;DisplayChartServlet&lt;/servlet-name&gt;<br>
 * &nbsp;&nbsp;&nbsp;&nbsp;&lt;servlet-class&gt;com.ijchart.xychart.DisplayChartServlet&lt;/servlet-class&gt;<br>
 * &lt;/servlet&gt;<br>
 * &lt;servlet-mapping&gt;<br>
 * &nbsp;&nbsp;&nbsp;&nbsp;&lt;servlet-name&gt;DisplayChartServlet&lt;/servlet-name&gt;<br>
 * &nbsp;&nbsp;&nbsp;&nbsp;&lt;url-pattern&gt;/DisplayChartServlet&lt;/url-pattern&gt;<br>
 * &lt;/servlet-mapping&gt;<br>
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class DisplayChartServlet extends HttpServlet {

  /**
   * Service method.
   *
   * @param request  the request.
   * @param response  the response.
   *
   * @throws ServletException
   * @throws IOException
   */
  public void service(HttpServletRequest request, HttpServletResponse response) throws
    ServletException, IOException {
    String chartId = request.getParameter(WebChart.PARAM_CHART_ID) ;
    HttpSession session = request.getSession() ;
    IJChart chart = (IJChart) session.getAttribute(chartId) ;
    session.removeAttribute(chartId) ;
    if (chart != null) {
      String imageType = request.getParameter(WebChart.PARAM_CHART_IMAGE_TYPE) ;
      if (StringUtil.isEmpty(imageType)) {
        imageType = WebChart.DEFAULT_IMAGE_TYPE.toString() ;
      }

      String imageWidth = request.getParameter(WebChart.PARAM_CHART_IMAGE_WIDTH) ;
      if (StringUtil.isEmpty(imageWidth)) {
        imageWidth = String.valueOf(ChartPanel.DEFAULT_MAXIMUM_DRAW_WIDTH) ;
      }

      String imageHeight = request.getParameter(WebChart.PARAM_CHART_IMAGE_HEIGHT) ;
      if (StringUtil.isEmpty(imageHeight)) {
        imageHeight = String.valueOf(ChartPanel.DEFAULT_MAXIMUM_DRAW_HEIGHT) ;
      }

      int imgWidth = Integer.parseInt(imageWidth) ;
      int imgHeight = Integer.parseInt(imageHeight) ;

      String mimeType = null ;
      ByteArrayOutputStream outStream = new ByteArrayOutputStream() ;
      if (imageType.equals(ImageType.JPEG.toString())) {
        chart.saveChartAsJPEG(outStream, imgWidth, imgHeight) ;
        mimeType = "image/jpeg" ;
      } else if (imageType.equals(ImageType.PNG.toString())) {
        chart.saveChartAsPNG(outStream, imgWidth, imgHeight) ;
        mimeType = "image/png" ;
      }

      //  Set HTTP headers
      if (mimeType != null) {
        response.setHeader("Content-Type", mimeType) ;
      }
      response.setHeader("Content-Length", String.valueOf(outStream.size())) ;
      SimpleDateFormat sdf = new SimpleDateFormat("EEE, dd MMM yyyy HH:mm:ss z", Locale.ENGLISH) ;
      sdf.setTimeZone(TimeZone.getTimeZone("GMT")) ;
      response.setHeader("Last-Modified", sdf.format(Calendar.getInstance().getTime())) ;

      BufferedOutputStream bos = new BufferedOutputStream(response.getOutputStream()) ;
      bos.write(outStream.toByteArray()) ;
      bos.flush() ;
      outStream.close() ;
      bos.close() ;
    } else {
      throw new ServletException("No chart found.") ;
    }

  }

}
